//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "FriendList.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

#define EMFRIENDS_MET_FILENAME	_T("emfriends.met")

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CFriendList::CFriendList(void)
{
	LoadList();
	m_nLastSaved = ::GetTickCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CFriendList::~CFriendList(void)
{
	SaveList();
	for (POSITION pos = m_listFriends.GetHeadPosition();pos != 0;m_listFriends.GetNext(pos))
		delete m_listFriends.GetAt(pos);
	m_listFriends.RemoveAll();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadList() loads the users friends list from the "emfriends.met" file.
bool CFriendList::LoadList()
{
	CFriend	* pFriend = NULL;
	CSafeFile	file;

	try
	{
		CString	strFileName = CString(g_eMuleApp.m_pGlobPrefs->GetConfigDir()) + CString(EMFRIENDS_MET_FILENAME);
	//CString strFileName = CString(g_eMuleApp.m_pGlobPrefs->GetAppDir()) + CString("emfriends.met");

		if (!file.Open(strFileName.GetBuffer(), CFile::modeRead | CFile::osSequentialScan))
			return false;

		uint8	header;

		file.Read(&header, 1);
		if (header != MET_HEADER)
		{
			file.Close();
			return false;
		}
		uint32	nFriendsNumber;

		file.Read(&nFriendsNumber, 4);
		for (uint32 i = 0; i < nFriendsNumber; i++)
		{
			pFriend = new CFriend();
			pFriend->LoadFromFile(file);
			m_listFriends.AddTail(pFriend);
		}
		file.Close();
		return true;
	}
	catch (CFileException * error)
	{
		OUTPUT_DEBUG_TRACE();
		if (error->m_cause == CFileException::endOfFile)
			AddLogLine(true, IDS_ERR_EMFRIENDSINVALID);
		else
		{
			AddLogLine(true, IDS_ERR_READEMFRIENDS, GetErrorMessage(error));
		}
		error->Delete();
		return false;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendList::SaveList()
{
	m_nLastSaved = ::GetTickCount();
	CFile file;
	CString strFileName = CString(g_eMuleApp.m_pGlobPrefs->GetConfigDir()) + CString(EMFRIENDS_MET_FILENAME);
//CString strFileName = CString(g_eMuleApp.m_pGlobPrefs->GetAppDir()) + CString("emfriends.met");
	if (!file.Open(strFileName.GetBuffer(), CFile::modeCreate | CFile::modeWrite))
		return;
	uint8 header = MET_HEADER;
	file.Write(&header, 1);
	uint32 nRecordsNumber = m_listFriends.GetCount();
	file.Write(&nRecordsNumber, 4);
	for (POSITION pos = m_listFriends.GetHeadPosition();pos != 0;m_listFriends.GetNext(pos))
	{
		m_listFriends.GetAt(pos)->WriteToFile(file);
	}
	file.Close();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CFriend* CFriendList::LinkFriend(uchar* abyUserHash, uint32 m_dwIp, uint16 m_uPort)
{
	CFriend * backup_friend = NULL;
	for (POSITION pos = m_listFriends.GetHeadPosition();pos != 0;m_listFriends.GetNext(pos))
	{
		CFriend * cur_friend = m_listFriends.GetAt(pos);
		if (cur_friend->m_dwLastUsedIP == m_dwIp && cur_friend->m_nLastUsedPort == m_uPort)
			backup_friend = cur_friend;
		if (cur_friend->m_dwHasHash && !md4cmp(cur_friend->m_abyUserhash, abyUserHash))
		{
			cur_friend->m_dwLastSeen = time(NULL);
			if (cur_friend->m_pLinkedClient)
			{
				cur_friend->m_pLinkedClient->m_pFriend = NULL;
				cur_friend->m_pLinkedClient = NULL;
			}
			return cur_friend;
		}
	}
	return backup_friend;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendList::RefreshFriend(CFriend *pFriend)
{
	if (m_pctlFriendList != NULL && ::IsWindow(m_pctlFriendList->m_hWnd))
		m_pctlFriendList->RefreshFriend(pFriend);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFriendList::ShowFriends()
{
	if (!m_pctlFriendList)
	{
		ASSERT (false);
		return;
	}
	m_pctlFriendList->DeleteAllItems();
	for (POSITION pos = m_listFriends.GetHeadPosition();pos != 0;m_listFriends.GetNext(pos))
	{
		CFriend * cur_friend = m_listFriends.GetAt(pos);
		m_pctlFriendList->AddFriend(cur_friend);
	}
}

//Added this to work with the IRC.. Probably a better way to do it.. But wanted this in the release..
bool CFriendList::AddFriend(uchar t_m_abyUserhash[16], uint32 tm_dwLastSeen, uint32 tm_dwLastUsedIP, uint32 tm_nLastUsedPort, uint32 tm_dwLastChatted, CString tm_strName, uint32 tm_dwHasHash)
{
	// client must have an IP (HighID) or a hash
	if (tm_dwLastUsedIP<16777216 && tm_dwHasHash==0)
		return false;
	if ( tm_dwLastUsedIP && IsAlreadyFriend(tm_dwLastUsedIP, tm_nLastUsedPort))
		return false;
	CFriend* Record = new CFriend(t_m_abyUserhash, tm_dwLastSeen, tm_dwLastUsedIP, tm_nLastUsedPort, tm_dwLastChatted, tm_strName, tm_dwHasHash);
	m_listFriends.AddTail(Record);
	ShowFriends();
	SaveList();
	return true;
}

//	Added for the friends function in the IRC..
bool CFriendList::IsAlreadyFriend(uint32 tm_dwLastUsedIP, uint32 tm_nLastUsedPort)
{
	for (POSITION pos = m_listFriends.GetHeadPosition();pos != 0;m_listFriends.GetNext(pos))
	{
		CFriend * cur_friend = m_listFriends.GetAt(pos);
		if (cur_friend->m_dwLastUsedIP == tm_dwLastUsedIP && cur_friend->m_nLastUsedPort == tm_nLastUsedPort)
		{
			return true;
		}
	}
	return false;
}

bool CFriendList::AddFriend(CUpDownClient* pClient)
{
	if (pClient->IsFriend())
		return false;
	// client must have an IP (HighID) or a hash
	if (pClient->HasLowID() && !pClient->HasValidHash())
		return false;
	CFriend* NewFriend = new CFriend(pClient);
	pClient->m_pFriend = NewFriend;
	m_listFriends.AddTail(NewFriend);
	if (m_pctlFriendList)
		m_pctlFriendList->AddFriend(NewFriend);
	SaveList();
//	MOREVIT - I'm betting we don't need the throttle if we use the message queue
	g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlQueueList.PostMessage(WM_QL_REFRESHCLIENT, 0, reinterpret_cast<LPARAM>(pClient));
	g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlUploadList.PostMessage(WM_UL_REFRESHCLIENT, 0, reinterpret_cast<LPARAM>(pClient));
	g_eMuleApp.m_pDownloadList->UpdateSource(pClient);
	return true;
}

void CFriendList::RemoveFriend(CFriend* todel)
{
	POSITION pos = m_listFriends.Find(todel);
	if (!pos)
	{
		ASSERT (false);
		return;
	}
	if (todel->m_pLinkedClient)
	{
		todel->m_pLinkedClient->m_pFriend = NULL;
		todel->m_pLinkedClient = NULL;
	}
	if (m_pctlFriendList)
		m_pctlFriendList->RemoveFriend(todel);
	m_listFriends.RemoveAt(pos);
	delete todel;
	SaveList();
}

void CFriendList::RemoveFriend(CUpDownClient* pClient)
{
	for (POSITION pos = m_listFriends.GetHeadPosition();pos != 0;m_listFriends.GetNext(pos))
	{
		CFriend * cur_friend = m_listFriends.GetAt(pos);
		if (cur_friend->m_dwLastUsedIP == pClient->GetIP() && cur_friend->m_nLastUsedPort == pClient->GetUserPort())
		{
			RemoveFriend(cur_friend);
			g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlQueueList.PostMessage(WM_QL_REFRESHCLIENT, 0, reinterpret_cast<LPARAM>(pClient));
			g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlUploadList.PostMessage(WM_UL_REFRESHCLIENT, 0, reinterpret_cast<LPARAM>(pClient));
			g_eMuleApp.m_pDownloadList->UpdateSource(pClient);
			break;
		}
	}
}

void CFriendList::Process()
{
	if (::GetTickCount() - m_nLastSaved > 300000)
		this->SaveList();
}