//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "preferences.h"
#include <time.h>
#include "opcodes.h"
#include "otherfunctions.h"
#include "ini2.h"
#include "stdlib.h"
#include "stdio.h"
#include <locale.h>
#include "resource.h"
#include "emule.h"
#include <io.h>

// For SecuredVars
const DWORD CCriticalSection_INL2::s_dwSafeValue = 0x159D37BF;


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ELEMENT_COUNT(X) (sizeof(X) / sizeof(X[0]))

bool CPreferences::bWinVerAlreadyDetected = false;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CPreferences::CPreferences()
{
//	Seed the random number generator
	srand((uint32)time(0)); // we need random numbers sometimes
	m_bIsNTBased = false;

//
//	Create the legacy preferences structs
//
	m_prefs = new Preferences_Struct_DONTUSE;
//	memset2(m_prefs,0,sizeof(Preferences_Struct));
	m_prefsExt=new Preferences_Ext_Struct;
	memset2(m_prefsExt,0,sizeof(Preferences_Ext_Struct));

//	Get application start directory
	TCHAR path[MAX_PATH];

	::GetModuleFileName(NULL, path, MAX_PATH);
	
	TCHAR drive[_MAX_DRIVE];
	TCHAR dir[_MAX_DIR];

	_tsplitpath(path, drive, dir, NULL, NULL); 
	
	m_strAppDir = CString(drive) + dir;	
	m_strConfigDir = m_strAppDir + _T("Config\\");
//	m_strWebServerDir = m_strAppDir + _T("WebServer\\");
//	m_strLangDir = m_strAppDir + _T("lang\\");

	::CreateDirectory(GetConfigDir(),0);

	// lets move config-files in the appdir to the configdir
	if ( PathFileExists(m_strAppDir+"adresses.dat")) MoveFile(m_strAppDir+"adresses.dat",m_strConfigDir+"adresses.dat");
//	Fix spelling
	if ( PathFileExists(m_strConfigDir+"adresses.dat")) MoveFile(m_strConfigDir+"adresses.dat",m_strConfigDir+"addresses.dat");

	if ( PathFileExists(m_strAppDir+"cryptkey.dat")) MoveFile(m_strAppDir+"cryptkey.dat",m_strConfigDir+"cryptkey.dat");
	if ( PathFileExists(m_strAppDir+"gui.dat")) MoveFile(m_strAppDir+"gui.dat",m_strConfigDir+"gui.dat");
	if ( PathFileExists(m_strAppDir+"IPFilter.dat")) MoveFile(m_strAppDir+"IPFilter.dat",m_strConfigDir+"IPFilter.dat");
	if ( PathFileExists(m_strAppDir+"onlinesig.dat")) MoveFile(m_strAppDir+"onlinesig.dat",m_strConfigDir+"onlinesig.dat");
	if ( PathFileExists(m_strAppDir+"partperm.dat")) MoveFile(m_strAppDir+"partperm.dat",m_strConfigDir+"partperm.dat");
	if ( PathFileExists(m_strAppDir+"preferences.dat")) MoveFile(m_strAppDir+"preferences.dat",m_strConfigDir+"preferences.dat");
	if ( PathFileExists(m_strAppDir+"shareddir.dat")) MoveFile(m_strAppDir+"shareddir.dat",m_strConfigDir+"shareddir.dat");
	if ( PathFileExists(m_strAppDir+"staticservers.dat")) MoveFile(m_strAppDir+"staticservers.dat",m_strConfigDir+"staticservers.dat");
	if ( PathFileExists(m_strAppDir+"tempdir.dat")) MoveFile(m_strAppDir+"tempdir.dat",m_strConfigDir+"tempdir.dat");
	if ( PathFileExists(m_strAppDir+"traffic.dat")) MoveFile(m_strAppDir+"traffic.dat",m_strConfigDir+"traffic.dat");
	if ( PathFileExists(m_strAppDir+"userhash.dat")) MoveFile(m_strAppDir+"userhash.dat",m_strConfigDir+"userhash.dat");
	if ( PathFileExists(m_strAppDir+"webservices.dat")) MoveFile(m_strAppDir+"webservices.dat",m_strConfigDir+"webservices.dat");
	if ( PathFileExists(m_strAppDir+"category.ini")) MoveFile(m_strAppDir+"category.ini",m_strConfigDir+"category.ini");
	if ( PathFileExists(m_strAppDir+"fileinfo.ini")) MoveFile(m_strAppDir+"fileinfo.ini",m_strConfigDir+"fileinfo.ini");
	if ( PathFileExists(m_strAppDir+"preferences.ini")) MoveFile(m_strAppDir+"preferences.ini",m_strConfigDir+"preferences.ini");
	if ( PathFileExists(m_strAppDir+"statbkup.ini")) MoveFile(m_strAppDir+"statbkup.ini",m_strConfigDir+"statbkup.ini");
	if ( PathFileExists(m_strAppDir+"statbkuptmp.ini")) MoveFile(m_strAppDir+"statbkuptmp.ini",m_strConfigDir+"statbkuptmp.ini");
	if ( PathFileExists(m_strAppDir+"clients.met")) MoveFile(m_strAppDir+"clients.met",m_strConfigDir+"clients.met");
	if ( PathFileExists(m_strAppDir+"emfriends.met")) MoveFile(m_strAppDir+"emfriends.met",m_strConfigDir+"emfriends.met");
	if ( PathFileExists(m_strAppDir+"known.met")) MoveFile(m_strAppDir+"known.met",m_strConfigDir+"known.met");
	if ( PathFileExists(m_strAppDir+"server.met")) MoveFile(m_strAppDir+"server.met",m_strConfigDir+"server.met");

//	Generate a random user hash in 'm_userHash', mark it as eMule, and copy it to 'm_prefs->m_userHash'.
	CreateUserHash();
	md4cpy(&m_prefs->m_userHash,&m_userHash);

//	Load preferences from "preferences.ini" file
	LoadIniPreferences();

	LoadDatPreferences();

//	Mark as emule client. that will be need in later version
	m_userHash[5] = 14;
	m_userHash[14] = 111;

	LoadSharedDirs();

	LoadTempDirs();
		
	LoadServerlistAddresses();

//	Maverick
    m_bUseProxyListenPort = false;
	m_nListenPort = 0;

//	Make sure the Incoming and primary Temp dirs exist.
	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CPreferences::~CPreferences()
{	
	CCat::Finalize();

	delete m_prefs;
	delete m_prefsExt;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadServerlistAddresses() loads the server addresses from "config/addresses.dat" into 'm_addressesList'.
void CPreferences::LoadServerlistAddresses()
{
// Added by Tarod
//	First we must free list
	m_addressesList.RemoveAll();

	CString		strServerAddress ;
	CString		strFullPath = CString(m_strConfigDir) + _T("addresses.dat") ;
	CStdioFile	addressesFile;

	if (addressesFile.Open(strFullPath, CFile::modeRead))
	{
		m_addressesList.RemoveAll();
		while (addressesFile.ReadString(strServerAddress))
		{
        // [i_a] allow commented-out entries in the addresses.dat file.
			strServerAddress.TrimLeft();
			if (strServerAddress.GetLength() > 0 && strServerAddress[0] != '#')
				m_addressesList.AddHead(strServerAddress);
		}
		addressesFile.Close();
	}	
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	SetStandardValues() sets default values for the legacy preference info.
void CPreferences::SetStandardValues()
{
//	Read the user hash from "config/userhash.dat".
	CString		strFullPath;

	strFullPath.Format(_T("%suserhash.dat"), m_strConfigDir);

	FILE		*pHashFile = _tfopen(strFullPath, _T("rb")) ;
	
	if (pHashFile)
	{ 
		if (!fread(m_userHash, sizeof(m_userHash), 1, pHashFile))
            CreateUserHash();
		fclose(pHashFile);
	}
	md4cpy(&m_prefs->m_userHash,&m_userHash);

	WINDOWPLACEMENT		defaultWPM;

	defaultWPM.length = sizeof(WINDOWPLACEMENT);
	defaultWPM.rcNormalPosition.left=10;
	defaultWPM.rcNormalPosition.top=10;
	defaultWPM.rcNormalPosition.right=700;
	defaultWPM.rcNormalPosition.bottom=500;
	defaultWPM.showCmd=0;
	m_prefs->EmuleWindowPlacement=defaultWPM;

	Save();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetMaxDownload() handles download rate limiting based on the max upload rate setting.
uint16 CPreferences::GetMaxDownload()
{
//	Don't be a Lam3r :)
	uint16		nMaxUp = GetMaxUpload();
	uint16		nMaxDown = GetMaxDownloadValue();

//	max-up...
//	0..4:	if max-up < 1/3 max-down, max-down = 3 * max-up. Otherwise unchanged
//	4..9:	if max-up < 1/4 max-down, max-down = 4 * max-up. Otherwise unchanged
//	10..:	Unchanged (or unlimited if the "limitless download" preference is set)
	if (nMaxUp >= 10 && LimitlessDownload()) // DonGato
		return UNLIMITED;

	if (nMaxUp < 4)
		return ((nMaxUp * 3 < nMaxDown) ? nMaxUp * 3 : nMaxDown);
	else
		return (((nMaxUp < 10) && (nMaxUp * 4 < nMaxDown)) ? nMaxUp * 4 : nMaxDown);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Save() writes all the various preferences out to their appropriate files
bool CPreferences::Save()
{
	bool		bError = false;
	CString		strFullPath;

	strFullPath.Format(_T("%spreferences.dat"),m_strConfigDir);

	FILE		*pPrefFile = _tfopen(strFullPath,_T("wb"));

	m_prefsExt->version = PREFFILE_VERSION;
	
	if (pPrefFile != NULL)
	{
		m_prefsExt->version = PREFFILE_VERSION;
		m_prefsExt->EmuleWindowPlacement = m_prefs->EmuleWindowPlacement;
		md4cpy(&m_prefsExt->m_userHash,&m_prefs->m_userHash);

		bError = fwrite(m_prefsExt,sizeof(Preferences_Ext_Struct),1,pPrefFile);
	//	v1:- eklmn: bugfix(19): by shutdown force OS flush data direct to disk [SlugFiller]
		if (!g_eMuleApp.m_pdlgEmule->IsRunning())
			_commit(_fileno(pPrefFile));
		fclose(pPrefFile);
	}
	else
		bError = true;

//	Saving HashID to "config/userhash.dat"
	strFullPath.Format(_T("%suserhash.dat"), m_strConfigDir) ;

	FILE		*pHashFile = _tfopen(strFullPath, _T("wb")) ;
	
	if (pHashFile)
	{
		bError = fwrite(m_prefs->m_userHash, sizeof(m_prefs->m_userHash), 1, pHashFile) ;
	//	v1:- eklmn: bugfix(19): by shutdown force OS flush data direct to disk [SlugFiller]
		if (!g_eMuleApp.m_pdlgEmule->IsRunning())
			_commit(_fileno(pHashFile));
		fclose(pHashFile) ;
	}
	else
		bError = true ;

	SaveIniPreferences();
	
	strFullPath.Format(_T("%sshareddir.dat"),m_strConfigDir);

	CStdioFile		sharedDirsFile;

	if (sharedDirsFile.Open(strFullPath,CFile::modeCreate|CFile::modeWrite))
	{
		for (POSITION pos = m_sharedDirList.GetHeadPosition(); pos != 0; m_sharedDirList.GetNext(pos))
		{
			sharedDirsFile.WriteString(m_sharedDirList.GetAt(pos).GetBuffer());
			sharedDirsFile.Write("\n",1);
		}
		sharedDirsFile.Close();
	}
	else
		bError = true;

//	start added by InterCeptor (multiple tempdirs) 19.11.02	
	strFullPath.Format(_T("%stempdir.dat"),m_strConfigDir);

	CStdioFile		tempDirsFile;

	if (tempDirsFile.Open(strFullPath,CFile::modeCreate|CFile::modeWrite))
	{
		for (POSITION pos = m_tempDirList.GetHeadPosition(); pos != 0; m_tempDirList.GetNext(pos))
		{
			tempDirsFile.WriteString(m_tempDirList.GetAt(pos).GetBuffer());
			tempDirsFile.Write("\n",1);
		}
		tempDirsFile.Close();
	}
	else
		bError = true;
//	end added by InterCeptor (multiple tempdirs) 19.11.02	

	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);

	return bError;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CreateUserHash() generates a random 16 byte ID in 'm_userHash'.
void CPreferences::CreateUserHash()
{
	for (int i = 0; i != 8; i++)
	{ 
		uint16	random = rand();

		memcpy2(&m_userHash[i*2],&random,2);
	}
//	Mark as emule client. that will be need in later version
	m_userHash[5] = 14;
	m_userHash[14] = 111;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CPreferences::GetColumnWidth(EnumTable t, int index) const
{
	switch (t)
	{
		case TABLE_DOWNLOAD:
			return m_prefs->downloadColumnWidths[index];
		case TABLE_UPLOAD:
			return m_prefs->uploadColumnWidths[index];
		case TABLE_QUEUE:
			return m_prefs->queueColumnWidths[index];
		case TABLE_SEARCH:
			return m_prefs->searchColumnWidths[index];
		case TABLE_SHARED:
			return m_prefs->sharedColumnWidths[index];
		case TABLE_SERVER:
			return m_prefs->serverColumnWidths[index];
		case TABLE_IRC:
			return m_prefs->ircColumnWidths[index];
		case TABLE_CLIENTLIST:
			return m_prefs->clientListColumnWidths[index];
	}
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SetColumnWidth(EnumTable t, int index, int width)
{
	switch (t)
	{
		case TABLE_DOWNLOAD:
			m_prefs->downloadColumnWidths[index] = width;
			break;
		case TABLE_UPLOAD:
			m_prefs->uploadColumnWidths[index] = width;
			break;
		case TABLE_QUEUE:
			m_prefs->queueColumnWidths[index] = width;
			break;
		case TABLE_SEARCH:
			m_prefs->searchColumnWidths[index] = width;
			break;
		case TABLE_SHARED:
			m_prefs->sharedColumnWidths[index] = width;
			break;
		case TABLE_SERVER:
			m_prefs->serverColumnWidths[index] = width;
			break;
		case TABLE_IRC:
			m_prefs->ircColumnWidths[index] = width;
			break;
		case TABLE_CLIENTLIST:
			m_prefs->clientListColumnWidths[index] = width;
			break;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CPreferences::GetColumnHidden(EnumTable t, int index) const
{
	switch (t)
	{
		case TABLE_DOWNLOAD:
			return m_prefs->downloadColumnHidden[index];
		case TABLE_UPLOAD:
			return m_prefs->uploadColumnHidden[index];
		case TABLE_QUEUE:
			return m_prefs->queueColumnHidden[index];
		case TABLE_SEARCH:
			return m_prefs->searchColumnHidden[index];
		case TABLE_SHARED:
			return m_prefs->sharedColumnHidden[index];
		case TABLE_SERVER:
			return m_prefs->serverColumnHidden[index];
		case TABLE_IRC:
			return m_prefs->ircColumnHidden[index];
		case TABLE_CLIENTLIST:
			return m_prefs->clientListColumnHidden[index];
	}
	return FALSE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SetColumnHidden(EnumTable t, int index, BOOL bHidden)
{
	switch (t)
	{
		case TABLE_DOWNLOAD:
			m_prefs->downloadColumnHidden[index] = bHidden;
			break;
		case TABLE_UPLOAD:
			m_prefs->uploadColumnHidden[index] = bHidden;
			break;
		case TABLE_QUEUE:
			m_prefs->queueColumnHidden[index] = bHidden;
			break;
		case TABLE_SEARCH:
			m_prefs->searchColumnHidden[index] = bHidden;
			break;
		case TABLE_SHARED:
			m_prefs->sharedColumnHidden[index] = bHidden;
			break;
		case TABLE_SERVER:
			m_prefs->serverColumnHidden[index] = bHidden;
			break;
		case TABLE_IRC:
			m_prefs->ircColumnHidden[index] = bHidden;
			break;
		case TABLE_CLIENTLIST:
			m_prefs->clientListColumnHidden[index] = bHidden;
			break;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CPreferences::GetColumnOrder(EnumTable t, int index) const
{
	switch (t)
	{
		case TABLE_DOWNLOAD:
			return m_prefs->downloadColumnOrder[index];
		case TABLE_UPLOAD:
			return m_prefs->uploadColumnOrder[index];
		case TABLE_QUEUE:
			return m_prefs->queueColumnOrder[index];
		case TABLE_SEARCH:
			return m_prefs->searchColumnOrder[index];
		case TABLE_SHARED:
			return m_prefs->sharedColumnOrder[index];
		case TABLE_SERVER:
			return m_prefs->serverColumnOrder[index];
		case TABLE_IRC:
			return m_prefs->ircColumnOrder[index];
		case TABLE_CLIENTLIST:
			return m_prefs->clientListColumnOrder[index];
	}
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SetColumnOrder(EnumTable t, INT *piOrder)
{
	switch (t)
	{
		case TABLE_DOWNLOAD:
			memcpy2(m_prefs->downloadColumnOrder, piOrder, sizeof(m_prefs->downloadColumnOrder));
			break;
		case TABLE_UPLOAD:
			memcpy2(m_prefs->uploadColumnOrder, piOrder, sizeof(m_prefs->uploadColumnOrder));
			break;
		case TABLE_QUEUE:
			memcpy2(m_prefs->queueColumnOrder, piOrder, sizeof(m_prefs->queueColumnOrder));
			break;
		case TABLE_SEARCH:
			memcpy2(m_prefs->searchColumnOrder, piOrder, sizeof(m_prefs->searchColumnOrder));
			break;
		case TABLE_SHARED:
			memcpy2(m_prefs->sharedColumnOrder, piOrder, sizeof(m_prefs->sharedColumnOrder));
			break;
		case TABLE_SERVER:
			memcpy2(m_prefs->serverColumnOrder, piOrder, sizeof(m_prefs->serverColumnOrder));
			break;
		case TABLE_IRC:
			memcpy2(m_prefs->ircColumnOrder, piOrder, sizeof(m_prefs->ircColumnOrder));
			break;
		case TABLE_CLIENTLIST:
			memcpy2(m_prefs->clientListColumnOrder, piOrder, sizeof(m_prefs->clientListColumnOrder));
			break;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetRecommendedMaxConnections() calculates a recommended connection limit based on the maximum
//		supported by the OS.
int CPreferences::GetRecommendedMaxConnections()
{
//	Get an estimate of the maximum number of connections the OS will support
	int iRealMax = ::GetMaxConnections();

//	If limit is out of range, recommend 500
	if(iRealMax == -1 || iRealMax > 520)
		return 500;

//	If limit is between 0 and 19, recommend it
	if(iRealMax < 20)
		return iRealMax;

//	If limit is between 20 and 256, recommend it be reduced by 10
	if(iRealMax <= 256)
		return iRealMax - 10;

//	If limit is between 257 and 520, recommend it be reduced by 20
	return iRealMax - 20;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SaveIniPreferences()
{
	EMULE_TRY

	CString		buffer;
	CString		fullpath;

	fullpath.Format(_T("%spreferences.ini"),m_strConfigDir);
	
	CIni		ini( fullpath );

	ini.SetDefaultCategory(_T("eMule"));

	ini.SetString(_T("AppVersion"), CURRENT_VERSION_LONG);

	buffer = m_sNick.Get();
	ini.SetString(_T("Nick"),buffer);

	buffer = m_sIncomingDir.Get();
	ini.SetString(_T("IncomingDir"),buffer );

	buffer = m_sTempDir.Get();
	ini.SetString(_T("TempDir"),buffer );

	buffer = m_sVideoPlayer.Get();	//Cax2 - official mule code.
	ini.SetString(_T("VideoPlayer"),buffer );

	ini.SetInt(_T("MaxUpload"), m_nMaxUpload);
	ini.SetInt(_T("MaxDownload"), m_nMaxDownload);
	ini.SetInt(_T("MaxConnections"), m_nMaxConnections);
	ini.SetInt(_T("DeadServerAction"), m_nDeadServer);
	ini.SetInt(_T("Port"), m_uPort);
	ini.SetInt(_T("UDPPort"), m_uUDPPort);
	ini.SetInt(_T("MaxSourcesPerFile"), m_nMaxSourcePerFile);
	ini.SetWORD(_T("Language"), m_nLanguageID);
	ini.SetInt(_T("SeeShare"), m_nSeeShares.Get());
	//--- xrmb:partTrafficPrefs ---
	ini.SetBool(_T("PartTrafficEnabled"), m_bUsePartTraffic);
	ini.SetInt(_T("UploadbarStyle"), m_nUpbarPartTraffic);
	ini.SetInt(_T("UploadbarColor"), m_nUpbarColorPartTraffic);
	//--- :xrmb ---
	ini.SetInt(_T("ToolTipDelay"), m_nTooltipDelayTime);
	ini.SetInt(_T("QueueSize"), m_nQueueSize);
	//ini.SetInt(_T("RemoteQueueLimit"), m_prefs->remotequeuelimit); //remote QR 
	ini.SetBool(_T("UseRemoteQueueLimit"), m_bUseRQlimit);
	ini.SetInt(_T("StatGraphsInterval"), m_nTrafficOMeterInterval);
	ini.SetInt(_T("StatsInterval"), m_nStatsInterval);
	ini.SetInt(_T("DetailColumnWidth"), m_nDetailColumnWidth);
	ini.SetInt(_T("DownloadCapacity"), m_nMaxGraphDownloadRate);
	ini.SetInt(_T("UploadCapacity"), m_nMaxGraphUploadRate);
	ini.SetInt(_T("DeadServerRetries"), m_nDeadServerRetries);
	ini.SetInt(_T("SplitterbarPosition"), m_nSplitterBarPosition);
	ini.SetInt(_T("VariousStatisticsMaxValue"), m_nStatsMax);
	ini.SetInt(_T("StatsAverageMinutes"), m_nStatsAverageMinutes);
	ini.SetInt(_T("MaxConnectionsPerFiveSeconds"), m_nMaxConPerFive);

	ini.SetInt(_T("BadClientBanTime"), m_nBadClientBanTime);
	ini.SetInt(_T("BadClientMinRequestTime"), m_nMinRequestTime);
	ini.SetInt(_T("BadClientMinRequestNum"), m_nBadClientBan);
	ini.SetBool(_T("BanMessageEnabled"), m_bBanMessageEnabled);
	ini.SetBool(_T("BanEnabled"), m_bBanEnabled); // DbT:BanEnDisable

	ini.SetInt(_T("SLS_WhenSourcesOutdated"), m_nSLSOutdated);
	ini.SetInt(_T("SLS_MaxSourcesPerFile"),m_SLSMaxSourcesPerFile);
	ini.SetBool(_T("SLS_Enable"), m_bSLSEnable);

	ini.SetBool(_T("Reconnect"), m_bReconnect);
	ini.SetBool(_T("Scoresystem"), m_nScoreSystem);

	ini.SetBool(_T("UAP"), m_bUAP);
	ini.SetBool(_T("DAP"), m_bDAP);
	ini.SetBool(_T("DisableXS"), m_bDisableXS);
	ini.SetBool(_T("StartDownloadPaused"), m_bStartDownloadPaused);
	ini.SetBool(_T("DownloadPausedOnComplete"), m_bDownloadPausedOnComplete);
	
	ini.SetString(_T("ToolbarSettings"), m_sToolbarSettings.Get()); // Added by reCDVst
	buffer = m_sToolbarBitmap.Get();
	ini.SetString(_T("ToolbarBitmap"),buffer);
	buffer = m_sToolbarBitmapFolder.Get();
	ini.SetString(_T("ToolbarBitmapFolder"),buffer);
	ini.SetInt(_T("ToolbarLabels"), m_nToolbarLabels);
	ini.SetBool(_T("Serverlist"), m_bAutoServerList);
	ini.SetBool(_T("UpdateNotify"), m_bUpdateNotify);
	ini.SetBool(_T("MinimizeToTray"), m_bMinToTray);
	ini.SetBool(_T("AddServersFromConnections"), m_bAddServersFromConn);
	ini.SetBool(_T("Splashscreen"), m_bSplashScreen);
	ini.SetBool(_T("BringToFront"), m_bBringToForeground);
	ini.SetBool(_T("BeepOnError"), m_bBeepOnError);
	ini.SetBool(_T("ConfirmExit"), m_bConfirmExit);
	ini.SetBool(_T("FilterBadIPs"), m_bFilterBadIP);
	ini.SetBool(_T("FilterServersByIP"), m_bFilterServersByIP);
	ini.SetBool(_T("Autoconnect"), m_bAutoConnect);
	ini.SetBool(_T("OnlineSignature"), m_bOnlineSig);
	ini.SetBool(_T("StartupMinimized"), m_bStartMinimized);
	ini.SetBool(_T("ToolbarSpeedMeter"), m_bShowToolbarSpeedMeter);
	ini.SetBool(_T("CloseToTray"), m_bCloseToTray);
	ini.SetBool(_T("DetailsOnClick"), m_bDetailsOnClick);
	ini.SetInt(_T("SrvConTimeout"), m_nSrvConTimeout);
	ini.SetBool(_T("SlowComplete"), m_bSlowComplete);
	ini.SetInt(_T("SlowCompleteBlockSize"), m_nSlowCompleteBlockSize);
	ini.SetBool(_T("ShowRatesOnTitle"),m_prefs->showRatesInTitle);

	// Barry - New properties...
    ini.SetBool(_T("AutoConnectStaticOnly"), m_bAutoConnectStaticOnly);  
    ini.SetBool(_T("GlobalSearch"), m_prefs->globalsearch);  
    ini.SetInt (_T("3DDepth"), m_nDepth3D); 

	ini.SetBool(_T("NotifyOnDownload"), m_bUseDownloadNotifier);
	ini.SetBool(_T("NotifyOnDownloadAdd"), m_bUseDownloadAddNotifier);
	ini.SetBool(_T("NotifyOnChat"), m_bUseChatNotifier);		  
	ini.SetBool(_T("NotifyOnLog"), m_bUseLogNotifier);
	ini.SetBool(_T("NotifierUseSound"), m_bUseSoundInNotifier);
	ini.SetBool(_T("NotifierUseScheduler"), m_bUseSchedulerNotifier);
	ini.SetBool(_T("NotifierPopEveryChatMessage"), m_bNotifierPopsEveryChatMsg);
	ini.SetBool(_T("NotifyOnImportantError"), m_bNotifierImportantError);
	ini.SetBool(_T("NotifyOnWebServerError"), m_bNotifierWebServerError);
	ini.SetBool(_T("NotifyOnServerError"), m_bNotifierServerError);
	ini.SetString(_T("NotifierSoundPath"), CString(m_sNotifierSoundFilePath.Get()));

	ini.SetString(_T("MessageFilter"),m_prefs->messageFilter);
	ini.SetString(_T("CommentFilter"), m_sCommentFilter.Get());
	ini.SetString(_T("TxtEditor"),m_prefs->TxtEditor);
	ini.SetBool(_T("DisableRatingIcons"), m_bDisableRatingIcons);
    ini.SetString(_T("IRCServer"), m_sIRCServer.Get());
	ini.SetString(_T("IRCNick"), m_sIRCNick.Get());
	ini.SetBool(_T("IRCAddTimestamp"), m_bIRCAddTimestamp);
	ini.SetString(_T("IRCFilterName"), m_sIRCChanNameFilter.Get());
	ini.SetInt(_T("IRCFilterUser"), m_nIRCChannelUserFilter);
	ini.SetBool(_T("IRCUseFilter"), m_bIRCUseChanFilter);
	ini.SetString(_T("IRCPerformString"), m_sIRCPerformString.Get());
	ini.SetBool(_T("IRCUsePerform"), m_bIRCUsePerform);
	ini.SetBool(_T("IRCListOnConnect"), m_bIRCListOnConnect);
//	ini.SetBool("IRCAcceptLinks", m_bIRCAcceptLinks);
	ini.SetBool(_T("IRCIgnoreInfoMessage"), m_bIRCIgnoreInfoMessage);
	ini.SetBool(_T("IRCIgnoreEmuleProtoInfoMessage"), m_bIRCIgnoreEmuleProtoInfoMessage);
	ini.SetBool(_T("SmartIdCheck"), m_bSmartIdCheck);

	ini.SetBool(_T("UseSortingDefaults"), m_bUseSort);
	ini.SetBool(_T("Use2ndSortSources"), m_bUseSrcSortCol2);
	ini.SetInt(_T("SortServersCode"), m_nServerSortCol);	
	ini.SetInt(_T("SortUploadsCode"), m_nUploadSortCol);
	ini.SetInt(_T("SortQueueCode"), m_nQueueSortCol);
	ini.SetInt(_T("SortSearchCode"), m_nSearchSortCol);
	ini.SetInt(_T("SortIrcCode"), m_nIRCSortCol);
	ini.SetInt(_T("SortFilesCode"), m_nFileSortCol);
	ini.SetInt(_T("SortDownloadsCode"), m_nDownloadSortCol);
	ini.SetInt(_T("SortClientListCode"), m_nClientListSortCol);
	ini.SetInt(_T("1stSortSourcesCode"), m_nSrcSortCol1);
	ini.SetInt(_T("2ndSortSourcesCode"), m_nSrcSortCol2);

	ini.SetBool(_T("CommunityEnabled"), m_bCommunityEnabled);
	ini.SetString(_T("CommunityString"), m_sCommunityString.Get());
	ini.SetBool(_T("CommunityNoBan"), m_bCommunityNoBan);
	ini.SetInt(_T("NotificationDisplayTime"), m_nNotificationDisplayTime);
	ini.SetInt(_T("NotificationFontSize"), m_nNotificationFontSize);
	ini.SetInt(_T("Profile"), m_nProfile);
	ini.SetInt(_T("PriorityHigh"), m_nPriorityHigh);
	ini.SetInt(_T("PriorityLow"), m_nPriorityLow);
	ini.SetBool(_T("RestartWaiting"), m_bRestartWaiting);
	ini.SetBool(_T("LogToFile"), m_bLogToFile);
	ini.SetBool(_T("BugReport"), m_prefs->BugReport);
	ini.SetBool(_T("DisableXSUpTo"), m_bDisableXSUpTo);
	ini.SetInt(_T("XSUpTo"), m_nXSUpTo);
	ini.SetString(_T("UsedFont"), m_sUsedFont.Get());
	ini.SetInt(_T("FontSize"), m_nFontSize);
	ini.SetInt(_T("HashingPriority"), m_HashingPriority);
	ini.SetBool(_T("UploadParts"), m_bUploadParts);
	ini.SetBool(_T("A4AFStringEnabled"), m_bA4AFStringEnabled);
	ini.SetInt(_T("SearchMethod"), m_nSearchMethod);
	ini.SetBool(_T("AutoTakeED2KLinks"), m_bAutoTakeLinks); 
	ini.SetInt(_T("FileBufferSizePref"), m_nFileBufferSize);
	ini.SetInt(_T("MainProcessPriority"), m_nMainProcessPriority);
	ini.SetBool(_T("ShowAverageDataRate"),m_bShowAverageDataRate);
	ini.SetBool(_T("LimitlessDownload"),m_bLimitlessDownload);
	ini.SetBool(_T("SCHEnabled"), m_bSCHEnabled);
	ini.SetBool(_T("SCHExceptMon"), m_bSCHExceptMon);
	ini.SetBool(_T("SCHExceptTue"), m_bSCHExceptTue);
	ini.SetBool(_T("SCHExceptWed"), m_bSCHExceptWed);
	ini.SetBool(_T("SCHExceptThu"), m_bSCHExceptThu);
	ini.SetBool(_T("SCHExceptFri"), m_bSCHExceptFri);
	ini.SetBool(_T("SCHExceptSat"), m_bSCHExceptSat);
	ini.SetBool(_T("SCHExceptSun"), m_bSCHExceptSun);
	ini.SetInt(_T("SCHShift1"),m_nSCHShift1);
	ini.SetInt(_T("SCHShift2"),m_nSCHShift2);
	ini.SetInt(_T("SCHShift1Up"),m_nSCHShift1Upload);
	ini.SetInt(_T("SCHShift1Down"),m_nSCHShift1Download);
	ini.SetInt(_T("SCHShift1Conn"),m_nSCHShift1conn);
	ini.SetInt(_T("SCHShift15sec"),m_nSCHShift15sec);
	ini.SetInt(_T("SCHShift2Up"),m_nSCHShift2Upload);
	ini.SetInt(_T("SCHShift2Down"),m_nSCHShift2Download);
	ini.SetInt(_T("SCHShift2Conn"),m_nSCHShift2conn);
	ini.SetInt(_T("SCHShift25sec"),m_nSCHShift25sec);
	ini.SetBool(_T("ShowFileTypeIcon"), m_bShowFileTypeIcon);

	ini.SetBool(_T("MultipleInstances"), m_bMultiple);

	ini.SetBool(_T("Verbose"), m_bVerbose);
	ini.SetBool(_T("UpdateQueueList"), m_bUpdateQueueList);
	ini.SetBool(_T("ManualHighPrio"), m_bManualHighPrio);
//	ini.SetBool("StartNextFile", m_prefs->m_bstartnextfile); //duplicates downloadpausedoncomplete
	ini.SetBool(_T("ShowOverhead"), m_bShowOverhead);	//Cax2 re-enabled this preference
	ini.SetBool(_T("VideoPreviewBackupped"), m_bBackupPreview);	//Cax2 string kept for compatibility with off client

	ini.SetInt(_T("AllCatType"), CCat::GetAllCatType());
	ini.SetBool(_T("ShowInfoOnCatTabs"), m_bShowCatTabInfo);

	ini.SetArray(m_prefs->downloadColumnWidths, ELEMENT_COUNT(m_prefs->downloadColumnWidths), _T("DownloadColumnWidths"));
	ini.SetArray(m_prefs->downloadColumnHidden, ELEMENT_COUNT(m_prefs->downloadColumnHidden), _T("DownloadColumnHidden"));
	ini.SetArray(m_prefs->downloadColumnOrder, ELEMENT_COUNT(m_prefs->downloadColumnOrder), _T("DownloadColumnOrder"));
	ini.SetArray(m_prefs->uploadColumnWidths, ELEMENT_COUNT(m_prefs->uploadColumnWidths), _T("UploadColumnWidths"));
	ini.SetArray(m_prefs->uploadColumnHidden, ELEMENT_COUNT(m_prefs->uploadColumnHidden), _T("UploadColumnHidden"));
	ini.SetArray(m_prefs->uploadColumnOrder, ELEMENT_COUNT(m_prefs->uploadColumnOrder), _T("UploadColumnOrder"));
	ini.SetArray(m_prefs->queueColumnWidths, ELEMENT_COUNT(m_prefs->queueColumnWidths), _T("QueueColumnWidths"));
	ini.SetArray(m_prefs->queueColumnHidden, ELEMENT_COUNT(m_prefs->queueColumnHidden), _T("QueueColumnHidden"));
	ini.SetArray(m_prefs->queueColumnOrder, ELEMENT_COUNT(m_prefs->queueColumnOrder), _T("QueueColumnOrder"));
	ini.SetArray(m_prefs->searchColumnWidths, ELEMENT_COUNT(m_prefs->searchColumnWidths), _T("SearchColumnWidths"));
	ini.SetArray(m_prefs->searchColumnHidden, ELEMENT_COUNT(m_prefs->searchColumnHidden), _T("SearchColumnHidden"));
	ini.SetArray(m_prefs->searchColumnOrder, ELEMENT_COUNT(m_prefs->searchColumnOrder), _T("SearchColumnOrder"));
	ini.SetArray(m_prefs->sharedColumnWidths, ELEMENT_COUNT(m_prefs->sharedColumnWidths), _T("SharedColumnWidths"));
	ini.SetArray(m_prefs->sharedColumnHidden, ELEMENT_COUNT(m_prefs->sharedColumnHidden), _T("SharedColumnHidden"));
	ini.SetArray(m_prefs->sharedColumnOrder, ELEMENT_COUNT(m_prefs->sharedColumnOrder), _T("SharedColumnOrder"));
	ini.SetArray(m_prefs->serverColumnWidths, ELEMENT_COUNT(m_prefs->serverColumnWidths), _T("ServerColumnWidths"));
	ini.SetArray(m_prefs->serverColumnHidden, ELEMENT_COUNT(m_prefs->serverColumnHidden), _T("ServerColumnHidden"));
	ini.SetArray(m_prefs->serverColumnOrder, ELEMENT_COUNT(m_prefs->serverColumnOrder), _T("ServerColumnOrder"));
	ini.SetArray(m_prefs->ircColumnWidths, ELEMENT_COUNT(m_prefs->ircColumnWidths), _T("IrcColumnWidths"));
	ini.SetArray(m_prefs->ircColumnHidden, ELEMENT_COUNT(m_prefs->ircColumnHidden), _T("IrcColumnHidden"));
	ini.SetArray(m_prefs->ircColumnOrder, ELEMENT_COUNT(m_prefs->ircColumnOrder), _T("IrcColumnOrder"));
	ini.SetArray(m_prefs->clientListColumnWidths, ELEMENT_COUNT(m_prefs->clientListColumnWidths), _T("ClientListColumnWidths"));
	ini.SetArray(m_prefs->clientListColumnHidden, ELEMENT_COUNT(m_prefs->clientListColumnHidden), _T("ClientListColumnHidden"));
	ini.SetArray(m_prefs->clientListColumnOrder, ELEMENT_COUNT(m_prefs->clientListColumnOrder), _T("ClientListColumnOrder"));

//	Barry - Provide a mechanism for all tables to store/retrieve sort order
	ini.SetInt(_T("TableSortItemDownload"), m_prefs->tableSortItemDownload);
	ini.SetInt(_T("TableSortItemDownload2"), m_prefs->tableSortItemDownload2);
	ini.SetInt(_T("TableSortItemUpload"), m_prefs->tableSortItemUpload);
	ini.SetInt(_T("TableSortItemQueue"), m_prefs->tableSortItemQueue);
	ini.SetInt(_T("TableSortItemSearch"), m_prefs->tableSortItemSearch);
	ini.SetInt(_T("TableSortItemShared"), m_prefs->tableSortItemShared);
	ini.SetInt(_T("TableSortItemServer"), m_prefs->tableSortItemServer);
	ini.SetInt(_T("TableSortItemClientList"), m_prefs->tableSortItemClientList);
	ini.SetBool(_T("TableSortAscendingDownload"), m_prefs->tableSortAscendingDownload);
	ini.SetBool(_T("TableSortAscendingDownload2"), m_prefs->tableSortAscendingDownload2);
	ini.SetBool(_T("TableSortAscendingUpload"), m_prefs->tableSortAscendingUpload);
	ini.SetBool(_T("TableSortAscendingQueue"), m_prefs->tableSortAscendingQueue);
	ini.SetBool(_T("TableSortAscendingSearch"), m_prefs->tableSortAscendingSearch);
	ini.SetBool(_T("TableSortAscendingShared"), m_prefs->tableSortAscendingShared);
	ini.SetBool(_T("TableSortAscendingServer"), m_prefs->tableSortAscendingServer);
	ini.SetBool(_T("TableSortAscendingClientList"), m_prefs->tableSortAscendingClientList);

	CString buffer2;
	for (int i=0;i<15;i++)
	{
		buffer.Format(_T("%I64Lu"),GetStatsColor(i));
		buffer2.Format(_T("StatColor%i"),i);
		ini.SetString(buffer2,buffer);
	}
	ini.SetInt(_T("ConnectionsGraphRatio"), m_nGraphRatio);

	ini.SetBool(_T("MlDonkeyCM"), m_bMlDonkeyCM);
	ini.SetBool(_T("CMNotLog"), m_bCMNotLog);

	//AutoCheck for new version
	ini.SetBool(_T("AutoCheckForNewVersion"), m_bAutoCheckForNewVersion);
	ini.SetInt(_T("AutoCheckLastTime"), m_iAutoCheckLastTime);

	CString hex_hash = FileHashToString(m_prefs->A4AF_file_hash);
	ini.SetString(_T("A4AFAutoFileHash"), hex_hash);	

	//START enkeyDEV(kei-kun) -Messaging preferences-
	ini.SetInt(_T("AcceptMessageFrom"),m_nAcceptMessagesFrom);	
	ini.SetString(_T("AwayStateMessage"), CString(m_sAwayStateMessage.Get()));

	ini.SetBool(_T("ScanFilter"), m_bScanFilter);
	ini.SetBool(_T("PreviewSmallBlocks"), m_iPreviewSmallBlocks);
	ini.SetBool(_T("ShowFullFileStatusIcons"), m_bShowFullFileStatusIcons);
	ini.SetBool(_T("ShowPausedGray"), m_bShowPausedGray);
	ini.SetBool(_T("TransferredOnCompleted"), m_bTransferredOnCompleted);
	ini.SetBool(_T("UseDwlPercentage"), m_bUseDwlPercentage);
	ini.SetBool("SecureIdent", m_bUseSecureIdent);// change the name in future version to enable it by default
	ini.SetString(_T("URLsForICC"), CString(m_sURLsForICC.Get()));

	// Maverick
	ini.SetDefaultCategory(_T("Proxy"));
	ini.SetBool(_T("EnablePassword"),m_prefs->proxy.m_bEnablePassword);
	ini.SetBool(_T("EnableProxy"),m_prefs->proxy.m_bUseProxy);
	ini.SetString(_T("Name"),m_prefs->proxy.m_strName);
	ini.SetString(_T("Password"),m_prefs->proxy.m_strPassword);
	ini.SetString(_T("User"),m_prefs->proxy.m_strUser);
	ini.SetInt(_T("Port"),m_prefs->proxy.m_uPort);
	ini.SetInt(_T("Type"),m_prefs->proxy.m_nType);
	ini.SetBool(_T("ConnectWithoutProxy"),m_bIsASCWOP);

	// Web Server
	ini.SetDefaultCategory(_T("WebServer"));
	ini.SetString(_T("Password"), CString(m_sWebPassword.Get()));	
	ini.SetString(_T("PasswordLow"), CString(m_sLowWebPassword.Get()));	
	ini.SetInt(_T("Port"), m_nWebPort);
	ini.SetBool(_T("Enabled"), m_bWebEnabled);
	ini.SetBool(_T("EnabledLow"), m_bWebLowEnabled);
	ini.SetBool(_T("UseGzip"), m_bWebUseGzip);
	ini.SetInt(_T("PageRefreshTime"), m_nWebPageRefresh);
	ini.SetBool(_T("IntruderDetection"), m_bWebIntruderDetection);
	ini.SetInt(_T("TempDisableLogin"), m_nWebTempDisableLogin);
	ini.SetInt(_T("LoginAttemptsAllowed"), m_nWebLoginAttemptsAllowed);
	ini.SetString(_T("WebTemplateFile"),m_prefs->m_sTemplateFile);	// netwolf

	//mobileMule
	ini.SetDefaultCategory(_T("MobileMule"));
	ini.SetString("Password", GetMMPass());
	ini.SetBool("Enabled", m_prefs->m_bMMEnabled);
	ini.SetInt("Port", m_prefs->m_nMMPort);

	// LANCAST (moosetea) - Save the settings
	CString t_strLanCastIP;
	t_strLanCastIP.Format(_T("%i.%i.%i.%i"),(uint8)m_LancastIP,(uint8)(m_LancastIP>>8),(uint8)(m_LancastIP>>16),(uint8)(m_LancastIP>>24));
	CString t_strLanCastSubnet;
	t_strLanCastSubnet.Format(_T("%i.%i.%i.%i"),(uint8)m_LancastSubnet,(uint8)(m_LancastSubnet>>8),(uint8)(m_LancastSubnet>>16),(uint8)(m_LancastSubnet>>24));
	ini.SetDefaultCategory(_T("Lancast"));
	ini.SetBool(_T("Enabled"),m_LancastEnabled);
	ini.SetBool(_T("UseDefaultIP"),m_UseDefaultIP);
	ini.SetString(_T("IPAddress"),t_strLanCastIP);
	ini.SetString(_T("Subnet"),t_strLanCastSubnet);
	ini.SetInt(_T("Port"),m_LancastPort);

	//SyruS: cleanup
	ini.SetDefaultCategory(_T("Cleanup"));
	ini.SetBool(_T("AutoFilenameCleanup"), m_bAutoFilenameCleanup);
	ini.SetBool(_T("CleanupTags"), m_bFilenameCleanupTags);
	ini.SetString(_T("FilenameCleanups"), CString(m_sFilenameCleanups.Get()));

	//DonGato: SMTP
	ini.SetDefaultCategory(_T("SMTP"));
	ini.SetString(_T("Server"), CString(m_sSMTPServer.Get()));
	ini.SetString(_T("Name"), CString(m_sSMTPName.Get()));
	ini.SetString(_T("From"), CString(m_sSMTPFrom.Get()));
	ini.SetString(_T("To"), CString(m_sSMTPTo.Get()));
	ini.SetString(_T("UserName"), CString(m_sSMTPUserName.Get()));
	ini.SetString(_T("Password"), CString(m_sSMTPPassword.Get()));
	ini.SetBool(_T("Authenticated"), m_bSMTPAuthenticated);
	ini.SetBool(_T("Info"), m_bSMTPInfo);
	ini.SetBool(_T("Warning"), m_bSMTPWarning);

	//N@boleo (Backup feature)
	ini.SetDefaultCategory(_T("Backup"));
	ini.SetBool(_T("DatFiles"), m_bDatFiles);
	ini.SetBool(_T("MetFiles"), m_bMetFiles);
	ini.SetBool(_T("IniFiles"), m_bIniFiles);
	ini.SetBool(_T("PartFiles"), m_bPartFiles);
	ini.SetBool(_T("PartMetFiles"), m_bPartMetFiles);
	ini.SetBool(_T("AutoBackup"), m_bAutoBackup);
	ini.SetBool(_T("BackupOverwrite"), m_bBackupOverwrite);
	ini.SetString(_T("BackupDir"), CString(m_sBackupDir.Get()));

	//FakeCheck
	ini.SetDefaultCategory(_T("FakeCheck"));
	ini.SetInt(_T("FakesDatVersion"),m_nFakesDatVersion);
	ini.SetBool(_T("UpdateFakeStartup"),m_bUpdateFakeStartup);
	ini.SetString(_T("FakeListURL"), m_sFakeListURL);

	// Shortcut management
	ini.SetDefaultCategory(_T("Shortcuts"));
	ini.SetInt(_T("GeneralMinimizeWindowToTrayShortcut"),	m_anShortcutCode[IndexGalMinimize]);
	ini.SetInt(_T("GeneralSwitchWindowShortcut"),			m_anShortcutCode[IndexGalSwitchWindow]);
	ini.SetInt(_T("GeneralShowServerWndShortcut"),			m_anShortcutCode[IndexGalShowServer]);
	ini.SetInt(_T("GeneralShowTransferWndShortcut"),		m_anShortcutCode[IndexGalShowTransfer]);
	ini.SetInt(_T("GeneralShowSearchWndShortcut"),			m_anShortcutCode[IndexGalShowSearch]);
	ini.SetInt(_T("GeneralShowSharedFilesWndShortcut"),		m_anShortcutCode[IndexGalShowShare]);
	ini.SetInt(_T("GeneralShowChatWndShortcut"),			m_anShortcutCode[IndexGalShowChat]);
	ini.SetInt(_T("GeneralShowIRCWndShortcut"),				m_anShortcutCode[IndexGalShowIRC]);
	ini.SetInt(_T("GeneralShowStatisticsWndShortcut"),		m_anShortcutCode[IndexGalShowStats]);
	ini.SetInt(_T("GeneralShowPreferencesWndShortcut"),		m_anShortcutCode[IndexGalShowPrefs]);
	ini.SetInt(_T("DownloadCancelShortcut"),				m_anShortcutCode[IndexDLCancel]);
	ini.SetInt(_T("DownloadStopShortcut"),					m_anShortcutCode[IndexDLStop]);
	ini.SetInt(_T("DownloadPauseShortcut"),					m_anShortcutCode[IndexDLPause]);
	ini.SetInt(_T("DownloadResumeShortcut"),				m_anShortcutCode[IndexDLResume]);
	ini.SetInt(_T("DownloadOpenShortcut"),					m_anShortcutCode[IndexDLOpen]);
	ini.SetInt(_T("DownloadPreviewShortcut"),				m_anShortcutCode[IndexDLPreview]);
	ini.SetInt(_T("DownloadViewFileCommentsShortcut"),		m_anShortcutCode[IndexDLComments]);
	ini.SetInt(_T("DownloadViewFileDetailsShortcut"),		m_anShortcutCode[IndexDLDetails]);
	ini.SetInt(_T("DownloadClearCompletedFilesShortcut"),	m_anShortcutCode[IndexDLClear]);
	ini.SetInt(_T("DownloadPreallocateShortcut"),			m_anShortcutCode[IndexDLPreallocate]);
	ini.SetInt(_T("DownloadA4AFShortcut"),					m_anShortcutCode[IndexDLA4AF]);
	ini.SetInt(_T("DownloadA4AFAutoShortcut"),				m_anShortcutCode[IndexDLA4AFAuto]);
	ini.SetInt(_T("DownloadED2KLinkShortcut"),				m_anShortcutCode[IndexDLED2KLink]);
	ini.SetInt(_T("DownloadED2kLinkHtmlShortcut"),			m_anShortcutCode[IndexDLED2kLinkHtml]);
	ini.SetInt(_T("DownloadED2kLinkSourceShortcut"),		m_anShortcutCode[IndexDLED2kLinkSource]);
	ini.SetInt(_T("UploadViewClientDetailsShortcut"),		m_anShortcutCode[IndexULDetails]);
	ini.SetInt(_T("UploadAddClientToFriendsShortcut"),		m_anShortcutCode[IndexULFriend]);
	ini.SetInt(_T("UploadSendMessageClientShortcut"),		m_anShortcutCode[IndexULMessage]);
	ini.SetInt(_T("UploadViewClientFilesShortcut"),			m_anShortcutCode[IndexULFiles]);
	ini.SetInt(_T("ShareOpenShortcut"),						m_anShortcutCode[IndexShOpenFolder]);
	ini.SetInt(_T("ShareEditFileCommentsShortcut"),			m_anShortcutCode[IndexShComments]);
	ini.SetInt(_T("ShareED2KLinkShortcut"),					m_anShortcutCode[IndexShED2KLink]);
	ini.SetInt(_T("ShareED2kLinkHtmlShortcut"),				m_anShortcutCode[IndexShED2kLinkHtml]);
	ini.SetInt(_T("ShareED2kLinkSourceShortcut"),			m_anShortcutCode[IndexShED2kLinkSource]);

	ini.SaveAndClose();

//	Now file is close, so we can update a stats over SaveStats()
	SaveStats();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SaveCats()
{
	// Cats
	CString		catinif, ixStr, buffer;

	catinif.Format(_T("%scategory.ini"), m_strConfigDir);
	remove(catinif);

	if (CCat::GetNumCats() > 1)
	{
		CIni		catini(catinif);

		catini.AddCategory("General");
		catini.SetInt(_T("Count"),CCat::GetNumCats()-CCat::GetNumPredefinedCats(),_T("General"));
		
		for (int i = 1; i < CCat::GetNumPredefinedCats(); i++)
		{
			CCat	*pCat = CCat::GetCatByIndex(i);

			ixStr.Format(_T("PredefinedCat%#i"),i);
			catini.SetInt(ixStr,pCat->m_eCatID,_T("General"));
		}

	//	If there are user-defined categories...
		if (CCat::GetNumCats() > CCat::GetNumPredefinedCats())
		{
			for (int ix = CCat::GetNumPredefinedCats(); ix < CCat::GetNumCats(); ix++)
			{
				CCat	*pCat = CCat::GetCatByIndex(ix);

				ixStr.Format("Cat#%i",ix-CCat::GetNumPredefinedCats()+1);
				catini.AddCategory(ixStr);
				catini.SetString(_T("Title"), pCat->m_strTitle, ixStr);
				catini.SetString(_T("Incoming"), pCat->m_strSavePath, ixStr);
				catini.SetString(_T("Comment"), pCat->m_strComment, ixStr);
				buffer.Format("%lu",pCat->m_crColor,ixStr);
				catini.SetString(_T("Color"), buffer, ixStr);
				catini.SetInt(_T("Priority"), pCat->m_iPriority, ixStr);
				catini.SetString(_T("AutoCat"),pCat->m_strAutoCatExt,ixStr); 
				catini.SetInt(_T("CategoryID"),pCat->m_eCatID,ixStr);
			}
			catini.SaveAndClose();
		}
		catini.SaveAndClose();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetDefaultStatsColor()
DWORD CPreferences::GetDefaultStatsColor(int index)
{
	switch(index)
	{
		case 0 : return RGB(0,0,64);
		case 1 : return RGB(192,192,255);
		case 2 : return RGB(128, 255, 128);
		case 3 : return RGB(0, 210, 0);
		case 4 : return RGB(0, 128, 0);
		case 5 : return RGB(255, 128, 128);
		case 6 : return RGB(200, 0, 0);
		case 7 : return RGB(140, 0, 0);
		case 8 : return RGB(150, 150, 255);
		case 9 : return RGB(192,   0, 192);
		case 10 : return RGB(255, 192, 255);
		case 11 : return RGB(255, 255, 128);
		default : return RGB(255, 255, 255);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadIniPreferences() loads preferences stored in the "preferences.ini" file.
void CPreferences::LoadIniPreferences()
{
//	CSingleLock PrefLock(&m_Pref_Mutex,TRUE);
	
//
//	Quick hack to add version tag to preferences.ini-file and solve the issue with the FlatStatusBar tag...
	CString strFileName;

	strFileName.Format(_T("%spreferences.ini"), m_strConfigDir);

	CIni* pIni = new CIni(strFileName);

	pIni->SetDefaultCategory(_T("eMule"));

	CString strCurrVersion, strPrefsVersion;

	strCurrVersion = CURRENT_VERSION_LONG;
	strPrefsVersion = pIni->GetString(_T("AppVersion"),_T(""));
	pIni->SetString(_T("AppVersion"),CURRENT_VERSION_LONG);
	pIni->SaveAndClose();
	delete pIni;

	CFileFind findFileName;

	bool bChangedVersion = false;
	int loadstatsFromOld = 0;	// -khaos--+++> Fix to stats being lost when version changes!

	if ((strCurrVersion != strPrefsVersion) && findFileName.FindFile(strFileName) && strPrefsVersion!= _T(""))
	{
		CFile file;
		CFileFind findNewName;
		CString strNewName;

	//	Cax2 we can add the code to find our version's setting, if the ini file is from a different version...

	//
	//	Rename the old .ini file
		strNewName.Format(_T("%spreferences[%s].ini"), m_strConfigDir,strPrefsVersion);	
		if (findNewName.FindFile(strNewName))
			file.Remove(strNewName);
		file.Rename(strFileName, strNewName);
		strFileName = strNewName;

		bChangedVersion = true;
		loadstatsFromOld = 2; // -khaos--+++> Set this to 2 so that LoadStats will load 'em from ini.old
	}

	CIni ini(strFileName);
	ini.SetDefaultCategory(_T("eMule"));
	//--- end Ozon :)

//
//	Generate a random six character default nickname and add the version tag to it
	CString		strDefNick;

	srand((unsigned)time(NULL));

	CString buffer;

	for (int i = 0; i < 6; i++)
		buffer += char(rand() % ('z' - 'a') + 'a');
	
	strDefNick.Format(_T("%s [ePlus]"), buffer);
	m_sNick.Put(ini.GetString(_T("Nick"),strDefNick).GetBuffer(0));
	
//	Get the incoming directory (default appdir\Incoming)
	buffer.Format(_T("%sIncoming"),m_strAppDir);
	m_sIncomingDir.Put(ini.GetString(_T("IncomingDir"),buffer).GetBuffer(0));

//	Get the temp directory (default appdir\Temp)
	buffer.Format(_T("%sTemp"),m_strAppDir);
	m_sTempDir.Put(ini.GetString(_T("TempDir"),buffer).GetBuffer(0));

//	Get the VLC directory (default appdir\VLC)
	buffer.Format(_T("%sVLC"),m_strAppDir);
	m_sVlcDir.Put(ini.GetString(_T("VLCDir"),buffer).GetBuffer(0));

//	Get the Video Player path (default VLCDir\vlc.exe)
	buffer.Format(_T("%s\\vlc.exe"),GetVlcDir());		//Cax2 - for backward compatibility
	m_sVideoPlayer.Put(ini.GetString(_T("VideoPlayer"),buffer).GetBuffer(0));		//Cax2 - replace buffer with "" in later (non b-compatible) versions

//	Get the Upload Capacity (default 16)
	m_nMaxGraphUploadRate = ini.GetInt(_T("UploadCapacity"),16);
//	Get the Download Capacity (default 96)
	m_nMaxGraphDownloadRate = ini.GetInt(_T("DownloadCapacity"),96);
//	Get the Upload Limit (default UNLIMITED - pinned to Upload Capacity)
	m_nMaxUpload = ini.GetInt(_T("MaxUpload"),UNLIMITED);		//Cax2 new limits enforced on old settings
	if (m_nMaxUpload>m_nMaxGraphUploadRate)
		m_nMaxUpload = m_nMaxGraphUploadRate;
//	Get the Download limit (default UNLIMITED - pinned to Download Capacity)
	m_nMaxDownload = ini.GetInt(_T("MaxDownload"),UNLIMITED);
	if (m_nMaxDownload>m_nMaxGraphDownloadRate)
		m_nMaxDownload = m_nMaxGraphDownloadRate;
//	Get the Connection Limit (default - a bit less than the max the OS will support)
	m_nMaxConnections = ini.GetInt(_T("MaxConnections"),GetRecommendedMaxConnections());
//	Get the Dead Server Action (default 2? - non-zero indicates "remove")
	m_nDeadServer = ini.GetInt(_T("DeadServerAction"),2);
//	Get the TCP port (default 4662)
	m_uPort = ini.GetInt(_T("Port"),4662);
//	Get the UDP port (default 4672)
	m_uUDPPort = ini.GetInt(_T("UDPPort"),m_uPort + 10);
//	Get the Sources Per File Limit (default 400)
	m_nMaxSourcePerFile = ini.GetInt(_T("MaxSourcesPerFile"),400);
//	Get the Queue Size Limit (default 3 x Min Queue Size - pinned between Min Queue Size and Max Queue Size)
	m_nQueueSize = ini.GetInt(_T("QueueSize"), MIN_QUEUE*3);
	if(m_nQueueSize > MAX_QUEUE)
		m_nQueueSize = MAX_QUEUE;
	else if(m_nQueueSize < MIN_QUEUE)
		m_nQueueSize = MIN_QUEUE;
//	Get the Remote Queue Size Limit (default Queue Size Limit - pinned above 2000)
	m_nRQlimit = (m_nQueueSize >= 2000)? m_nQueueSize : 2000; //Cax2 - remote queue limit linked to queue size, but no lower than 2000
	m_bUseRQlimit = ini.GetBool(_T("UseRemoteQueueLimit"), false); //bond006 - set default value
	m_nLanguageID = ini.GetWORD(_T("Language"),0);
//	Get the See Shares setting (default Everybody)
	m_nSeeShares.Put((enSeeShare)ini.GetInt(_T("SeeShare"),SEE_SHARE_EVERYBODY));
	//--- xrmb:partTrafficPrefs ---
	m_bUsePartTraffic = ini.GetBool(_T("PartTrafficEnabled"),false);
	m_nUpbarPartTraffic = ini.GetInt(_T("UploadbarStyle"), 2);
	m_nUpbarColorPartTraffic = ini.GetInt(_T("UploadbarColor"), 0);
	//--- :xrmb ---
	m_nTooltipDelayTime = ini.GetInt(_T("ToolTipDelay"),1);
	m_nTrafficOMeterInterval = ini.GetInt(_T("StatGraphsInterval"),3);
	m_nStatsInterval = ini.GetInt(_T("StatsInterval"),5);
	m_nDetailColumnWidth = ini.GetInt(_T("DetailColumnWidth"), 150);
	m_nDeadServerRetries = ini.GetInt(_T("DeadServerRetries"),1);
	m_nSplitterBarPosition = ini.GetInt(_T("SplitterbarPosition"),75);
	m_nStatsMax = ini.GetInt(_T("VariousStatisticsMaxValue"),100);
	m_nStatsAverageMinutes = ini.GetInt(_T("StatsAverageMinutes"),5);
	m_nMaxConPerFive = ini.GetInt(_T("MaxConnectionsPerFiveSeconds"),GetDefaultMaxConPerFiveSecs());
	m_prefs->showRatesInTitle=ini.GetBool(_T("ShowRatesOnTitle"),false);

	// DonGato
	int nMax = (LOBYTE(GetWindowsVersion()) == 0x04)?100:200;

	if(m_nMaxConPerFive > nMax)
		m_nMaxConPerFive = nMax;
	else
	{
		if(m_nMaxConPerFive < 5)
			m_nMaxConPerFive = 5;
	}

	m_nBadClientBanTime = ini.GetInt(_T("BadClientBanTime"),3600000);
	if(m_nBadClientBanTime > 18000000)
		m_nBadClientBanTime = 18000000;
	
	m_nMinRequestTime = ini.GetInt(_T("BadClientMinRequestTime"),590000);
	if(m_nMinRequestTime > 590000)
		m_nMinRequestTime = 590000;
	
	m_nBadClientBan = ini.GetInt(_T("BadClientMinRequestNum"),3);
	if(m_nBadClientBan < 3)
		m_nBadClientBan = 3;
	m_bBanMessageEnabled = ini.GetBool(_T("BanMessageEnabled"), false);
	m_bBanEnabled = ini.GetBool(_T("BanEnabled"),true); // DbT:BanEnDisable

	m_nSLSOutdated = ini.GetInt(_T("SLS_WhenSourcesOutdated"),1);
	if(m_nSLSOutdated > 2)
		m_nSLSOutdated = 2;
	else if(m_nSLSOutdated < 1)
		m_nSLSOutdated = 1;

	m_SLSMaxSourcesPerFile = ini.GetInt(_T("SLS_MaxSourcesPerFile"),10);
	if(m_SLSMaxSourcesPerFile > 20)
		m_SLSMaxSourcesPerFile = 20;
	else if(m_SLSMaxSourcesPerFile < 5)
		m_SLSMaxSourcesPerFile = 5;

	m_bSLSEnable = ini.GetBool(_T("SLS_Enable"),false);

	m_bReconnect = ini.GetBool(_T("Reconnect"),true);
	m_nScoreSystem = ini.GetBool(_T("Scoresystem"),true);
	
	m_bUAP = ini.GetBool(_T("UAP"), true);
	m_bDAP = ini.GetBool(_T("DAP"), true);
	m_bDisableXS = ini.GetBool(_T("DisableXS"), false);
	m_bStartDownloadPaused = ini.GetBool(_T("StartDownloadPaused"), false);
	m_bDownloadPausedOnComplete = ini.GetBool(_T("DownloadPausedOnComplete"), false);
	
	m_sToolbarSettings.Put(ini.GetString(_T("ToolbarSettings"), strDefaultToolbar).GetBuffer(0));
	m_sToolbarBitmap.Put(ini.GetString(_T("ToolbarBitmap"), _T("")).GetBuffer(0)); // Added by reCDVst
	m_sToolbarBitmapFolder.Put(ini.GetString(_T("ToolbarBitmapFolder"), _T("")).GetBuffer(0)); // Added by reCDVst
	m_nToolbarLabels = ini.GetInt(_T("ToolbarLabels"),1);
	m_bAutoServerList = ini.GetBool(_T("Serverlist"),false);
	m_bUpdateNotify = ini.GetBool(_T("UpdateNotify"),true);
	m_bMinToTray = ini.GetBool(_T("MinimizeToTray"),true);
	m_bAddServersFromConn = ini.GetBool(_T("AddServersFromConnections"),true);
	m_bSplashScreen = ini.GetBool(_T("Splashscreen"),true);
	m_bBringToForeground = ini.GetBool(_T("BringToFront"),true);
	m_bBeepOnError = ini.GetBool(_T("BeepOnError"),true);
	m_bConfirmExit = ini.GetBool(_T("ConfirmExit"),true);
	m_bFilterBadIP = ini.GetBool(_T("FilterBadIPs"),true);
	m_bFilterServersByIP = ini.GetBool(_T("FilterServersByIP"),false);
	m_bAutoConnect = ini.GetBool(_T("Autoconnect"),false);
	m_bOnlineSig = ini.GetBool(_T("OnlineSignature"),false);
	m_bStartMinimized = ini.GetBool(_T("StartupMinimized"),false);
	m_bShowToolbarSpeedMeter = ini.GetBool(_T("ToolbarSpeedMeter"), false);
	m_bCloseToTray = ini.GetBool(_T("CloseToTray"),false);	
	m_bDetailsOnClick = ini.GetBool(_T("DetailsOnClick"),false);	

	CCat::SetAllCatType(static_cast<_EnumCategories>(ini.GetInt("AllCatType",CAT_ALL)));
	m_bShowCatTabInfo = ini.GetBool("ShowInfoOnCatTabs",false);

	m_nSrvConTimeout = ini.GetInt(_T("SrvConTimeout"),20000);		//Cax2 - bugfix for time out - ticks are in milliseconds!!!!!
	if(m_nSrvConTimeout < 10000)
		m_nSrvConTimeout = 10000;
	else if(m_nSrvConTimeout > 60000)
		m_nSrvConTimeout = 60000;
	
	m_bSlowComplete = ini.GetBool(_T("SlowComplete"),false);	
	m_nSlowCompleteBlockSize = ini.GetInt(_T("SlowCompleteBlockSize"),32);
	if(m_nSlowCompleteBlockSize > 1024)
		m_nSlowCompleteBlockSize = 1024;

	// Barry - New properties...
	m_bAutoConnectStaticOnly = ini.GetBool(_T("AutoConnectStaticOnly"),false); 
	m_prefs->globalsearch = ini.GetBool(_T("GlobalSearch"),true); 
	m_nDepth3D = ini.GetInt(_T("3DDepth"), 0);

	// as a temporary converter for previous versions
	if (strPrefsVersion < _T("0.25a")) // before 0.25a
		if (ini.GetBool(_T("FlatStatusBar"),false))
			m_nDepth3D = 0;
		else 
			m_nDepth3D = 5;

	// Added by enkeyDEV
	m_bUseDownloadAddNotifier = ini.GetBool(_T("NotifyOnDownloadAdd"),false);
	m_bUseDownloadNotifier = ini.GetBool(_T("NotifyOnDownload"),false);
	m_bUseChatNotifier = ini.GetBool(_T("NotifyOnChat"),false);
	m_bUseLogNotifier = ini.GetBool(_T("NotifyOnLog"),false);
	m_bUseSoundInNotifier = ini.GetBool(_T("NotifierUseSound"),false);
	m_bUseSchedulerNotifier = ini.GetBool(_T("NotifierUseScheduler"),false);
	m_bNotifierPopsEveryChatMsg = ini.GetBool(_T("NotifierPopEveryChatMessage"),false);
	m_bNotifierImportantError = ini.GetBool(_T("NotifyOnImportantError"),false);
	m_bNotifierWebServerError = ini.GetBool(_T("NotifyOnWebServerError"),false);
	m_bNotifierServerError = ini.GetBool(_T("NotifyOnServerError"),false);
	m_sNotifierSoundFilePath.Put(ini.GetString(_T("NotifierSoundPath"),_T("")).GetBuffer(0));

	m_sIRCServer.Put(ini.GetString(_T("IRCServer"),_T("217.172.179.168")).GetBuffer(0));
	m_sIRCNick.Put(ini.GetString(_T("IRCNick"),_T("eMule Plus")).GetBuffer(0));
	m_bIRCAddTimestamp = ini.GetBool(_T("IRCAddTimestamp"),true);
	m_sIRCChanNameFilter.Put(ini.GetString(_T("IRCFilterName"), _T("")).GetBuffer(0));
	m_bIRCUseChanFilter = ini.GetBool(_T("IRCUseFilter"), false);
	m_nIRCChannelUserFilter = ini.GetInt(_T("IRCFilterUser"), 0);
	m_sIRCPerformString.Put(ini.GetString(_T("IRCPerformString"), _T("/join #emule+")).GetBuffer(0));
	m_bIRCUsePerform = ini.GetBool(_T("IRCUsePerform"), false);
	m_bIRCListOnConnect = ini.GetBool(_T("IRCListOnConnect"), true);
	//	m_bIRCAcceptLinks = ini.GetBool("IRCAcceptLinks", false);
	m_bIRCIgnoreInfoMessage = ini.GetBool(_T("IRCIgnoreInfoMessage"), false);
	m_bIRCIgnoreEmuleProtoInfoMessage = ini.GetBool(_T("IRCIgnoreEmuleProtoInfoMessage"), true);
	m_bSmartIdCheck = ini.GetBool(_T("SmartIdCheck"),true);
	m_bVerbose = ini.GetBool(_T("Verbose"),false);
	m_bUpdateQueueList = ini.GetBool(_T("UpdateQueueList"),false);
	m_bManualHighPrio=ini.GetBool(_T("ManualHighPrio"),false);
	m_bShowOverhead = ini.GetBool(_T("ShowOverhead"),true);
	m_bBackupPreview = ini.GetBool(_T("VideoPreviewBackupped"),false);	//Cax2 string kept for compatibility with off client

	m_prefs->TxtEditor = ini.GetString(_T("TxtEditor"),_T("notepad.exe"));
	m_prefs->messageFilter=ini.GetString(_T("MessageFilter"),_T("Your client has an infinite queue|http://www.chez.com/theworld/"));
	m_sCommentFilter.Put(ini.GetString(_T("CommentFilter"),_T("http://www.|emulespeedup")).GetBuffer(0));	//(SyruS) netwolf commentfilter
	m_bDisableRatingIcons=ini.GetBool(_T("DisableRatingIcons"),false);	//netwolf: disable icons

	m_bUseSort = ini.GetBool(_T("UseSortingDefaults"),true);
	m_bUseSrcSortCol2 = ini.GetBool(_T("Use2ndSortSources"),false);
	m_nServerSortCol = ini.GetInt(_T("SortServersCode"),0);
	m_nUploadSortCol = ini.GetInt(_T("SortUploadsCode"),12);
	m_nQueueSortCol = ini.GetInt(_T("SortQueueCode"),105);
	m_nSearchSortCol = ini.GetInt(_T("SortSearchCode"),12);
	m_nIRCSortCol = ini.GetInt(_T("SortIrcCode"),11);
	m_nFileSortCol = ini.GetInt(_T("SortFilesCode"),0);
	m_nDownloadSortCol = ini.GetInt(_T("SortDownloadsCode"),8);
	m_nClientListSortCol = ini.GetInt(_T("SortClientListCode"),0);
	m_nSrcSortCol1 = ini.GetInt(_T("1stSortSourcesCode"),0);
	m_nSrcSortCol2 = ini.GetInt(_T("2ndSortSourcesCode"),DL_DEFAULTSOURCESORT);

	m_bCommunityEnabled = ini.GetBool(_T("CommunityEnabled"), false);
	m_sCommunityString.Put(ini.GetString(_T("CommunityString"),_T("")).GetBuffer(0));
	m_bCommunityNoBan = ini.GetBool(_T("CommunityNoBan"), false);
	m_nNotificationDisplayTime = ini.GetInt(_T("NotificationDisplayTime"),4000);
	m_nNotificationFontSize = ini.GetInt(_T("NotificationFontSize"),70);
	m_nProfile = ini.GetInt(_T("Profile"),0);
	m_nPriorityHigh = ini.GetInt(_T("PriorityHigh"),100);
	m_nPriorityLow = ini.GetInt(_T("PriorityLow"),300);
	if(m_nPriorityLow != 0)
	{
		if(m_nPriorityLow < m_nPriorityHigh)
			m_nPriorityLow = m_nPriorityHigh * 2;
	}
	else
	{
		m_nPriorityLow = 300;
		m_nPriorityHigh = 100;
	}
	m_bRestartWaiting = ini.GetBool(_T("RestartWaiting"),true);
	m_bLogToFile = ini.GetBool(_T("LogToFile"),false);
	m_prefs->BugReport = ini.GetBool(_T("BugReport"),false);
	m_bDisableXSUpTo = ini.GetBool(_T("DisableXSUpTo"),false);
	m_nXSUpTo = ini.GetInt(_T("XSUpTo"),GetMaxSourcePerFileSoft());
	if(m_nXSUpTo > GetMaxSourcePerFileSoft())
		m_nXSUpTo = GetMaxSourcePerFileSoft();
	m_sUsedFont.Put(ini.GetString(_T("UsedFont"), _T("MS Sans Serif")).GetBuffer(0));
	m_nFontSize = ini.GetInt(_T("FontSize"),80);
	m_HashingPriority.Put((enHashPriority)ini.GetInt(_T("HashingPriority"),2));
	m_bUploadParts = ini.GetBool(_T("UploadParts"),false);
	m_bA4AFStringEnabled = ini.GetBool(_T("A4AFStringEnabled"), false);
	m_bBanMessageEnabled = ini.GetBool(_T("BanMessageEnabled"), false);
	m_nSearchMethod = ini.GetInt(_T("SearchMethod"),0);
	m_bAutoTakeLinks = ini.GetBool(_T("AutoTakeED2KLinks"),false);
	m_nFileBufferSize = ini.GetInt(_T("FileBufferSizePref"),16);
	m_nMainProcessPriority = ini.GetInt(_T("MainProcessPriority"),0);	// netwolf 05.05.03 (default: normal MainProcessPriority)
	m_bShowAverageDataRate = ini.GetBool(_T("ShowAverageDataRate"),false);
	m_bLimitlessDownload = ini.GetBool(_T("LimitlessDownload"),false);

	m_bSCHEnabled = ini.GetBool(_T("SCHEnabled"), false);
	m_bSCHExceptMon = ini.GetBool(_T("SCHExceptMon"), false);
	m_bSCHExceptTue = ini.GetBool(_T("SCHExceptTue"), false);
	m_bSCHExceptWed = ini.GetBool(_T("SCHExceptWed"), false);
	m_bSCHExceptThu = ini.GetBool(_T("SCHExceptThu"), false);
	m_bSCHExceptFri = ini.GetBool(_T("SCHExceptFri"), false);
	m_bSCHExceptSat = ini.GetBool(_T("SCHExceptSat"), true);
	m_bSCHExceptSun = ini.GetBool(_T("SCHExceptSun"), true);
	m_nSCHShift1 = ini.GetInt(_T("SCHShift1"),64800);
	m_nSCHShift2 = ini.GetInt(_T("SCHShift2"),21600);
	m_nSCHShift1Upload = ini.GetInt(_T("SCHShift1Up"),UNLIMITED);
	m_nSCHShift1Download = ini.GetInt(_T("SCHShift1Down"),UNLIMITED);
	m_nSCHShift1conn = ini.GetInt(_T("SCHShift1Conn"),512);
	m_nSCHShift15sec = ini.GetInt(_T("SCHShift15sec"),40);
	m_nSCHShift2Upload = ini.GetInt(_T("SCHShift2Up"),UNLIMITED);
	m_nSCHShift2Download = ini.GetInt(_T("SCHShift2Down"),UNLIMITED);
	m_nSCHShift2conn = ini.GetInt(_T("SCHShift2Conn"),256);
	m_nSCHShift25sec = ini.GetInt(_T("SCHShift25sec"),20);
	m_bShowFileTypeIcon = ini.GetBool(_T("ShowFileTypeIcon"), true);

	m_bMultiple = ini.GetBool(_T("MultipleInstances"),false);

	ini.GetArray(m_prefs->downloadColumnWidths, ELEMENT_COUNT(m_prefs->downloadColumnWidths), _T("DownloadColumnWidths"));
	ini.GetArray(m_prefs->downloadColumnHidden, ELEMENT_COUNT(m_prefs->downloadColumnHidden), _T("DownloadColumnHidden"));
	ini.GetArray(m_prefs->downloadColumnOrder, ELEMENT_COUNT(m_prefs->downloadColumnOrder), _T("DownloadColumnOrder"));
	ini.GetArray(m_prefs->uploadColumnWidths, ELEMENT_COUNT(m_prefs->uploadColumnWidths), _T("UploadColumnWidths"));
	ini.GetArray(m_prefs->uploadColumnHidden, ELEMENT_COUNT(m_prefs->uploadColumnHidden), _T("UploadColumnHidden"));
	ini.GetArray(m_prefs->uploadColumnOrder, ELEMENT_COUNT(m_prefs->uploadColumnOrder), _T("UploadColumnOrder"));
	ini.GetArray(m_prefs->queueColumnWidths, ELEMENT_COUNT(m_prefs->queueColumnWidths), _T("QueueColumnWidths"));
	ini.GetArray(m_prefs->queueColumnHidden, ELEMENT_COUNT(m_prefs->queueColumnHidden), _T("QueueColumnHidden"));
	ini.GetArray(m_prefs->queueColumnOrder, ELEMENT_COUNT(m_prefs->queueColumnOrder), _T("QueueColumnOrder"));
	ini.GetArray(m_prefs->searchColumnWidths, ELEMENT_COUNT(m_prefs->searchColumnWidths), _T("SearchColumnWidths"));
	ini.GetArray(m_prefs->searchColumnHidden, ELEMENT_COUNT(m_prefs->searchColumnHidden), _T("SearchColumnHidden"));
	ini.GetArray(m_prefs->searchColumnOrder, ELEMENT_COUNT(m_prefs->searchColumnOrder), _T("SearchColumnOrder"));
	ini.GetArray(m_prefs->sharedColumnWidths, ELEMENT_COUNT(m_prefs->sharedColumnWidths), _T("SharedColumnWidths"));
	ini.GetArray(m_prefs->sharedColumnHidden, ELEMENT_COUNT(m_prefs->sharedColumnHidden), _T("SharedColumnHidden"));
	ini.GetArray(m_prefs->sharedColumnOrder, ELEMENT_COUNT(m_prefs->sharedColumnOrder), _T("SharedColumnOrder"));
	ini.GetArray(m_prefs->serverColumnWidths, ELEMENT_COUNT(m_prefs->serverColumnWidths), _T("ServerColumnWidths"));
	ini.GetArray(m_prefs->serverColumnHidden, ELEMENT_COUNT(m_prefs->serverColumnHidden), _T("ServerColumnHidden"));
	ini.GetArray(m_prefs->serverColumnOrder, ELEMENT_COUNT(m_prefs->serverColumnOrder), _T("ServerColumnOrder"));
	ini.GetArray(m_prefs->ircColumnWidths, ELEMENT_COUNT(m_prefs->ircColumnWidths), _T("IrcColumnWidths"));
	ini.GetArray(m_prefs->ircColumnHidden, ELEMENT_COUNT(m_prefs->ircColumnHidden), _T("IrcColumnHidden"));
	ini.GetArray(m_prefs->ircColumnOrder, ELEMENT_COUNT(m_prefs->ircColumnOrder), _T("IrcColumnOrder"));
	ini.GetArray(m_prefs->clientListColumnWidths, ELEMENT_COUNT(m_prefs->clientListColumnWidths), _T("ClientListColumnWidths"));
	ini.GetArray(m_prefs->clientListColumnHidden, ELEMENT_COUNT(m_prefs->clientListColumnHidden), _T("ClientListColumnHidden"));
	ini.GetArray(m_prefs->clientListColumnOrder, ELEMENT_COUNT(m_prefs->clientListColumnOrder), _T("ClientListColumnOrder"));

//	Barry - Provide a mechanism for all tables to store/retrieve sort order
	m_prefs->tableSortItemDownload = ini.GetInt(_T("TableSortItemDownload"), 0);
	m_prefs->tableSortItemDownload2 = ini.GetInt(_T("TableSortItemDownload2"), 0);
	m_prefs->tableSortItemUpload = ini.GetInt(_T("TableSortItemUpload"), 0);
	m_prefs->tableSortItemQueue = ini.GetInt(_T("TableSortItemQueue"), 0);
	m_prefs->tableSortItemSearch = ini.GetInt(_T("TableSortItemSearch"), 0);
	m_prefs->tableSortItemShared = ini.GetInt(_T("TableSortItemShared"), 0);
	m_prefs->tableSortItemServer = ini.GetInt(_T("TableSortItemServer"), 0);
	m_prefs->tableSortItemClientList = ini.GetInt(_T("TableSortItemClientList"), 0);
	m_prefs->tableSortAscendingDownload = ini.GetBool(_T("TableSortAscendingDownload"), true);
	m_prefs->tableSortAscendingDownload2 = ini.GetBool(_T("TableSortAscendingDownload2"), true);
	m_prefs->tableSortAscendingUpload = ini.GetBool(_T("TableSortAscendingUpload"), true);
	m_prefs->tableSortAscendingQueue = ini.GetBool(_T("TableSortAscendingQueue"), true);
	m_prefs->tableSortAscendingSearch = ini.GetBool(_T("TableSortAscendingSearch"), true);
	m_prefs->tableSortAscendingShared = ini.GetBool(_T("TableSortAscendingShared"), true);
	m_prefs->tableSortAscendingServer = ini.GetBool(_T("TableSortAscendingServer"), true);
	m_prefs->tableSortAscendingClientList = ini.GetBool(_T("TableSortAscendingClientList"), true);

	if (m_nStatsAverageMinutes < 1)
		m_nStatsAverageMinutes = 5;

	CString buffer2;
	for (int i=0;i<15;i++)
	{
		buffer2.Format(_T("StatColor%i"),i);
		buffer.Format(ini.GetString(buffer2,_T("-1")));	//we can now have black as background
		m_nStatColors[i] = _tstoi64(buffer);
		if (m_nStatColors[i] == -1) 
			m_nStatColors[i] = GetDefaultStatsColor(i);
	}
	m_nGraphRatio = ini.GetInt(_T("ConnectionsGraphRatio"),1);

	m_bMlDonkeyCM = ini.GetBool(_T("MlDonkeyCM"),true);
	m_bCMNotLog = ini.GetBool(_T("CMNotLog"),false);	//netwolf

	//AutoCheck for new version
	m_bAutoCheckForNewVersion = ini.GetBool(_T("AutoCheckForNewVersion"), true);
	m_iAutoCheckLastTime = ini.GetInt(_T("AutoCheckLastTime"), 0);

	// Added by Tarod -to load hash of A4AF auto file
	char hex_byte[3];
	int byte;
	CString A4AF_hash = ini.GetString(_T("A4AFAutoFileHash"), _T("00000000000000000000000000000000"));
	if (A4AF_hash.GetLength() != 32) A4AF_hash = _T("00000000000000000000000000000000");
	hex_byte[2] = '\0';
	for (int i = 0, j = 0; i < 16; i++, j += 2)
	{
		hex_byte[0] = A4AF_hash.GetAt(j);
		hex_byte[1] = A4AF_hash.GetAt((j + 1));
		sscanf(hex_byte, "%02X", &byte);
		m_prefs->A4AF_file_hash[i] = (uchar)byte;
	}

	//START enkeyDEV(kei-kun) -Messaging preferences-
	m_nAcceptMessagesFrom = ini.GetInt(_T("AcceptMessageFrom"),1);
	m_nAwayState = false;
	m_sAwayStateMessage.Put(ini.GetString(_T("AwayStateMessage"),_T("")).GetBuffer(0));

	m_bScanFilter = ini.GetBool(_T("ScanFilter"),false);
	m_iPreviewSmallBlocks = ini.GetBool(_T("PreviewSmallBlocks"), false);
	m_bShowFullFileStatusIcons = ini.GetBool(_T("ShowFullFileStatusIcons"), true);
	m_bShowPausedGray = ini.GetBool(_T("ShowPausedGray"), false);
	m_bTransferredOnCompleted = ini.GetBool(_T("TransferredOnCompleted"), false);
	m_bUseDwlPercentage = ini.GetBool(_T("UseDwlPercentage"), true);
	m_bUseSecureIdent=ini.GetBool("SecureIdent",true);
	m_sURLsForICC.Put(ini.GetString(_T("URLsForICC"),"www.microsoft.com|www.google.com|www.yahoo.com|www.ibm.com|www.army.mil|www.networksolutions.com|www.ripe.net|www.w3.org").GetBuffer(0));

	// Maverick
	ini.SetDefaultCategory(_T("Proxy"));
	m_prefs->proxy.m_bEnablePassword = ini.GetBool(_T("EnablePassword"),false);
	m_prefs->proxy.m_bUseProxy = ini.GetBool(_T("EnableProxy"),false);
	m_prefs->proxy.m_strName = ini.GetString(_T("Name"),"").GetBuffer(0);
	m_prefs->proxy.m_strPassword = ini.GetString(_T("Password"), "").GetBuffer(0);
	m_prefs->proxy.m_strUser = ini.GetString(_T("User"),"").GetBuffer(0);
	m_prefs->proxy.m_uPort = ini.GetInt(_T("Port"),1080);
	m_prefs->proxy.m_nType = ini.GetInt(_T("Type"),PROXYTYPE_NOPROXY);
	m_bIsASCWOP = ini.GetBool(_T("ConnectWithoutProxy"),false);

	// Web Server
	ini.SetDefaultCategory(_T("WebServer"));
	m_sWebPassword.Put(ini.GetString(_T("Password"), _T("mypass")).GetBuffer(0));
	m_sLowWebPassword.Put(ini.GetString(_T("PasswordLow"), _T("mypass")).GetBuffer(0));
	m_nWebPort = ini.GetInt(_T("Port"), 4711);
	m_bWebEnabled = ini.GetBool(_T("Enabled"), false);
	m_bWebLowEnabled = ini.GetBool(_T("EnabledLow"), false);
	m_bWebUseGzip = ini.GetBool(_T("UseGzip"), true);
	m_nWebPageRefresh = ini.GetInt(_T("PageRefreshTime"), 120);
	m_bWebIntruderDetection = ini.GetBool(_T("IntruderDetection"), true);
	m_nWebTempDisableLogin = ini.GetInt(_T("TempDisableLogin"), 30);
	m_nWebLoginAttemptsAllowed = ini.GetInt(_T("LoginAttemptsAllowed"), 3);
	sprintf(m_prefs->m_sTemplateFile,"%s",ini.GetString(_T("WebTemplateFile"),_T("eMule.tmpl")));	// netwolf

	// mobilemule
	ini.SetDefaultCategory(_T("MobileMule"));
	sprintf(m_prefs->m_sMMPassword,"%s",ini.GetString("Password", ""));
	m_prefs->m_bMMEnabled=ini.GetBool("Enabled", false);
	m_prefs->m_nMMPort = ini.GetInt("Port", 80);

	// LANCAST (moosetea) - Read the settings
	// Get the Defualt Local IP address, using this clients hostname,
	// we will use this ip as the "default" Lancast ip, if no ip is found in the ini file.
	char localHost[255];
	gethostname(localHost, 255);						// Get this clients hostname
	struct hostent *host = gethostbyname(localHost);	// Get the host info
	// Get this clients default ip as a string
	CString defaultIP(inet_ntoa (*(struct in_addr*)*host->h_addr_list));
	// Get the ini file settings
	ini.SetDefaultCategory(_T("Lancast"));
	m_LancastEnabled = ini.GetBool(_T("Enabled"),false);
	m_UseDefaultIP = ini.GetBool(_T("UseDefaultIP"),true);
	m_DefaultLancastIP = inet_addr(ini.GetString(_T("IPAddress"),defaultIP));
	m_LancastIP = inet_addr(ini.GetString(_T("IPAddress"),defaultIP));
	m_LancastSubnet = inet_addr(ini.GetString(_T("Subnet"),_T("255.255.0.0")));
	m_LancastPort = ini.GetInt(_T("Port"),5000);

	//SyruS: cleanup
	ini.SetDefaultCategory(_T("Cleanup"));
	m_bAutoFilenameCleanup = ini.GetBool(_T("AutoFilenameCleanup"), false);
	m_bFilenameCleanupTags = ini.GetBool(_T("CleanupTags"), false);
	buffer2.SetString(_T("http|www.|.com|sharereactor|sharelive|filedonkey|saugstube|eselfilme|eseldownloads|donkey-mania|emulemovies|spanishare|eselpsychos.de|saughilfe.de|goldesel.6x.to|goldesel.tk|freedivx.org|elitedivx|deviance|filenexus|ac3-guru"));
	m_sFilenameCleanups.Put(ini.GetString(_T("FilenameCleanups"), buffer2).GetBuffer(0));

	//DonGato: SMTP
	ini.SetDefaultCategory(_T("SMTP"));
	m_sSMTPServer.Put(ini.GetString(_T("Server"), "smtp.server.com").GetBuffer(0));
	m_sSMTPName.Put(ini.GetString(_T("Name"), "eMule Plus Messenger").GetBuffer(0));
	m_sSMTPFrom.Put(ini.GetString(_T("From"), "messenger@emuleplus.sf.net").GetBuffer(0));
	m_sSMTPTo.Put(ini.GetString(_T("To"), "user@domain.com").GetBuffer(0));
	m_sSMTPUserName.Put(ini.GetString(_T("UserName"), "username").GetBuffer(0));
	m_sSMTPPassword.Put(ini.GetString(_T("Password"), Crypt("password")).GetBuffer(0));
	m_bSMTPAuthenticated = ini.GetBool(_T("Authenticated"), false);
	m_bSMTPInfo = ini.GetBool(_T("Info"), false);
	m_bSMTPWarning = ini.GetBool(_T("Warning"), false);

    //N@boleo (Backup feature)
	ini.SetDefaultCategory(_T("Backup"));
	m_bDatFiles = ini.GetBool(_T("DatFiles"),false);
	m_bMetFiles = ini.GetBool(_T("MetFiles"),false);
	m_bIniFiles = ini.GetBool(_T("IniFiles"),false);
	m_bPartFiles = ini.GetBool(_T("PartFiles"),false);
	m_bPartMetFiles = ini.GetBool(_T("PartMetFiles"),false);
	m_bAutoBackup = ini.GetBool(_T("AutoBackup"), true);
	m_bBackupOverwrite = ini.GetBool(_T("BackupOverwrite"), true);
	buffer.Format(_T("%sBackup"),m_strAppDir);
	m_sBackupDir.Put(ini.GetString(_T("BackupDir"),buffer).GetBuffer(0));

	//FakeCheck
	ini.SetDefaultCategory(_T("FakeCheck"));
	m_nFakesDatVersion=ini.GetInt(_T("FakesDatVersion"),0);
	m_bUpdateFakeStartup=ini.GetBool(_T("UpdateFakeStartup"),false);
	m_sFakeListURL = ini.GetString(_T("FakeListURL"), "http://donkeyfakes.gambri.net/fakecheck/update/").Trim();

	// Shortcut management
	ini.SetDefaultCategory(_T("Shortcuts"));
	// VK_RETURN is used as the default value because it is the first element
	// in the dropdown list of the shortcut dialog (in the settings)
	m_anShortcutCode[IndexGalMinimize]		= ini.GetInt(_T("GeneralMinimizeWindowToTrayShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalSwitchWindow]	= ini.GetInt(_T("GeneralSwitchWindowShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowServer]	= ini.GetInt(_T("GeneralShowServerWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowTransfer]	= ini.GetInt(_T("GeneralShowTransferWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowSearch]	= ini.GetInt(_T("GeneralShowSearchWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowShare]		= ini.GetInt(_T("GeneralShowSharedFilesWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowChat]		= ini.GetInt(_T("GeneralShowChatWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowIRC]		= ini.GetInt(_T("GeneralShowIRCWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowStats]		= ini.GetInt(_T("GeneralShowStatisticsWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexGalShowPrefs]		= ini.GetInt(_T("GeneralShowPreferencesWndShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLCancel]			= ini.GetInt(_T("DownloadCancelShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLStop]			= ini.GetInt(_T("DownloadStopShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLPause]			= ini.GetInt(_T("DownloadPauseShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLResume]			= ini.GetInt(_T("DownloadResumeShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLOpen]			= ini.GetInt(_T("DownloadOpenShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLPreview]		= ini.GetInt(_T("DownloadPreviewShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLComments]		= ini.GetInt(_T("DownloadViewFileCommentsShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLDetails]		= ini.GetInt(_T("DownloadViewFileDetailsShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLClear]			= ini.GetInt(_T("DownloadClearCompletedFilesShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLPreallocate]	= ini.GetInt(_T("DownloadPreallocateShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLA4AF]			= ini.GetInt(_T("DownloadA4AFShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLA4AFAuto]		= ini.GetInt(_T("DownloadA4AFAutoShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLED2KLink]		= ini.GetInt(_T("DownloadED2KLinkShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLED2kLinkHtml]	= ini.GetInt(_T("DownloadED2kLinkHtmlShortcut"), VK_RETURN);
	m_anShortcutCode[IndexDLED2kLinkSource]	= ini.GetInt(_T("DownloadED2kLinkSourceShortcut"), VK_RETURN);
	m_anShortcutCode[IndexULDetails]		= ini.GetInt(_T("UploadViewClientDetailsShortcut"), VK_RETURN);
	m_anShortcutCode[IndexULFriend]			= ini.GetInt(_T("UploadAddClientToFriendsShortcut"), VK_RETURN);
	m_anShortcutCode[IndexULMessage]		= ini.GetInt(_T("UploadSendMessageClientShortcut"), VK_RETURN);
	m_anShortcutCode[IndexULFiles]			= ini.GetInt(_T("UploadViewClientFilesShortcut"), VK_RETURN);
	m_anShortcutCode[IndexShOpenFolder]		= ini.GetInt(_T("ShareOpenShortcut"), VK_RETURN);
	m_anShortcutCode[IndexShComments]		= ini.GetInt(_T("ShareEditFileCommentsShortcut"), VK_RETURN);
	m_anShortcutCode[IndexShED2KLink]		= ini.GetInt(_T("ShareED2KLinkShortcut"), VK_RETURN);
	m_anShortcutCode[IndexShED2kLinkHtml]	= ini.GetInt(_T("ShareED2kLinkHtmlShortcut"), VK_RETURN);
	m_anShortcutCode[IndexShED2kLinkSource]	= ini.GetInt(_T("ShareED2kLinkSourceShortcut"), VK_RETURN);

	SetLanguage();

	ini.CloseWithoutSave();

//	Now when file is closed, let's call LoadStats()
	// statsitic 
	LoadStats(loadstatsFromOld, strFileName);

	//	Categories
	LoadCats();
	if (CCat::GetNumCats() == 1)
		CCat::SetAllCatType(CCat::GetCatByIndex(0)->m_eCatID);

	if(bChangedVersion)
		SaveIniPreferences();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadDatPreferences() loads the legacy preference info (version, user hash, window position, etc) from
//		"config/preferences.dat" 
void CPreferences::LoadDatPreferences()
{
	CString		strFullPath;

	strFullPath.Format(_T("%spreferences.dat"),m_strConfigDir);

	FILE		*pPrefsFile = _tfopen(strFullPath,_T("rb"));

	if (pPrefsFile == NULL)
	{
		SetStandardValues();
		return;
	}

	fread(m_prefsExt,sizeof(Preferences_Ext_Struct),1,pPrefsFile);

	if (ferror(pPrefsFile))
		SetStandardValues();

//	Import old pref-files
	if (m_prefsExt->version < 20)
	{
		if (m_prefsExt->version > 17)
		{// v0.20b+
			prefsImport20b=new Preferences_Import20b_Struct;
			memset2(prefsImport20b,0,sizeof(Preferences_Import20b_Struct));
			fseek(pPrefsFile,0,0);
			fread(prefsImport20b,sizeof(Preferences_Import20b_Struct),1,pPrefsFile);

			md4cpy(&m_prefs->m_userHash,&prefsImport20b->m_userHash);
			m_sIncomingDir.Put(prefsImport20b->incomingdir);
			m_sTempDir.Put(prefsImport20b->tempdir);
			m_sNick.Put(prefsImport20b->nick);

			m_nTotalDownloadedBytes.Put(prefsImport20b->totalDownloadedBytes);
			m_nTotalUploadedBytes.Put(prefsImport20b->totalUploadedBytes);

		}
		else if (m_prefsExt->version > 7)
		{ // v0.20a
			prefsImport20a = new Preferences_Import20a_Struct;
			memset2(prefsImport20a,0,sizeof(Preferences_Import20a_Struct));
			fseek(pPrefsFile,0,0);
			fread(prefsImport20a,sizeof(Preferences_Import20a_Struct),1,pPrefsFile);

			md4cpy(&m_prefs->m_userHash,&prefsImport20a->m_userHash);
			m_sIncomingDir.Put(prefsImport20a->incomingdir);
			m_sTempDir.Put(prefsImport20a->tempdir);
			m_sNick.Put(prefsImport20a->nick);

			m_nTotalDownloadedBytes.Put(prefsImport20a->totalDownloaded);
			m_nTotalUploadedBytes.Put(prefsImport20a->totalUploaded);

		}
		else
		{	//v0.19c-
			prefsImport19c = new Preferences_Import19c_Struct;
			memset2(prefsImport19c,0,sizeof(Preferences_Import19c_Struct));

			fseek(pPrefsFile,0,0);
			fread(prefsImport19c,sizeof(Preferences_Import19c_Struct),1,pPrefsFile);

			if (m_prefsExt->version<3)
			{
				CreateUserHash();
				md4cpy(&m_prefs->m_userHash,&m_userHash);
			} 
			else 
				md4cpy(&m_prefs->m_userHash,&prefsImport19c->m_userHash);
			m_sIncomingDir.Put(prefsImport19c->incomingdir);
			m_sTempDir.Put(prefsImport19c->tempdir);
			m_sNick.Put(prefsImport19c->nick);
		}
 	}
	else
	{
		md4cpy(&m_prefs->m_userHash,&m_prefsExt->m_userHash);
		m_prefs->EmuleWindowPlacement = m_prefsExt->EmuleWindowPlacement;
	}
	fclose(pPrefsFile);
	md4cpy(&m_userHash,&m_prefs->m_userHash);
	m_nSmartIdState = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadSharedDirs() loads the shared file paths from "config/shareddir.dat" into 'm_sharedDirList'.
void CPreferences::LoadSharedDirs()
{
	CString		strFullPath;

	strFullPath.Format(_T("%sshareddir.dat"),m_strConfigDir);

	CStdioFile	sDirFile;

	if (sDirFile.Open(strFullPath,CFile::modeRead))
	{
		CString		strSharedFilePath;

		while (sDirFile.ReadString(strSharedFilePath))
			m_sharedDirList.AddHead(strSharedFilePath);
		sDirFile.Close();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadTempDirs() loads the supplemental temp directory paths from "config/tempdir.dat" into 'm_tempDirList'.
void CPreferences::LoadTempDirs()
{
	CString			strFullPath;

// start added by InterCeptor (multiple tempdirs) 19.11.02
	strFullPath.Format(_T("%stempdir.dat"),m_strConfigDir);

	CStdioFile		tempDirFile;

	if (tempDirFile.Open(strFullPath,CFile::modeRead))
	{
		CString		strTempPath;

		while (tempDirFile.ReadString(strTempPath))
		{
			if(m_tempDirList.Find(strTempPath) == NULL)
			{
				m_tempDirList.AddHead(strTempPath);
				::CreateDirectory(strTempPath,0);
			}
		}
		tempDirFile.Close();
	}
// end added by InterCeptor (multiple tempdirs) 19.11.02
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::LoadCats() 
{
	CString		ixStr,catinif,cat_a,cat_b,cat_c;
	TCHAR		buffer[100];

	CCat::g_iNumPredefinedCats = 0;
	m_bShowCatTabInfo = true;

	catinif.Format(_T("%sCategory.ini"), m_strConfigDir);

	// default cat
	CCat		*newcat = new CCat();

	newcat->m_strTitle = CCat::GetPredefinedCatTitle(CAT_ALL,GetLanguageID());
	newcat->m_strSavePath = GetIncomingDir();
	newcat->m_strComment = _T("");
	newcat->m_iPriority = 0;
	newcat->m_crColor = RGB(0,0,0);
	newcat->m_strAutoCatExt = _T("");
	newcat->m_eCatID = CAT_ALL;
	newcat->m_bIsPredefined = true;
	CCat::AddCat(newcat,false);
	CCat::g_iNumPredefinedCats++;

	if(!PathFileExists(catinif)) 
		return;

	CIni catini(catinif);

	for (int i = 1; i <= CAT_MAXPREDEFINEDCATS; i++)
	{
		EnumCategories		eCatID = CAT_NONE;

		ixStr.Format("PredefinedCat%i",i);
		eCatID = static_cast<_EnumCategories>(catini.GetInt(ixStr,CAT_NONE,"General"));
		if (eCatID == CAT_NONE)
			break;
		else
		{
			CCat		*newcat = new CCat();

			newcat->m_strTitle = CCat::GetPredefinedCatTitle(eCatID,GetLanguageID());
			newcat->m_strSavePath = GetIncomingDir();
			newcat->m_strComment = _T("");
			newcat->m_iPriority = 0;
			newcat->m_crColor = 0;
			newcat->m_strAutoCatExt = _T("");
			newcat->m_eCatID = eCatID;
			newcat->m_bIsPredefined = true;
			CCat::AddCat(newcat,false);
			CCat::g_iNumPredefinedCats++;
		}
	}

	int max = catini.GetInt(_T("Count"), 0, _T("General"));

	for(int ix = 1; ix <= max; ix++)
	{
		ixStr.Format(_T("Cat#%i"), ix);
		catini.SetDefaultCategory(ixStr);

		CCat	*newcat = new CCat();
		TCHAR	strSavePath[MAX_PATH];

		newcat->m_strTitle = catini.GetString(_T("Title"),_T(""),ixStr);
		_tcscpy(strSavePath, catini.GetString(_T("Incoming"),_T(""),ixStr));
		MakeFolderName(strSavePath);
		if (!IsShareableDirectory(strSavePath))
		{
			_tcsncpy(strSavePath, GetIncomingDir(),ARRSIZE(strSavePath));
			MakeFolderName(strSavePath);
		}
		newcat->m_strSavePath = strSavePath;
		newcat->m_strComment = catini.GetString(_T("Comment"),_T(""),ixStr);
		newcat->m_iPriority = catini.GetInt(_T("Priority"),0,ixStr);
		_tcscpy(buffer, catini.GetString(_T("Color"),_T("0"),ixStr));
		newcat->m_crColor = _tstoi64(buffer);
		newcat->m_strAutoCatExt = catini.GetString(_T("AutoCat"),_T(""),ixStr);
		newcat->m_eCatID = static_cast<_EnumCategories>(catini.GetInt(_T("CategoryID"),ix,ixStr));
		newcat->m_bIsPredefined = false;

		CCat::AddCat(newcat,false);
		if (!PathFileExists(newcat->m_strSavePath))
			::CreateDirectory(newcat->m_strSavePath,0);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// LANCAST (moosetea) - get the lancast ip address
uint32 CPreferences::GetLancastIP()
{
	if (GetLancastUseDefaultIP())
		return m_DefaultLancastIP;
	else
		return m_LancastIP;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
WORD CPreferences::GetWindowsVersion()
{
	if (!bWinVerAlreadyDetected)
	{	
		bWinVerAlreadyDetected = true;

		BOOL		bIsNTNBased	= FALSE;

		m_wWinVer = DetectWinVersion(bIsNTNBased);	
		m_bIsNTBased = bIsNTNBased; // <== Hrm. May have misread the member name when I renamed it. MOREVIT
	}	
	return m_wWinVer;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetDefaultMaxConPerFiveSecs() picks a safe value for the maximum number of connections in a 5 second
//		period based on the OS version.
uint16 CPreferences::GetDefaultMaxConPerFiveSecs()
{
	switch (GetWindowsVersion())
	{
		case _WINVER_98_: /*= _WINVER_NT4_*/
			if ( m_bIsNTBased )
				return 10;
			return 5;
		case _WINVER_95_:	
			return 5;
		case _WINVER_ME_:
			return 10;
		case _WINVER_2K_:
			return 20;
		case _WINVER_XP_:
			return 20;
		case _WINVER_SE_:	//Win Server 2003
			return 20;
		default:
			return 10;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Barry - Provide a mechanism for all tables to store/retrieve sort order
int CPreferences::GetColumnSortItem(EnumTable t) const
{
	switch (t) 
	{
		case TABLE_DOWNLOAD:
			return m_prefs->tableSortItemDownload;
		case TABLE_DOWNLOAD2:
			return m_prefs->tableSortItemDownload2;
		case TABLE_UPLOAD:
			return m_prefs->tableSortItemUpload;
		case TABLE_QUEUE:
			return m_prefs->tableSortItemQueue;
		case TABLE_SEARCH:
			return m_prefs->tableSortItemSearch;
		case TABLE_SHARED:
			return m_prefs->tableSortItemShared;
		case TABLE_SERVER:
			return m_prefs->tableSortItemServer;
		case TABLE_CLIENTLIST:
			return m_prefs->tableSortItemClientList;
	}
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Barry - Provide a mechanism for all tables to store/retrieve sort order
bool CPreferences::GetColumnSortAscending(EnumTable t) const
{
	switch (t) 
	{
		case TABLE_DOWNLOAD:
			return m_prefs->tableSortAscendingDownload;
		case TABLE_DOWNLOAD2:
			return m_prefs->tableSortAscendingDownload2;
		case TABLE_UPLOAD:
			return m_prefs->tableSortAscendingUpload;
		case TABLE_QUEUE:
			return m_prefs->tableSortAscendingQueue;
		case TABLE_SEARCH:
			return m_prefs->tableSortAscendingSearch;
		case TABLE_SHARED:
			return m_prefs->tableSortAscendingShared;
		case TABLE_SERVER:
			return m_prefs->tableSortAscendingServer;
		case TABLE_CLIENTLIST:
			return m_prefs->tableSortAscendingClientList;
	}
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortItem(EnumTable t, int sortItem)
{
	switch (t) 
	{
		case TABLE_DOWNLOAD:
			m_prefs->tableSortItemDownload = sortItem;
			break;
		case TABLE_DOWNLOAD2:
			m_prefs->tableSortItemDownload2 = sortItem;
			break;
		case TABLE_UPLOAD:
			m_prefs->tableSortItemUpload = sortItem;
			break;
		case TABLE_QUEUE:
			m_prefs->tableSortItemQueue = sortItem;
			break;
		case TABLE_SEARCH:
			m_prefs->tableSortItemSearch = sortItem;
			break;
		case TABLE_SHARED:
			m_prefs->tableSortItemShared = sortItem;
			break;
		case TABLE_SERVER:
			m_prefs->tableSortItemServer = sortItem;
			break;
		case TABLE_CLIENTLIST:
			m_prefs->tableSortItemClientList = sortItem;
			break;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortAscending(EnumTable t, bool sortAscending)
{
	switch(t) 
	{
		case TABLE_DOWNLOAD:
			m_prefs->tableSortAscendingDownload = sortAscending;
			break;
		case TABLE_DOWNLOAD2:
			m_prefs->tableSortAscendingDownload2 = sortAscending;
			break;
		case TABLE_UPLOAD:
			m_prefs->tableSortAscendingUpload = sortAscending;
			break;
		case TABLE_QUEUE:
			m_prefs->tableSortAscendingQueue = sortAscending;
			break;
		case TABLE_SEARCH:
			m_prefs->tableSortAscendingSearch = sortAscending;
			break;
		case TABLE_SHARED:
			m_prefs->tableSortAscendingShared = sortAscending;
			break;
		case TABLE_SERVER:
			m_prefs->tableSortAscendingServer = sortAscending;
			break;
		case TABLE_CLIENTLIST:
			m_prefs->tableSortAscendingClientList = sortAscending;
			break;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SetLanguage()
{
	switch (m_nLanguageID)
	{
		// read from ini and supported by the program?
		case MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "english");break;
		case MAKELANGID(LANG_BASQUE,SUBLANG_DEFAULT):setlocale( LC_ALL, "basque");break;
		case MAKELANGID(LANG_CATALAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "esp");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED):setlocale( LC_ALL, "chs");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL):setlocale( LC_ALL, "cht");break;
		case MAKELANGID(LANG_DANISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "danish");break;
		case MAKELANGID(LANG_DUTCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "dutch");break;
		case MAKELANGID(LANG_FINNISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "finnish");break;
		case MAKELANGID(LANG_FRENCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "french");break;
		case MAKELANGID(LANG_GERMAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "german");break;
		case MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT):setlocale( LC_ALL, "greek");break;
		case MAKELANGID(LANG_HEBREW,SUBLANG_DEFAULT):setlocale( LC_ALL, "hebrew");break;
		case MAKELANGID(LANG_ITALIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "italian");break;
		case MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "korean");break;
		case MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "lithuanian");break;
		case MAKELANGID(LANG_NORWEGIAN,SUBLANG_NORWEGIAN_BOKMAL):setlocale( LC_ALL, "nor");break;
		case MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "polish");break;
		case MAKELANGID(LANG_PORTUGUESE,SUBLANG_PORTUGUESE):setlocale( LC_ALL, "ptg");break;
		case MAKELANGID(LANG_PORTUGUESE,SUBLANG_PORTUGUESE_BRAZILIAN):setlocale( LC_ALL, "ptb");break;
		case MAKELANGID(LANG_ROMANIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "romanian");break;
		case MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "russian");break;
		case MAKELANGID(LANG_SPANISH,SUBLANG_SPANISH):setlocale( LC_ALL, "spanish");break;
        	case MAKELANGID(LANG_TURKISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "turkish");break;
		default:
		{
			uint8 nLocalLang = ::GetThreadLocale();
			
            CString strLng1 = GetResString(IDS_MB_LANGUAGEINFO,MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
			CString strLng2 = _T("");
			switch(PRIMARYLANGID(nLocalLang))
			{
				case	LANG_ENGLISH:
				case    LANG_BASQUE:
				case	LANG_CATALAN:
				case 	LANG_CHINESE:
				case 	LANG_DANISH:
				case 	LANG_DUTCH:
				case	LANG_FINNISH:
				case 	LANG_FRENCH:
				case 	LANG_GERMAN:
				case	LANG_GREEK:
				case	LANG_HEBREW:
				case	LANG_ITALIAN:
				case 	LANG_KOREAN:
				case	LANG_LITHUANIAN:
				case	LANG_NORWEGIAN:
				case 	LANG_POLISH:
				case 	LANG_PORTUGUESE:
				case 	LANG_RUSSIAN:
				case    LANG_ROMANIAN:
				case 	LANG_SPANISH:
				case    LANG_TURKISH:
				{
					m_nLanguageID = MAKELANGID(PRIMARYLANGID(nLocalLang),SUBLANG_DEFAULT);
					strLng1 = GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID);//Cax2 fix
					break;
				}
				default:
					LANGID langid = LANGIDFROMLCID(::GetThreadLocale());
					m_nLanguageID = MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT);
			}
			if ( m_bIsNTBased )
				SetThreadLocale(m_nLanguageID); // THIS FUNCTION DOEN NOT WORK ON WIN9X !!!!!!!!1
			else
				ASSERT(false); // locale NOT set because we just can't do it on Win9x!!!!

			CString strMessage = strLng1 + CString("\n\n") + strLng2;;
			MessageBox(0, strMessage.GetBuffer(),_T("Language"),64);
			
			// DonGato
			CString strQuestion;
			if (nLocalLang == LANG_SPANISH)
				strQuestion = CString("\xbf") + GetResString(IDS_MULTIPLE, m_nLanguageID) + CString("?");
			else
				strQuestion = GetResString(IDS_MULTIPLE, m_nLanguageID) + CString("?");
			
			if (IDNO==MessageBox(0, strQuestion.GetBuffer(),_T("eMule Plus"),MB_YESNO | MB_DEFBUTTON2))	//SyruS make "no" the default for users
				return;
			else
				m_bMultiple = true;
		}
	}
	if ( m_bIsNTBased )
		SetThreadLocale(m_nLanguageID);

	// if loading a string fails, set language to English
	if (GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID).GetLength()==0)
	{
		m_nLanguageID = MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT);
		if ( g_eMuleApp.m_pGlobPrefs->IsNTBased() )
			SetThreadLocale(m_nLanguageID);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SetMainProcessPriority(int Value) 			
{
	m_nMainProcessPriority = Value;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CPreferences::IsNTBased(void)
{
	GetWindowsVersion();
	return (m_bIsNTBased);
}

void CPreferences::ResetStatistics()
{

	// Save a backup so that we can undo this action
	SaveStats(1);

	// SET ALL CUMULATIVE STAT VALUES TO 0  :'-(

	m_nTotalDownloadedBytes = 0;
	m_nTotalUploadedBytes = 0;
	cumDownOverheadTotal=0;
	cumDownOverheadFileReq=0;
	cumDownOverheadSrcEx=0;
	cumDownOverheadServer=0;
	cumDownOverheadOther=0;
	cumDownOverheadTotalPackets=0;
	cumDownOverheadFileReqPackets=0;
	cumDownOverheadSrcExPackets=0;
	cumDownOverheadServerPackets=0;
	cumDownOverheadOtherPackets=0;
	cumUpOverheadTotal=0;
	cumUpOverheadFileReq=0;
	cumUpOverheadSrcEx=0;
	cumUpOverheadServer=0;
	cumUpOverheadOther=0;
	cumUpOverheadTotalPackets=0;
	cumUpOverheadFileReqPackets=0;
	cumUpOverheadSrcExPackets=0;
	cumUpOverheadServerPackets=0;
	cumUpOverheadOtherPackets=0;
	cumUpSuccessfulSessions=0;
	cumUpFailedSessions=0;
	cumUpAvgTime=0;
	cumUpDataPort_4662=0;
	cumUpDataPort_OTHER=0;
	cumDownCompletedFiles=0;
	cumDownSuccessfulSessions=0;
	cumDownFailedSessions=0;
	cumDownAvgTime=0;
	cumLostFromCorruption=0;
	cumSavedFromCompression=0;
	cumPartsSavedByICH=0;
	for (int i = 0; i<SO_LAST; i++)
	{
		cumUpDataClients[i] = 0;
		cumDownDataClients[i] = 0;
	}
	cumDownDataPort_4662=0;
	cumDownDataPort_OTHER=0;
	cumConnAvgDownRate=0;
	cumConnMaxAvgDownRate=0;
	cumConnMaxDownRate=0;
	cumConnAvgUpRate=0;
	cumConnRunTime=0;
	cumConnNumReconnects=0;
	cumConnAvgConnections=0;
	cumConnMaxConnLimitReached=0;
	cumConnPeakConnections=0;
	cumConnDownloadTime=0;
	cumConnUploadTime=0;
	cumConnTransferTime=0;
	cumConnServerDuration=0;
	cumConnMaxAvgUpRate=0;
	cumConnMaxUpRate=0;
	cumSrvrsMostWorkingServers=0;
	cumSrvrsMostUsersOnline=0;
	cumSrvrsMostFilesAvail=0;
	cumSharedMostFilesShared=0;
	cumSharedLargestShareSize=0;
	cumSharedLargestAvgFileSize=0;
	cumSharedLargestFileSize=0;
	cumUpData_File=0;
	cumUpData_Partfile=0;
		
	// Set the time of last reset...
	time_t	timeNow;time(&timeNow);stat_datetimeLastReset = (__int64) timeNow;

	// Save the reset stats
	SaveStats();

	// End Reset Statistics
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Load Statistics
// This used to be integrated in LoadPreferences, but it has been altered
// so that it can be used to load the backup created when the stats are reset.
// Last Modified: 2-22-03 by Khaos
bool CPreferences::LoadStats(int loadBackUp, CString sFileName)
{
	EMULE_TRY
	
	// loadBackUp is 0 by default
	// loadBackUp = 0: Load the stats normally like we used to do in LoadPreferences
	// loadBackUp = 1: Load the stats from statbkup.ini and create a backup of the current stats.  Also, do not initialize session variables.
	// loadBackUp = 2: Load the stats from preferences.ini.old because the version has changed.
	CString fullpath; // i_a
	CFileFind findBackUp;
	bool alreadySaved = false;

	switch (loadBackUp)
	{
		case 0:
			fullpath.Format(_T("%spreferences.ini"),m_strConfigDir);
			break;
		case 1:
			fullpath.Format(_T("%sstatbkup.ini"),m_strConfigDir);
			if (!findBackUp.FindFile(fullpath))	return false;
			SaveStats(2); // Save our temp backup of current values to statbkuptmp.ini, we will be renaming it at the end of this function.
			break;
		case 2:
			if (sFileName.IsEmpty()) return false;
			fullpath = sFileName;
			if (!findBackUp.FindFile(fullpath))	return false;
			break;
	}

	CIni	ini(fullpath);

	ini.SetDefaultCategory(_T("Statistics"));

	m_nTotalDownloadedBytes.Put(ini.GetUInt64(_T("TotalDownloadedBytes"), 0));
	m_nTotalUploadedBytes.Put(ini.GetUInt64(_T("TotalUploadedBytes"), 0));

	// Load stats for cumulative downline overhead
	cumDownOverheadTotal = ini.GetUInt64(_T("DownOverheadTotal"),0);
	cumDownOverheadFileReq = ini.GetUInt64(_T("DownOverheadFileReq"),0);
	cumDownOverheadSrcEx = ini.GetUInt64(_T("DownOverheadSrcEx"),0);
	cumDownOverheadServer=ini.GetUInt64(_T("DownOverheadServer"),0);
	cumDownOverheadOther=ini.GetUInt64(_T("DownOverheadOther"),0);
	cumDownOverheadTotalPackets=ini.GetUInt64(_T("DownOverheadTotalPackets"),0);
	cumDownOverheadFileReqPackets=	ini.GetUInt64(_T("DownOverheadFileReqPackets"),0);
	cumDownOverheadSrcExPackets=ini.GetUInt64(_T("DownOverheadSrcExPackets"),0);
	cumDownOverheadServerPackets=ini.GetUInt64(_T("DownOverheadServerPackets"),0);
	cumDownOverheadOtherPackets=ini.GetUInt64(_T("DownOverheadOtherPackets"),0);

	// Load stats for cumulative upline overhead
	cumUpOverheadTotal = ini.GetUInt64(_T("UpOverheadTotal"),0);  // BavarianSnail - spelling corrected
	cumUpOverheadFileReq = ini.GetUInt64(_T("UpOverheadFileReq"),0);
	cumUpOverheadSrcEx = ini.GetUInt64(_T("UpOverheadSrcEx"),0);
	cumUpOverheadServer = ini.GetUInt64(_T("UpOverheadServer"),0);
	cumUpOverheadOther = ini.GetUInt64(_T("UpOverheadOther"),0);
	cumUpOverheadTotalPackets = ini.GetUInt64(_T("UpOverheadTotalPackets"),0); // BavarianSnail - spelling corrected
	cumUpOverheadFileReqPackets = ini.GetUInt64(_T("UpOverheadFileReqPackets"),0);
	cumUpOverheadSrcExPackets = ini.GetUInt64(_T("UpOverheadSrcExPackets"),0);
	cumUpOverheadServerPackets = ini.GetUInt64(_T("UpOverheadServerPackets"),0);
	cumUpOverheadOtherPackets = ini.GetUInt64(_T("UpOverheadOtherPackets"),0);

	// Load stats for cumulative upline data
	cumUpSuccessfulSessions =	ini.GetUInt32(_T("UpSuccessfulSessions"), 0 );
	cumUpFailedSessions =		ini.GetUInt32(_T("UpFailedSessions"), 0 );
	cumUpAvgTime =			ini.GetUInt32(_T("UpAvgTime"), 0 );

	// Load cumulative client breakdown stats for sent bytes
	cumUpDataClients[SO_EDONKEY] = ini.GetUInt64(_T("UpData_EDONKEY"),0);
	cumUpDataClients[SO_EDONKEYHYBRID] = ini.GetUInt64(_T("UpData_EDONKEYHYBRID"),0);
	cumUpDataClients[SO_EMULE] = ini.GetUInt64(_T("UpData_EMULE"),0);
	cumUpDataClients[SO_MLDONKEY] = ini.GetUInt64(_T("UpData_MLDONKEY"),0);
	cumUpDataClients[SO_LMULE] = ini.GetUInt64(_T("UpData_LMULE"),0);
	cumUpDataClients[SO_CDONKEY] = ini.GetUInt64(_T("UpData_CDONKEY"),0);
	cumUpDataClients[SO_OLDEMULE] = 0;	// eklmn: initialize unused
	cumUpDataClients[SO_SHAREAZA] = ini.GetUInt64(_T("UpData_SHAREAZA"),0);
	cumUpDataClients[SO_UNKNOWN] = 0;	// eklmn: initialize unused

	// Load cumulative port breakdown stats for sent bytes
	cumUpDataPort_4662 =	ini.GetUInt64(_T("UpDataPort_4662"),0);
	cumUpDataPort_OTHER = ini.GetUInt64(_T("UpDataPort_OTHER"),0);

	// Load cumulative source breakdown stats for sent bytes
	cumUpData_File = ini.GetUInt64(_T("UpData_File"),0);
	cumUpData_Partfile = ini.GetUInt64(_T("UpData_Partfile"),0);

	// Load stats for cumulative downline data
	cumDownCompletedFiles = ini.GetUInt32(_T("DownCompletedFiles"), 0 );
	cumDownSuccessfulSessions = ini.GetInt(_T("DownSuccessfulSessions"), 0 );
	cumDownFailedSessions = ini.GetInt(_T("DownFailedSessions"), 0 );
	cumDownAvgTime = ini.GetUInt32(_T("DownAvgTime"), 0 );

	// Cumulative statistics for saved due to compression/lost due to corruption
	cumLostFromCorruption = ini.GetUInt64(_T("LostFromCorruption"),0);
	cumSavedFromCompression = ini.GetUInt64(_T("SavedFromCompression"),0);
	cumPartsSavedByICH = ini.GetUInt32(_T("PartsSavedByICH"), 0 );

	// Load cumulative client breakdown stats for received bytes
	cumDownDataClients[SO_EDONKEY] = ini.GetUInt64(_T("DownData_EDONKEY"),0);
	cumDownDataClients[SO_EDONKEYHYBRID] = ini.GetUInt64(_T("DownData_EDONKEYHYBRID"),0);
	cumDownDataClients[SO_EMULE] = ini.GetUInt64(_T("DownData_EMULE"),0);
	cumDownDataClients[SO_MLDONKEY] = ini.GetUInt64(_T("DownData_MLDONKEY"),0);
	cumDownDataClients[SO_LMULE] = ini.GetUInt64(_T("DownData_LMULE"),0);
	cumDownDataClients[SO_CDONKEY] = ini.GetUInt64(_T("DownData_CDONKEY"),0);
	cumDownDataClients[SO_OLDEMULE] = 0;	// eklmn: initialize unused
	cumDownDataClients[SO_SHAREAZA] = ini.GetUInt64(_T("DownData_SHAREAZA"),0);
	cumDownDataClients[SO_UNKNOWN] = 0;

	// Load cumulative port breakdown stats for received bytes
	cumDownDataPort_4662 =	ini.GetUInt64(_T("DownDataPort_4662"),0);
	cumDownDataPort_OTHER = ini.GetUInt64(_T("DownDataPort_OTHER"),0);

	// Load stats for cumulative connection data
	cumConnAvgDownRate = ini.GetFloat(_T("ConnAvgDownRate"), 0 );
	cumConnMaxAvgDownRate = ini.GetFloat(_T("ConnMaxAvgDownRate"), 0 );
	cumConnMaxDownRate = ini.GetFloat(_T("ConnMaxDownRate"), 0 );
	cumConnAvgUpRate = ini.GetFloat(_T("ConnAvgUpRate"), 0 );
	cumConnMaxAvgUpRate = ini.GetFloat(_T("ConnMaxAvgUpRate"), 0 );
	cumConnMaxUpRate = ini.GetFloat(_T("ConnMaxUpRate"), 0 );

	cumConnRunTime = ini.GetUInt64(_T("ConnRunTime"), 0 );

	cumConnTransferTime=			ini.GetUInt32(_T("ConnTransferTime"), 0 );
	cumConnDownloadTime=		ini.GetUInt32(_T("ConnDownloadTime"), 0 );
	cumConnUploadTime=			ini.GetUInt32(_T("ConnUploadTime"), 0 );
	cumConnServerDuration=		ini.GetUInt32(_T("ConnServerDuration"), 0 );
	cumConnNumReconnects =		ini.GetInt(_T("ConnNumReconnects"), 0 );
	cumConnAvgConnections =		ini.GetInt(_T("ConnAvgConnections"), 0 );
	cumConnMaxConnLimitReached=	ini.GetInt(_T("ConnMaxConnLimitReached"), 0 );
	cumConnPeakConnections =		ini.GetInt(_T("ConnPeakConnections"), 0 );

	// Load date/time of last reset
	stat_datetimeLastReset = ini.GetUInt64(_T("statsDateTimeLastReset"),0);

	// Smart Load For Restores - Don't overwrite records that are greater than the backed up ones
	if (loadBackUp == 1) 
	{
		uint64 temp64;
		// Load records for servers / network
		if (ini.GetInt(_T("SrvrsMostWorkingServers"), 0) > cumSrvrsMostWorkingServers)
			cumSrvrsMostWorkingServers = ini.GetInt(_T("SrvrsMostWorkingServers"), 0 );
		if (ini.GetUInt32(_T("SrvrsMostUsersOnline"), 0) > cumSrvrsMostUsersOnline)
			cumSrvrsMostUsersOnline =	ini.GetUInt32(_T("SrvrsMostUsersOnline"), 0 );
		if (ini.GetUInt32(_T("SrvrsMostFilesAvail"), 0) > cumSrvrsMostFilesAvail)
			cumSrvrsMostFilesAvail = ini.GetUInt32(_T("SrvrsMostFilesAvail"), 0 );

		// Load records for shared files
		if (ini.GetInt(_T("SharedMostFilesShared "), 0) > cumSharedMostFilesShared)
			cumSharedMostFilesShared = ini.GetInt(_T("SharedMostFilesShared"), 0 );

		temp64 = ini.GetUInt64(_T("SharedLargestShareSize"),0);
		if (temp64 > cumSharedLargestShareSize) cumSharedLargestShareSize = temp64;
		temp64 = ini.GetUInt64(_T("SharedLargestAvgFileSize"),0);
		if (temp64 > cumSharedLargestAvgFileSize) cumSharedLargestAvgFileSize = temp64;
		temp64 = ini.GetUInt64(_T("SharedLargestFileSize"),0);
		if (temp64 > cumSharedLargestFileSize) cumSharedLargestFileSize = temp64;

		// Check to make sure the backup of the values we just overwrote exists.  If so, rename it to the backup file.
		// This allows us to undo a restore, so to speak, just in case we don't like the restored values...
		
		
		CString sINIBackUp;
		sINIBackUp.Format(_T("%sstatbkuptmp.ini"), m_strConfigDir);
		if (findBackUp.FindFile(sINIBackUp))
		{
			ini.SaveAndClose();
			CFile::Remove(fullpath); // Remove the backup that we just restored from
			CFile::Rename(sINIBackUp, fullpath); // Rename our temporary backup to the normal statbkup.ini filename.
			alreadySaved = true;
		}

		// Since we know this is a restore, now we should call ShowStatistics to update the data items to the new ones we just loaded.
		// Otherwise user is left waiting around for the tick counter to reach the next automatic update (Depending on setting in prefs)
		g_eMuleApp.m_pdlgEmule->m_dlgStatistics.ShowStatistics();
		
	}
	// Stupid Load -> Just load the values.
	else 
	{
		// Load records for servers / network
		cumSrvrsMostWorkingServers = ini.GetInt(_T("SrvrsMostWorkingServers"), 0 );
		cumSrvrsMostUsersOnline =	ini.GetUInt32(_T("SrvrsMostUsersOnline"), 0 );
		cumSrvrsMostFilesAvail = ini.GetUInt32(_T("SrvrsMostFilesAvail"), 0 );

		// Load records for shared files
		cumSharedMostFilesShared = ini.GetInt(_T("SharedMostFilesShared"), 0 );

		cumSharedLargestShareSize = ini.GetUInt64(_T("SharedLargestShareSize"),0);
		cumSharedLargestAvgFileSize = ini.GetUInt64(_T("SharedLargestAvgFileSize"),0);
		cumSharedLargestFileSize =	ini.GetUInt64(_T("SharedLargestFileSize"),0);

		// These are not stats, but they're part of my mod, so we will load them here anyway.
		statsExpandedTreeItems = ini.GetString(_T("statsExpandedTreeItems"), "111000000100000110000010000011110000010010");

		// Initialize new session statistic variables...
		sesDownCompletedFiles =		0;
		for (int i = 0; i<SO_LAST;i++)
		{
			sesDownDataClients[(EnumClientTypes)i] = 0;
			sesUpDataClients[(EnumClientTypes)i] = 0;
		}
		sesUpData_File = 				0;
		sesUpData_Partfile = 			0;
		sesUpDataPort_4662 =			0;
		sesUpDataPort_OTHER =		0;
		sesDownDataPort_4662 =		0;
		sesDownDataPort_OTHER =		0;
		sesDownSuccessfulSessions=	0;
		sesDownFailedSessions=		0;
		sesPartsSavedByICH=			0;
	}

	if(stat_datetimeLastReset == 0)
	{
		time_t timeNow;
		time(&timeNow);
		stat_datetimeLastReset = (__int64) timeNow;
	}

	// End Load Stats
	if(!alreadySaved)
		ini.CloseWithoutSave();

	return true;

	EMULE_CATCH
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This formats the UCT long value that is saved for stat_datetimeLastReset
// If this value is 0 (Never reset), then it returns Unknown.
CString CPreferences::GetStatsLastResetStr()
{
	CString	returnStr;

	if (GetStatsLastResetLng()) 
	{
		// DonGato: fixed localization of Last reset
		time_t	lastResetDateTime = (time_t) GetStatsLastResetLng();
		COleDateTime statsResetLocalized(lastResetDateTime);
		returnStr.Format("%s", statsResetLocalized.Format());
	}
	else 
		returnStr = GetResString(IDS_UNKNOWN);

	return returnStr;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// -khaos--+++> A whole bunch of methods!  Keep going until you reach the end tag.
void CPreferences::SaveStats(int bBackUp)
{
	// This function saves all of the new statistics in my addon.  It is also used to
	// save backups for the Reset Stats function, and the Restore Stats function (Which is actually LoadStats)
	// bBackUp = 0: DEFAULT; save to preferences.ini
	// bBackUp = 1: Save to statbkup.ini, which is used to restore after a reset
	// bBackUp = 2: Save to statbkuptmp.ini, which is temporarily created during a restore and then renamed to statbkup.ini

	CString fullpath; // i_a

	switch (bBackUp)
	{
		case 0:
			fullpath.Format(_T("%spreferences.ini"),m_strConfigDir);
			break;
		case 1:
			fullpath.Format(_T("%sstatbkup.ini"),m_strConfigDir);
			break;
		case 2: 
			fullpath.Format(_T("%sstatbkuptmp.ini"),m_strConfigDir);
			break;
	}

	CIni ini(fullpath);
	ini.SetDefaultCategory(_T("Statistics"));

	// Save cumulative statistics to preferences.ini, going in order as they appear in CStatisticsDlg::ShowStatistics.
	// We do NOT SET the values in prefs struct here.

	// Save Cum Down Data
	ini.SetUInt64(_T("TotalDownloadedBytes"), g_eMuleApp.stat_sessionReceivedBytes+GetTotalDownloaded());

	// Save Complete Downloads - This is saved and incremented in partfile.cpp.
	ini.SetInt(_T("DownCompletedFiles"), cumDownCompletedFiles);

	// Save Successful Download Sessions
	ini.SetInt(_T("DownSuccessfulSessions"), cumDownSuccessfulSessions);

	// Save Failed Download Sessions
	ini.SetInt(_T("DownFailedSessions"), cumDownFailedSessions);
	ini.SetInt(_T("DownAvgTime"), (GetDownC_AvgTime() + GetDownS_AvgTime()) / 2 );

	// Cumulative statistics for saved due to compression/lost due to corruption
	ini.SetUInt64(_T("LostFromCorruption"), cumLostFromCorruption );
	ini.SetUInt64(_T("SavedFromCompression"), cumSavedFromCompression);
	ini.SetInt(_T("PartsSavedByICH"), cumPartsSavedByICH+sesPartsSavedByICH);

	// Save cumulative client breakdown stats for received bytes...
	ini.SetUInt64(_T("DownData_EDONKEY"), GetCumDownData(SO_EDONKEY));
	ini.SetUInt64(_T("DownData_EDONKEYHYBRID"), GetCumDownData(SO_EDONKEYHYBRID));
	ini.SetUInt64(_T("DownData_EMULE"), GetCumDownData(SO_EMULE) );
	ini.SetUInt64(_T("DownData_MLDONKEY"), GetCumDownData(SO_MLDONKEY) );
	ini.SetUInt64(_T("DownData_LMULE"), GetCumDownData(SO_LMULE) );
	ini.SetUInt64(_T("DownData_CDONKEY"), GetCumDownData(SO_CDONKEY) );
	ini.SetUInt64(_T("DownData_SHAREAZA"), GetCumDownData(SO_SHAREAZA) );

	// Save cumulative port breakdown stats for received bytes
	ini.SetUInt64(_T("DownDataPort_4662"), GetCumDownDataPort_4662());
	ini.SetUInt64(_T("DownDataPort_OTHER"), GetCumDownDataPort_OTHER());

	// Save Cumulative Downline Overhead Statistics
	if (g_eMuleApp.m_pDownloadQueue)
	{
		ini.SetUInt64(_T("DownOverheadTotal"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadFileRequest() + g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadSourceExchange() + g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadServer() + g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadOther() + GetDownOverheadTotal() );
		ini.SetUInt64(_T("DownOverheadFileReq"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadFileRequest() + GetDownOverheadFileReq() );
		ini.SetUInt64(_T("DownOverheadSrcEx"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadSourceExchange() + GetDownOverheadSrcEx() );
		ini.SetUInt64(_T("DownOverheadServer"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadServer() + GetDownOverheadServer() );
		ini.SetUInt64(_T("DownOverheadOther"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadOther() + GetDownOverheadOther() );
		// BavarianSnail - corrected saving of overhead values
		ini.SetUInt64(_T("DownOverheadTotalPackets"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadFileRequestPackets() + g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadSourceExchangePackets() + g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadServerPackets() + g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadOtherPackets() + GetDownOverheadTotalPackets() );
		ini.SetUInt64(_T("DownOverheadFileReqPackets"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadFileRequestPackets() + GetDownOverheadFileReqPackets() );
		ini.SetUInt64(_T("DownOverheadSrcExPackets"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadSourceExchangePackets() + GetDownOverheadSrcExPackets() );
		ini.SetUInt64(_T("DownOverheadServerPackets"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadServerPackets() + GetDownOverheadServerPackets() );
		ini.SetUInt64(_T("DownOverheadOtherPackets"), (uint64)g_eMuleApp.m_pDownloadQueue->GetDownDataOverheadOtherPackets() + GetDownOverheadOtherPackets() );
	}

	// Save Cumulative Upline Statistics
	ini.SetUInt64(_T("TotalUploadedBytes"), g_eMuleApp.stat_sessionSentBytes+GetTotalUploaded() );
	if (g_eMuleApp.m_pUploadQueue)
	{
		ini.SetInt(_T("UpSuccessfulSessions"), g_eMuleApp.m_pUploadQueue->GetSuccessfulUpCount()+GetUpSuccessfulSessions());
		ini.SetInt(_T("UpFailedSessions"), g_eMuleApp.m_pUploadQueue->GetFailedUpCount()+GetUpFailedSessions());
		ini.SetInt(_T("UpAvgTime"), (g_eMuleApp.m_pUploadQueue->GetAverageUpTime()+GetUpAvgTime())/2);

		// Save Cumulative Upline Overhead Statistics
		ini.SetUInt64(_T("UpOverheadTotal"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadFileRequest() + g_eMuleApp.m_pUploadQueue->GetUpDataOverheadSourceExchange() + g_eMuleApp.m_pUploadQueue->GetUpDataOverheadServer() + g_eMuleApp.m_pUploadQueue->GetUpDataOverheadOther() + GetUpOverheadTotal() );
		ini.SetUInt64(_T("UpOverheadFileReq"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadFileRequest() + GetUpOverheadFileReq() );
		ini.SetUInt64(_T("UpOverheadSrcEx"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadSourceExchange() + GetUpOverheadSrcEx() );
		ini.SetUInt64(_T("UpOverheadServer"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadServer() + GetUpOverheadServer() );
		ini.SetUInt64(_T("UpOverheadOther"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadOther() + GetUpOverheadOther() );
		ini.SetUInt64(_T("UpOverheadTotalPackets"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadFileRequestPackets() + g_eMuleApp.m_pUploadQueue->GetUpDataOverheadSourceExchangePackets() + g_eMuleApp.m_pUploadQueue->GetUpDataOverheadServerPackets() + g_eMuleApp.m_pUploadQueue->GetUpDataOverheadOtherPackets() + GetUpOverheadTotalPackets() );
		ini.SetUInt64(_T("UpOverheadFileReqPackets"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadFileRequestPackets() + GetUpOverheadFileReqPackets() );
		ini.SetUInt64(_T("UpOverheadSrcExPackets"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadSourceExchangePackets() + GetUpOverheadSrcExPackets() );
		ini.SetUInt64(_T("UpOverheadServerPackets"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadServerPackets() + GetUpOverheadServerPackets() );
		ini.SetUInt64(_T("UpOverheadOtherPackets"), (uint64)g_eMuleApp.m_pUploadQueue->GetUpDataOverheadOtherPackets() + GetUpOverheadOtherPackets() );
	}

	// Save Cumulative Client Breakdown Stats For Sent Bytes
	ini.SetUInt64(_T("UpData_EDONKEY"), GetCumUpData(SO_EDONKEY));
	ini.SetUInt64(_T("UpData_EDONKEYHYBRID"), GetCumUpData(SO_EDONKEYHYBRID) );
	ini.SetUInt64(_T("UpData_EMULE"), GetCumUpData(SO_EMULE) );
	ini.SetUInt64(_T("UpData_MLDONKEY"), GetCumUpData(SO_MLDONKEY) );
	ini.SetUInt64(_T("UpData_LMULE"), GetCumUpData(SO_LMULE) );
	ini.SetUInt64(_T("UpData_CDONKEY"), GetCumUpData(SO_CDONKEY));
	ini.SetUInt64(_T("UpData_SHAREAZA"), GetCumUpData(SO_SHAREAZA) );

	// Save cumulative port breakdown stats for sent bytes
	ini.SetUInt64(_T("UpDataPort_4662"), GetCumUpDataPort_4662() );
	ini.SetUInt64(_T("UpDataPort_OTHER"), GetCumUpDataPort_OTHER() );

	// Save cumulative source breakdown stats for sent bytes
	ini.SetUInt64(_T("UpData_File"), GetCumUpData_File() );
	ini.SetUInt64(_T("UpData_Partfile"), GetCumUpData_Partfile() );

	// Save Cumulative Connection Statistics
	double tempRate = 0.0;
	if(g_eMuleApp.m_pdlgEmule)
	{
		if (g_eMuleApp.m_pdlgEmule->m_dlgStatistics)
		{
			// Download Rate Average
			tempRate = g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetAvgDownloadRate(AVG_TOTAL);
			ini.SetFloat(_T("ConnAvgDownRate"), tempRate);
			// Max Download Rate Average
			if (tempRate>GetConnMaxAvgDownRate()) Add2ConnMaxAvgDownRate(tempRate);
			ini.SetFloat(_T("ConnMaxAvgDownRate"), GetConnMaxAvgDownRate());
			// Upload Rate Average
			tempRate = g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetAvgUploadRate(AVG_TOTAL);
			ini.SetFloat(_T("ConnAvgUpRate"), tempRate);
			// Max Upload Rate Average
			if (tempRate>GetConnMaxAvgUpRate()) Add2ConnMaxAvgUpRate(tempRate);
			ini.SetFloat(_T("ConnMaxAvgUpRate"), GetConnMaxAvgUpRate());
		}
	}
	if (g_eMuleApp.m_pDownloadQueue)
	{
		// Max Download Rate
		tempRate = static_cast<double>(g_eMuleApp.m_pDownloadQueue->GetDataRate()) / 1024.0;
		if (tempRate>GetConnMaxDownRate()) Add2ConnMaxDownRate(tempRate);
		ini.SetFloat(_T("ConnMaxDownRate"), GetConnMaxDownRate());
	}
	// Max Upload Rate
	if (g_eMuleApp.m_pUploadQueue)
	{
		tempRate = static_cast<double>(g_eMuleApp.m_pUploadQueue->GetDataRate()) / 1024.0;
		if (tempRate>GetConnMaxUpRate()) Add2ConnMaxUpRate(tempRate);
		ini.SetFloat(_T("ConnMaxUpRate"), GetConnMaxUpRate());	
	}
	
	// Overall Run Time
	uint64 timeseconds = (uint64)(::GetTickCount()-g_eMuleApp.stat_starttime)/1000;
	timeseconds += GetConnRunTime();
	ini.SetUInt64(_T("ConnRunTime"),timeseconds );
	// Number of Reconnects
	if (g_eMuleApp.stat_reconnects>0) 
		ini.SetWORD(_T("ConnNumReconnects"), g_eMuleApp.stat_reconnects - 1 + GetConnNumReconnects());
	else
		ini.SetWORD(_T("ConnNumReconnects"), GetConnNumReconnects());

#ifdef OLD_SOCKETS_ENABLED
	if (g_eMuleApp.m_pServerConnect)
	{
		// Average Connections
		if (g_eMuleApp.m_pServerConnect->IsConnected())
			ini.SetUInt32(_T("ConnAvgConnections"), (uint32)(g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetAverageConnections()+cumConnAvgConnections)/2);
	}
	if (g_eMuleApp.m_pListenSocket)
	{
		// Max Connection Limit Reached
		if (g_eMuleApp.m_pListenSocket->GetMaxConnectionsReachedCount() > 0 ) 
			ini.SetUInt32(_T("ConnMaxConnLimitReached"), g_eMuleApp.m_pListenSocket->GetMaxConnectionsReachedCount() + cumConnMaxConnLimitReached);
	}
#endif //OLD_SOCKETS_ENABLED

	if(g_eMuleApp.m_pdlgEmule)
	{
		if (g_eMuleApp.m_pdlgEmule->m_dlgStatistics)
		{
			// Peak Connections
			if (g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetPeakConnections()>cumConnPeakConnections)
				cumConnPeakConnections = g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetPeakConnections();
			ini.SetInt(_T("ConnPeakConnections"), cumConnPeakConnections);

			// Time Stuff...
			ini.SetInt(_T("ConnTransferTime"), GetConnTransferTime() + g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetTransferTime());
			ini.SetInt(_T("ConnUploadTime"), GetConnUploadTime() + g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetUploadTime());
			ini.SetInt(_T("ConnDownloadTime"), GetConnDownloadTime() + g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetDownloadTime());
			ini.SetInt(_T("ConnServerDuration"), GetConnServerDuration() + g_eMuleApp.m_pdlgEmule->m_dlgStatistics.GetServerDuration());		
		}
	}
	// Compare and Save Server Records
	uint32 servtotal, servfail, servuser, servfile, servtuser, servtfile; 
	double servocc;
	if (g_eMuleApp.m_pServerList)
	{
		g_eMuleApp.m_pServerList->GetStatus( servtotal, servfail, servuser, servfile, servtuser, servtfile, servocc );

		if ((servtotal-servfail)>cumSrvrsMostWorkingServers)	cumSrvrsMostWorkingServers = servtotal-servfail;
		ini.SetInt(_T("SrvrsMostWorkingServers"), cumSrvrsMostWorkingServers);
		if (servtuser>cumSrvrsMostUsersOnline) cumSrvrsMostUsersOnline = servtuser;
		ini.SetInt(_T("SrvrsMostUsersOnline"), cumSrvrsMostUsersOnline);
		if (servtfile>cumSrvrsMostFilesAvail) cumSrvrsMostFilesAvail = servtfile;
		ini.SetInt(_T("SrvrsMostFilesAvail"), cumSrvrsMostFilesAvail);
	}

	if (g_eMuleApp.m_pSharedFilesList)
	{
		// Compare and Save Shared File Records
		if (g_eMuleApp.m_pSharedFilesList->GetCount()>cumSharedMostFilesShared)	
			cumSharedMostFilesShared = g_eMuleApp.m_pSharedFilesList->GetCount();
		ini.SetInt(_T("SharedMostFilesShared"), cumSharedMostFilesShared);
		uint64 bytesLargestFile = 0;
		uint64 allsize=g_eMuleApp.m_pSharedFilesList->GetDatasize(bytesLargestFile);
		if (allsize>cumSharedLargestShareSize) cumSharedLargestShareSize = allsize;
		ini.SetUInt64(_T("SharedLargestShareSize"), cumSharedLargestShareSize);
		if (bytesLargestFile>cumSharedLargestFileSize) cumSharedLargestFileSize = bytesLargestFile;
		ini.SetUInt64(_T("SharedLargestFileSize"), cumSharedLargestFileSize);
		if (g_eMuleApp.m_pSharedFilesList->GetCount() != 0)
		{
			uint64 tempint = allsize/g_eMuleApp.m_pSharedFilesList->GetCount();
			if (tempint>cumSharedLargestAvgFileSize)	cumSharedLargestAvgFileSize = tempint;
		}
		ini.SetUInt64(_T("SharedLargestAvgFileSize"), cumSharedLargestAvgFileSize);

		ini.SetUInt64(_T("statsDateTimeLastReset"), stat_datetimeLastReset);
	}

	// These aren't really statistics, but they're a part of my add-on, so we'll save them here and load them in LoadStats
	ini.SetString(_T("statsExpandedTreeItems"), statsExpandedTreeItems);

	// If we are saving a back-up or a temporary back-up, return now.
	// if (bBackUp != 0) return;

	// by default save chages & close the file
	// if (bBackUp == 0)
		ini.SaveAndClose();

	// End SaveStats
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SetRecordStructMembers()
{
	// The purpose of this function is to be called from CStatisticsDlg::ShowStatistics()
	// This was easier than making a bunch of functions to interface with the record
	// members of the prefs struct from ShowStatistics.

	// This function is going to compare current values with previously saved records, and if
	// the current values are greater, the corresponding member of m_prefs will be updated.
	// We will not write to INI here, because this code is going to be called a lot more often
	// than SaveStats()  - Khaos

	CString buffer;

	// Servers
	uint32 servtotal, servfail, servuser, servfile, servtuser, servtfile; double servocc;
	g_eMuleApp.m_pServerList->GetStatus( servtotal, servfail, servuser, servfile, servtuser, servtfile, servocc );
	if ((servtotal-servfail)>cumSrvrsMostWorkingServers) cumSrvrsMostWorkingServers = (servtotal-servfail);
	if (servtuser>cumSrvrsMostUsersOnline) cumSrvrsMostUsersOnline = servtuser;
	if (servtfile>cumSrvrsMostFilesAvail) cumSrvrsMostFilesAvail = servtfile;

	// Shared Files
	if (g_eMuleApp.m_pSharedFilesList->GetCount()>cumSharedMostFilesShared) cumSharedMostFilesShared = g_eMuleApp.m_pSharedFilesList->GetCount();
	uint64 bytesLargestFile = 0;
	uint64 allsize=g_eMuleApp.m_pSharedFilesList->GetDatasize(bytesLargestFile);
	if (allsize>cumSharedLargestShareSize) cumSharedLargestShareSize = allsize;
	if (bytesLargestFile>cumSharedLargestFileSize) cumSharedLargestFileSize = bytesLargestFile;
	if (g_eMuleApp.m_pSharedFilesList->GetCount() != 0) 
	{
		uint64 tempint = allsize/g_eMuleApp.m_pSharedFilesList->GetCount();
		if (tempint > cumSharedLargestAvgFileSize) cumSharedLargestAvgFileSize = tempint;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::Add2SessionTransferData(uint8 uClientID, uint16 uClientPort, BOOL bFromPF, EnumTransferType eUpDown, uint32 bytes)
{
	//	This function adds the transferred bytes to the appropriate variables,
	//	as well as to the totals for all clients. - Khaos
	//	PARAMETERS:
	//	uClientID - The identifier for which client software sent or received this data, eg SO_EMULE
	//	uClientPort - The remote port of the client that sent or received this data, eg 4662
	//	bFromPF - Applies only to uploads.  True is from partfile, False is from non-partfile.
	//	bUpDown - True is Up, False is Down
	//	bytes - Number of bytes sent by the client.  Subtract header before calling.

	switch (eUpDown)
	{
		case TRANSFERTYPE_UPLOAD:	//Upline Data

			if (uClientID < SO_LAST)
				sesUpDataClients[uClientID] += bytes;
			else
				sesUpDataClients[SO_UNKNOWN] += bytes;

			switch (uClientPort)
			{
				// Update session port breakdown stats for sent bytes...
				case 4662:			
					sesUpDataPort_4662 += bytes;
					break;
				default:
					sesUpDataPort_OTHER+=bytes;
			}

			//	Add to our total for sent bytes...
			g_eMuleApp.UpdateSentBytes(bytes);

			if (bFromPF)
				sesUpData_Partfile+=bytes;
			else						
				sesUpData_File+=bytes;

			break;

		case TRANSFERTYPE_DOWNLOAD: // Downline Data

			if (uClientID < SO_LAST)
				sesDownDataClients[uClientID] += bytes;
			else
				sesDownDataClients[SO_UNKNOWN] += bytes;

			switch (uClientPort)
			{
				// Update session port breakdown stats for received bytes...
				// For now we are only going to break it down by default and non-default.
				// A statistical analysis of all data sent from every single port/domain is
				// beyond the scope of this add-on.
				case 4662:
					sesDownDataPort_4662+=bytes;	
					break;
				default:
					sesDownDataPort_OTHER+=bytes;
			}

			//	Add to our total for received bytes...
			g_eMuleApp.UpdateReceivedBytes(bytes);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CPreferences::SaveCompletedDownloadsStat()
{
	EMULE_TRY

	// This function saves the values for the completed
	// download members to INI.  It is called from
	// CPartfile::PerformFileComplete ...   - Khaos

	CString fullpath; // i_a
	fullpath.Format(_T("%spreferences.ini"),m_strConfigDir);

	CIni ini(fullpath);
	ini.SetDefaultCategory(_T("Statistics"));

	ini.SetUInt32(_T("DownCompletedFiles"), GetDownCompletedFiles());
	ini.SetUInt32(_T("DownSessionCompletedFiles"), GetDownSessionCompletedFiles());

	// ini.CloseWithoutSave();
	ini.SaveAndClose(); // BavarianSnail
	
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// <-----khaos-
bool CPreferences::IsShareableDirectory(const CString& rstrDir) const
{
	if(IsInstallationDirectory(rstrDir))
		return false;

	CString strFullPath;

	if(PathCanonicalize(strFullPath.GetBuffer(MAX_PATH), rstrDir))
		strFullPath.ReleaseBuffer();
	else
		strFullPath = rstrDir;
	
	//	Skip sharing of several special eMule folders
	if(!CompareDirectories(strFullPath, GetTempDir()))			// ".\eMule\temp"
		return false;

	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPreferences::IsInstallationDirectory(const CString& rstrDir) const
{
	CString strFullPath;
	if(PathCanonicalize(strFullPath.GetBuffer(MAX_PATH), rstrDir))
		strFullPath.ReleaseBuffer();
	else
		strFullPath = rstrDir;
	
	// skip sharing of several special eMule folders
	if (!CompareDirectories(strFullPath, GetAppDir()))			// ".\eMule"
		return true;
	if (!CompareDirectories(strFullPath, GetConfigDir()))		// ".\eMule\config"
		return true;
//	if (!CompareDirectories(strFullPath, GetWebServerDir()))	// ".\eMule\WebServer"
//		return true;
//	if (!CompareDirectories(strFullPath, GetLangDir()))			// ".\eMule\lang"
//		return true;

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
