//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "SearchList.h"
#include "QArray.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//SyruS CQArray-Sorting operators for SearchFileStruct
bool operator > (SearchFileStruct &first, SearchFileStruct &second)
{
	return (first.m_strIndex.CompareNoCase(second.m_strIndex) > 0);
}

bool operator < (SearchFileStruct &first, SearchFileStruct &second)
{
	return (first.m_strIndex.CompareNoCase(second.m_strIndex) < 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchFile::CSearchFile(CFile& in_data,uint32 nSearchID,CUpDownClient* Sender,LPCTSTR pszDirectory)
{	// netwolf 20.05.03 enhanced source finding (bluecow)
	EMULE_TRY
	m_nSearchID = nSearchID;
	in_data.Read(&m_fileHash,16);
	in_data.Read(&clientip,4);
	in_data.Read(&clientport,2);
	uint32 tagcount;
	in_data.Read(&tagcount,4);
	for (int i = 0;i != tagcount; i++){
		CFileTag* toadd = new CFileTag(in_data);
		m_tagArray.Add(toadd);
	}

	//m_strFileName = GetStrTagValue(FT_FILENAME);
	SetFileName(GetStrTagValue(FT_FILENAME));
	if (m_strFileName.IsEmpty())
		m_strFileName = _T("-");

	m_lFileSize = GetIntTagValue(FT_FILESIZE);

	if (Sender){		//netwolf 20.05.03 enhanced source finding (bluecow)
		// To be capable of adding the client from which we got that search result
		// as a source, we have to explicitly store that client's data in the search result.
		// NOTE: Do *NOT* store the 'Sender' ptr here! At the time when using the 'toadd'
		// struct, the 'Sender' may have been deleted (just if the user has waited too long before
		// using one of the client's search results) -> Crash!
		m_eType = SFT_CLIENT;

		// Explicitly overwrite the already available client's IP, Port with the already validated UserID and Port.
		// So, we have proper (low)UserID+Server pair.
		*((uint32*)clientip) = Sender->GetUserID();
		clientport = Sender->GetUserPort();

		m_nClientServerIP = Sender->GetServerIP();
		m_nClientServerPort = Sender->GetServerPort();
	}
	else{
		m_eType = SFT_SERVER;
		m_nClientServerIP = 0;
		m_nClientServerPort = 0;
	}									// netwolf END
	m_strDirectory= pszDirectory ? pszDirectory : _T("");

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchFile::~CSearchFile()
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize();i++)
		safe_delete(m_tagArray[i]);
	m_tagArray.RemoveAll();
	m_tagArray.SetSize(0);
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 CSearchFile::GetIntTagValue(uint8 tagname)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++){
		if (m_tagArray[i]->GetSpecialTag() == tagname)
			return m_tagArray[i]->GetIntValue();
	}
	EMULE_CATCH
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CSearchFile::GetStrTagValue(uint8 tagname)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++){
		if (m_tagArray[i]->GetSpecialTag() == tagname)
			return m_tagArray[i]->GetStringValue();			
	}
	EMULE_CATCH
	return _T("");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 CSearchFile::AddSources(uint32 count)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++){
		if (m_tagArray[i]->GetSpecialTag() == FT_SOURCES){
			m_tagArray[i]->SetIntValue(m_tagArray[i]->GetIntValue() + count);
			return m_tagArray[i]->GetIntValue();
		}
	}
	EMULE_CATCH
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 CSearchFile::GetSourceCount()
{
	return GetIntTagValue(FT_SOURCES);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchList::CSearchList()
{
	m_pctlSearchList = NULL;
	m_bMobilMuleSearch = false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchList::~CSearchList()
{
	Clear();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::Clear()
{
	EMULE_TRY

	for (POSITION pos = list.GetHeadPosition(); pos != NULL;)
	{
		delete list.GetNext(pos);
	}
	list.RemoveAll();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::RemoveResults( uint32 nSearchID)
{
	EMULE_TRY
	// this will not delete the item from the window, make sure your code does it if you call this
	ASSERT( m_pctlSearchList );
	POSITION pos1, pos2;
	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		list.GetNext(pos1);
		CSearchFile* cur_file = list.GetAt(pos2);
		if( cur_file->GetSearchID() == nSearchID ){
			list.RemoveAt(pos2);
			delete cur_file;
		}
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::ShowResults( uint32 nSearchID)
{
	EMULE_TRY
	ASSERT( m_pctlSearchList );
	m_pctlSearchList->SetRedraw(false);
	for (POSITION pos = list.GetHeadPosition(); pos !=0;list.GetNext(pos)){
		if( ((CSearchFile*)list.GetAt(pos))->GetSearchID() == nSearchID ){
			m_pctlSearchList->AddResult(list.GetAt(pos));
		}
	}
	m_pctlSearchList->SetRedraw(true);
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::RemoveResults( CSearchFile* todel )
{
	EMULE_TRY

	POSITION remove_pos = list.Find(todel);
	if (remove_pos)
	{
		g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchList.RemoveResult( todel );
		list.RemoveAt(remove_pos);
		delete todel;
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::NewSearch(CSearchListCtrl* in_wnd, CString resTypes, uint16 nSearchID, bool MobilMuleSearch)
{
	EMULE_TRY
	if(in_wnd)
		m_pctlSearchList = in_wnd;

	m_strResultType=resTypes;
	m_dwCurrentSearchCount = nSearchID;
	myHashList="";
    m_bMobilMuleSearch = MobilMuleSearch;
	
	foundFilesCount.SetAt(nSearchID,0);
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::ProcessSearchAnswer(char* in_packet, uint32 size, CUpDownClient* Sender, LPCTSTR pszDirectory, bool bFirstDir)
{
	EMULE_TRY
	CSafeMemFile packet((BYTE*)in_packet,size,0);
	uint32 results;
	packet.Read(&results,4);

	uint32 mySearchID=( (Sender != NULL)? (uint32)Sender : m_dwCurrentSearchCount);

	if (Sender)
	{
		if (bFirstDir)
		{
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.DeleteSearch(mySearchID); // DonGato
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.CreateNewTab(CString(Sender->GetUserName()),mySearchID);
			foundFilesCount.SetAt(mySearchID,0);
		}
	}
	else
	{
		foundFilesCount.SetAt(mySearchID,0);
	}

	for (int i = 0; i != results; i++)
	{
		CSearchFile* toadd = new CSearchFile(packet, mySearchID, Sender, pszDirectory);	// netwolf 20.05.03 enhanced source finding (bluecow)
		AddToList(toadd, (Sender != NULL) );
	}
#ifdef OLD_SOCKETS_ENABLED
#ifdef MOBILE_MULE
	if (m_bMobilMuleSearch)
		g_eMuleApp.m_pMMServer->SearchFinished(false);
#endif //MOBILE_MULE
#endif //OLD_SOCKETS_ENABLED
	m_bMobilMuleSearch = false;
	packet.Close();	
	EMULE_CATCH
	return GetResultCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::ProcessUDPSearchAnswer(char* in_packet, uint32 size)
{
	EMULE_TRY
	CSafeMemFile packet((BYTE*)in_packet,size,0);
	CSearchFile* toadd = new CSearchFile(packet, m_dwCurrentSearchCount);
	AddToList(toadd);
	packet.Close();	
	EMULE_CATCH
	return GetResultCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CSearchList::AddToList(CSearchFile* pAddedFile, bool bClientResponse)
{
	EMULE_TRY

//	Cax2 bugfix: now searching for programs shows archives, TODO searching for users shows anything...

	CString		thisType = pAddedFile->GetFileType(true);

	if (
		(!bClientResponse 
			&& !(thisType==m_strResultType
				|| m_strResultType == GetResString(IDS_SEARCH_ANY)
				||m_strResultType == GetResString(IDS_SEARCH_USER)
				||m_strResultType == GetResString(IDS_SEARCH_DOC)
				||(m_strResultType==GetResString(IDS_SEARCH_PRG) 
					&& thisType==GetResString(IDS_SEARCH_ARC)))
		) 
		|| (g_eMuleApp.m_pdlgEmule->m_dlgSearch.IsDocumentSearch(pAddedFile->GetSearchID()) 
			&& thisType != GetResString(IDS_SEARCH_DOC)) 
		|| (!(pAddedFile->GetIntTagValue(FT_FILESIZE)) 
			|| pAddedFile->GetFileName().IsEmpty())
	   )
	{
		delete pAddedFile;
		return false;
	}
	CString sNotSearch = g_eMuleApp.m_pdlgEmule->m_dlgSearch.GetNotSearch(pAddedFile->GetSearchID());
	if(!sNotSearch.IsEmpty())
	{
		CString sFileName = CString(pAddedFile->GetFileName());
		sFileName.MakeLower();
		if(sFileName.Find(sNotSearch) != -1)
		{
			delete pAddedFile;
			return false;
		}
	}
	for (POSITION pos = list.GetHeadPosition(); pos != NULL; list.GetNext(pos))
	{
		CSearchFile* cur_file = list.GetAt(pos);
		if ( (!md4cmp(pAddedFile->GetFileHash(),cur_file->GetFileHash())) && cur_file->GetSearchID() ==  pAddedFile->GetSearchID()){
			cur_file->AddSources(pAddedFile->GetIntTagValue(FT_SOURCES));
			if (m_pctlSearchList && !m_bMobilMuleSearch)
				m_pctlSearchList->UpdateSources(cur_file);
			delete pAddedFile;
			return true;
		}
	}
	
	if (list.AddTail(pAddedFile)) 
	{	
		uint16 tempValue;
		foundFilesCount.Lookup(pAddedFile->GetSearchID(),tempValue);
		foundFilesCount.SetAt(pAddedFile->GetSearchID(),tempValue+1);
	}
	if (m_pctlSearchList && !m_bMobilMuleSearch)
		m_pctlSearchList->AddResult(pAddedFile);
	return true;
	EMULE_CATCH
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::GetResultCount(uint32 nSearchID)
{
	EMULE_TRY
	uint16 hits = 0;
	for (POSITION pos = list.GetHeadPosition(); pos != NULL; list.GetNext(pos)){
		if( list.GetAt(pos)->GetSearchID() == nSearchID )
			hits += list.GetAt(pos)->GetIntTagValue(FT_SOURCES);
	}
	return hits;
	EMULE_CATCH
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::GetResultCount()
{
	return GetResultCount(m_dwCurrentSearchCount);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CSearchList::GetWebList(CString strLinePattern, int iSortBy, bool bAscending, bool bShowColumn1, bool bShowColumn2, bool bShowColumn3, bool bShowColumn4, bool bShowColumn5) const
{
	EMULE_TRY

#ifdef _DEBUG
	DWORD dwStart = ::GetTickCount();
	DWORD dwSortTime;
#endif

	CSearchFile* pFile;
	SearchFileStruct structFile;
	CQArray<SearchFileStruct, SearchFileStruct> SearchFileArray;

	for (POSITION pos = list.GetHeadPosition(); pos != 0;)
	{
		pFile = list.GetNext(pos);

		if (pFile == NULL || pFile->GetFileSize() == 0 || pFile->GetFileName().IsEmpty())
			continue;

		structFile.m_strFileName = pFile->GetFileName();
		structFile.m_strFileType = GetFileType(structFile.m_strFileName.Right(4).MakeLower());
		structFile.m_strFileHash = EncodeBase16(pFile->GetFileHash(), 16);
		structFile.m_uSourceCount = pFile->GetSourceCount();
		structFile.m_uFileSize = pFile->GetFileSize();
		structFile.m_strFakeCheck = g_eMuleApp.m_pFakeCheck->IsFake(structFile.m_strFileHash, structFile.m_uFileSize);

		switch (iSortBy)
		{
			case 0:
				structFile.m_strIndex = structFile.m_strFileName;
				break;
			case 1:
				structFile.m_strIndex.Format(_T("%09u"), structFile.m_uFileSize);
				break;
			case 2:
				structFile.m_strIndex = structFile.m_strFileHash;
				break;
			case 3:
				structFile.m_strIndex.Format(_T("%09u"), structFile.m_uSourceCount);
				break;
			case 4:
				structFile.m_strIndex = structFile.m_strFakeCheck;
				break;
			case 5:
				structFile.m_strIndex = structFile.m_strFileType;
				break;
			default:
				structFile.m_strIndex.Empty();
		}
		SearchFileArray.Add(structFile);
	}

	SearchFileArray.QuickSort(bAscending);
	
#ifdef _DEBUG
	DWORD dwEnd = ::GetTickCount();
	dwSortTime = dwEnd - dwStart;
#endif

	uchar aFileHash[16];
	uchar nRed, nGreen, nBlue;
	CKnownFile* sameFile;
	CString strOverlayImage;
	CString strSourcesImage;
	CString strColorPrefix;
	CString strColorSuffix = _T("</font>");
	CString strColor;
	CString strSize;
	CString strSources;
	CString strFilename;
	CString strTemp;
	CString strOutput = _T("");

	for (uint16 i = 0; i < SearchFileArray.GetCount(); ++i)
	{
		structFile = SearchFileArray.GetAt(i);
		nRed = nGreen = nBlue = 255;
		DecodeBase16(structFile.m_strFileHash, 32, aFileHash);

		sameFile = g_eMuleApp.m_pSharedFilesList->GetFileByID(aFileHash);
		if (sameFile == NULL)
			sameFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(aFileHash);

		if (sameFile == NULL)
		{
			strOverlayImage = _T("none");
		}
		else
		{
			if (sameFile->IsPartFile())
			{
				strOverlayImage = _T("jumpstart");
				nBlue = 0;
				nRed = 0;
			}
			else
			{
				strOverlayImage = _T("release");
				nBlue = 0;
			}
		}
		strColor = _T("#");
		strColor.AppendFormat(_T("%02x"), nRed);
		strColor.AppendFormat(_T("%02x"), nGreen);
		strColor.AppendFormat(_T("%02x"), nBlue);
		strColorPrefix = _T("<font color=") + strColor + _T(">");

		if (structFile.m_uSourceCount < 5)
			strSourcesImage = _T("0");
		else if (structFile.m_uSourceCount > 4 && structFile.m_uSourceCount < 10)
			strSourcesImage = _T("5");
		else if (structFile.m_uSourceCount > 9 && structFile.m_uSourceCount < 25)
			strSourcesImage = _T("10");
		else if (structFile.m_uSourceCount > 24 && structFile.m_uSourceCount < 50)
			strSourcesImage = _T("25");
		else
			strSourcesImage = _T("50");

		strSize.Format(_T("%d"), structFile.m_uFileSize);
		strSources.Format(_T("%d"), structFile.m_uSourceCount);
		strFilename = structFile.m_strFileName;
		strFilename.Replace(_T("'"),_T("\\'"));

		strTemp.Format(strLinePattern,
			strSourcesImage,
			_T("ed2k://|file|") + strFilename + _T("|") + strSize + _T("|") + structFile.m_strFileHash + _T("|/"),
			strOverlayImage,
			structFile.m_strFileType,
			(bShowColumn1) ? strColorPrefix + StringLimit(structFile.m_strFileName,70) + strColorSuffix : _T(""),
			(bShowColumn2) ? strColorPrefix + CastItoXBytes(structFile.m_uFileSize) + strColorSuffix : _T(""),
			(bShowColumn3) ? strColorPrefix + structFile.m_strFileHash + strColorSuffix : _T(""),
			(bShowColumn4) ? strColorPrefix + strSources + strColorSuffix : _T(""),
			(bShowColumn5) ? strColorPrefix + structFile.m_strFakeCheck + strColorSuffix : _T(""),
			structFile.m_strFileHash
			);
		strOutput.Append(strTemp);
	}

#ifdef _DEBUG
	g_eMuleApp.AddDebugLogLine(false,_T("WebServer: Searchlist with %u elements sorted in %u ms, output generated in %u ms"), SearchFileArray.GetSize(), dwSortTime, ::GetTickCount()-dwEnd);
#endif

	return strOutput;

	EMULE_CATCH
	return _T("");
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CSearchList::GetFileType(CString strFileName) const
{
	TCHAR e[_MAX_EXT];
	_tsplitpath(strFileName, NULL, NULL, NULL, e);
	CString ext(_tcslwr(e));

	CString strFileType;
	if (ext==".bmp" || ext==".jpg" || ext==".jpeg" || ext==".gif" || ext==".png" || ext==".tga" ||
		ext==".tif" || ext==".xif" || ext==".tiff" || ext==".psd" || ext==".psp" || ext==".pcx" ||
		ext==".dcx" || ext==".pic" || ext==".pict" || ext==".pct" || ext==".wmf" || ext==".emf")
		strFileType = "picture";
	else if (ext==".ccd"|| ext==".sub" || ext==".cue" || ext==".bin" || ext==".iso" || ext==".nrg" ||
		ext==".img" || ext==".bwa" || ext==".bwi" || ext==".bws" || ext==".bwt" || ext==".mds" ||
		ext==".mdf")
		strFileType = "cdimage";
	else if (ext==".doc" || ext==".txt" || ext==".xls" || ext==".rtf" || ext==".nfo" || 
		ext==".xml" || ext==".htm" || ext==".html" || ext==".ppt")
		strFileType = "document";
	else if (ext==".pdf") 
		strFileType = "adobe";
	else if (ext==".exe" || ext==".msi" || ext==".com" || ext==".bat" || ext==".pif") 
		strFileType = "application";
	else if (ext==".zip" || ext==".rar" || ext==".ace" || ext==".tar" || ext==".gz" || ext==".tgz" ||
		ext==".arj" || ext==".lha" || ext==".uc2")
		strFileType = "archive";
	else if (ext==".avi" || ext==".divx" || ext==".ram" || ext==".rm" || ext==".rv" || ext==".ogm" ||
		ext==".mpg" || ext==".mpeg" || ext==".mpv" || ext==".mps" || ext==".m1v" || ext==".m2v" ||
		ext==".mpe" || ext==".mpa" || ext==".mp4" || ext==".vob" || ext==".mov" || ext==".asf" ||
		ext==".wmv" || ext==".vivo" || ext==".rmvb" || ext==".xvid")
		strFileType = "movie";
	else if (ext==".mp1" || ext==".mp2" || ext==".mp3" || ext==".mp4"|| ext==".mpc" || ext==".mp+" || ext==".mpp" ||
		ext==".mpga" || ext==".au" || ext==".aac" || ext==".ape" || ext==".ogg" || ext==".flac" ||
		ext==".midi" || ext==".mid" || ext==".ra" || ext==".rmi" || ext==".aif" || ext==".aiff" ||
		ext==".wav" || ext==".wma" || ext==".au")
		strFileType = "music";
	else 
		strFileType = "other";
	return strFileType;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::AddFileToDownloadByHash(uchar* hash,EnumCategories eCatID)
{
	EMULE_TRY

	CSearchFile		*pSearchFile;

	for (POSITION pos = list.GetHeadPosition(); pos != 0; list.GetNext(pos))
	{
		pSearchFile = static_cast<CSearchFile*>(list.GetAt(pos));//->GetSearchID() == nSearchID ){
		if (!md4cmp(hash,pSearchFile->GetFileHash()))
		{
			g_eMuleApp.m_pDownloadQueue->AddSearchToDownload(pSearchFile,eCatID);
			break;
		}
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchFile* CSearchList::DetachNextFile(uint32 nSearchID)
{
// mobilemule
	EMULE_TRY
	// the files are NOT deleted, make sure you do this if you call this function
	// find, removes and returns the searchresult with most Sources	
	uint32 nHighSource = 0;
	POSITION resultpos = 0;
	for (POSITION pos = list.GetHeadPosition(); pos != NULL; ){
		CSearchFile* cur_file = list.GetNext(pos);
		if( cur_file->GetSearchID() == nSearchID ){
			if(cur_file->GetIntTagValue(FT_SOURCES) > nHighSource){
				nHighSource = cur_file->GetIntTagValue(FT_SOURCES);
				resultpos = pos;
			}
		}
	}
	if (resultpos == 0){
		ASSERT ( false );
		return NULL;
	}
	CSearchFile* result = list.GetAt(resultpos);
	list.RemoveAt(resultpos);
	return result;
	EMULE_CATCH
	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
