//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#pragma once
#include "server.h"
#include "opcodes.h"
#include "preferences.h"
#include "serverlistctrl.h"
#include "WebServer.h"							// kuchin
#include "Loggable.h"							// kuchin
#ifdef USE_SAFE_ARRAY
	#include "safearray.h"
#endif
#include "afxmt.h"
class CServerList :
	public CLoggable
{
	friend class	CServerListCtrl;
	friend class	CWebServer;					// kuchin

public:
	CServerList(CPreferences *in_prefs);
	~CServerList(void);
	bool	Init();
	bool	AddServer(CServer *pServer);
	void	RemoveServer(CServer *pServer);
	void	RemoveAllServers(void);
	bool	AddServerMetToList(const CString &strFile, bool bMerge = true);
	void	AddServersFromTextFile(CString strFilename);
	bool	SaveServerMetToFile();
	void	ServerStats();
	void	ResetServerPos()		{ m_iServerPos = 0; }
	void	ResetSearchServerPos()	{ searchserverpos = 0; }
	CServer *GetNextServer();
	CServer *GetNextServer(CServer *lastserver);
	CServer *GetNextSearchServer();
	CServer *GetNextStatServer();
	CServer *GetServerAt(uint32 pos)	{ return(m_serverList.GetAt(m_serverList.FindIndex(pos))); }
	uint32	GetServerCount()			{ return(m_serverList.GetCount()); }
	CServer *FindServerByAddress(const CServer &in_server);
	CServer *GetServerByAddress(const CString &address, uint16 port);
	CServer *GetServerByIP(uint32 nIP, uint16 port = 0);
	bool	IsGoodServerIP(CServer *in_server); //<<--
	void	GetStatus(uint32 &total, uint32 &failed, uint32 &user, uint32 &file, uint32 &tuser, uint32 &tfile,
					  double &occ);
	void	GetUserFileStatus(uint32 &user, uint32 &file);
	bool	BroadCastPacket(Packet *packet);	//send Packet to all server in the list
	void	CancelUDPBroadcast();
	void	Sort();
	void	MoveServerDown(CServer *aServer);
	uint32	GetServerPostion()	{ return(m_iServerPos); }
	void SetServerPosition(uint32 iNewPosition)
	{
		if (iNewPosition < (uint32) m_serverList.GetCount())
		{
			m_iServerPos = iNewPosition;
		}
		else
		{
			m_iServerPos = 0;
		}
	}

	uint32					GetDeletedServerCount()		{ return(delservercount); }
	void					Process();
	void static CALLBACK	UDPTimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime);
	void					AutoUpdate();

private:
	uint32								m_iServerPos;
	void								SendNextPacket();
	uint32								searchserverpos;
	uint32								statserverpos;
	int8								m_uServerListVersion;
	uint32								m_iNumServers;
	CTypedPtrList<CPtrList, CServer *>	m_serverList;
	CPreferences						*m_pPrefs;
	uint32								udp_timer;
	POSITION							broadcastpos;
	Packet								*broadcastpacket;
	uint32								delservercount;
	uint32								m_nLastSaved;

protected:
	CMutex	m_ServerMetMutex;
};
