// OpCode.cpp: implementation of the COpCode class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef NEW_SOCKETS
#include "OpCode.h"
#include "EmEngine.h"

//////////////////////////////////////////////////////////////////////
template <> void CStream::operator << (const CString& strTxt)
{
	*this << (USHORT) strTxt.GetLength();
	DumpString(strTxt);
}

//////////////////////////////////////////////////////////////////////
void CStream::DumpString(const CString& strTxt)
{
	USHORT nLen = (USHORT) strTxt.GetLength();

#ifdef UNICODE
	if (m_bUnicode)
		Write((LPCWSTR) strTxt, nLen * sizeof(WCHAR));
	else
	{
		// convert it to ANSI string
		CQuickBuf stBuf;
		stBuf.Alloc(nLen);
		if (nLen != WideCharToMultiByte(AP_ACP, 0, strTxt, nLen, (PSTR) stBuf.m_pBufUse, nLen, NULL, NULL))
			Exception();

		Write(stBuf.m_pBufUse, nLen);
	}

#else // UNICODE

	if (m_bUnicode)
	{
		// convert it to unicode
		CQuickBuf stBuf;
		stBuf.Alloc(nLen * sizeof(WCHAR));
		if (nLen != MultiByteToWideChar(CP_ACP, 0, strTxt, nLen, (PWSTR) stBuf.m_pBufUse, nLen))
			Exception();

		Write(stBuf.m_pBufUse, nLen * sizeof(WCHAR));
	} else
		Write((LPCSTR) strTxt, nLen);

#endif // UNICODE
}

//////////////////////////////////////////////////////////////////////
template <> void CStream::operator >> (CString& strTxt)
{
	ushort nLen;
	*this >> nLen;
	InitString(strTxt, nLen);
}

//////////////////////////////////////////////////////////////////////
void CStream::InitTrailingString(CString& strTxt) {
	DWORD dwSize = SizeLeft();
	if (m_bUnicode)
		dwSize >>= 1; // /= 2
	if (dwSize > 0xFFFF)
		Exception();
	InitString(strTxt, SizeLeft());
}

//////////////////////////////////////////////////////////////////////
void CStream::InitString(CString& strTxt, ushort nLen)
{
	strTxt.Empty();
	PVOID pStr = strTxt.GetBuffer(nLen);

#ifdef UNICODE

	if (m_bUnicode)
		Read(pStr, nLen * sizeof(WCHAR));
	} else
	{
		// convert it to unicode
		CQuickBuf stBuf;
		stBuf.Alloc(nLen);
		Read(stBuf.m_pBufUse, nLen);

		if (nLen != MultiByteToWideChar(CP_ACP, 0, (PCSTR) stBuf.m_pBufUse, nLen, (PWSTR) pStr, nLen))
			Exception();
	}
#else // UNICODE
	if (m_bUnicode)
	{
		// convert it to ansi
		CQuickBuf stBuf;
		stBuf.Alloc(nLen * sizeof(WCHAR));
		Read(stBuf.m_pBufUse, nLen * sizeof(WCHAR));

		if (nLen != WideCharToMultiByte(CP_ACP, 0, (PCWSTR) stBuf.m_pBufUse, nLen, (PSTR) pStr, nLen, NULL, NULL))
			Exception();
	} else
		Read(pStr, nLen);
#endif // UNICODE

	strTxt.ReleaseBuffer(nLen);
}

//////////////////////////////////////////////////////////////////////
void CStream::DumpTags(CTagEntry* pTags, DWORD dwCount)
{
	// count how many tags we have
	DWORD dwTagsCount = 0;
	for (DWORD dwIndex = 0; dwIndex < dwCount; dwIndex++)
		if (pTags[dwIndex].m_pTag->m_bValid)
			dwTagsCount++;

	*this << dwTagsCount;

	// dump tags
	for (; dwCount--; pTags++)
		if (pTags->m_pTag->m_bValid)
		{
			*this << pTags->m_nType;

			// put this tag's id (or name)
			*this << pTags->m_nNameLen;
			if (1 == pTags->m_nNameLen)
				*this << (BYTE) pTags->m_dwName;
			else
				Write((PVOID) pTags->m_dwName, pTags->m_nNameLen);
			
			// tag's value
			switch (pTags->m_nType)
			{
			case OP_TAG_DWORD:
				*this << ((CTag_DWORD*) pTags->m_pTag)->m_dwValue;
				break;

			case OP_TAG_STRING:
				*this << ((CTag_String*) pTags->m_pTag)->m_strValue;
				break;

			default:
				Exception();
			}

		}
}

template <class T> void StreamDumpElement(CStream& stStream, const T& stElement)
{
	stStream << stElement;
}
template <class T> void StreamInitElement(CStream& stStream, T& stElement)
{
	stStream >> stElement;
}

//////////////////////////////////////////////////////////////////////
template <class T>
void CStream::DumpArr(const CArray<T, T&>& arrValue, int nCounterLen)
{
	int nSize = arrValue.GetSize();
	ASSERT(nCounterLen <= sizeof(nSize));
	if((sizeof(nCounterLen) == 1 && nSize > 255) || (sizeof(nCounterLen) == 2 && nSize > 65535))
		Exception(); // too long

	Write(&nSize, nCounterLen);
	for (int nItem = 0; nItem < nSize; nItem++)
		StreamDumpElement(*this, arrValue.GetAt(nItem));
}

//////////////////////////////////////////////////////////////////////
template <class T>
void CStream::InitArr(CArray<T, T&>& arrValue, int nCounterLen)
{
	int nSize;
	ASSERT(nCounterLen <= sizeof(nSize));
	Read(&nSize, nCounterLen);

	arrValue.SetSize(nSize);
	for (int nItem = 0; nItem < nSize; nItem++)
		StreamInitElement(*this, arrValue.GetAt(nItem));
}

//////////////////////////////////////////////////////////////////////
void CStream::InitTags(CTagEntry* pTags, DWORD dwCount)
{
	DWORD dwTagsCount;
	*this >> dwTagsCount;

	while (dwTagsCount--)
	{
		BYTE nType;
		*this >> nType;

		USHORT nLen;
		*this >> nLen;

		BYTE nTagID;
		CQuickBuf stTagName;

		if (1 == nLen)
			*this >> nTagID;
		else
		{
			stTagName.Alloc(nLen);
			Read(stTagName.m_pBufUse, nLen);
		}

		// check if we have such a tag id (or name)
		for (DWORD dwIndex = 0; dwIndex < dwCount; dwIndex++)
			if ((pTags[dwIndex].m_nNameLen == nLen) &&
				((1 == nLen) ? (nTagID == (BYTE) pTags[dwIndex].m_dwName) : !memcmp(stTagName.m_pBufUse, (PVOID) pTags[dwIndex].m_dwName, nLen)))
				break; // match

		CTagEntry* pTarget = (dwIndex < dwCount) ? pTags + dwIndex : NULL;

		if (pTarget && (pTarget->m_nType != nType))
			Exception(); // type mismatch

		switch (nType)
		{
		case OP_TAG_DWORD:
			if (pTarget)
				*this >> ((CTag_DWORD*) pTarget->m_pTag)->m_dwValue;
			else
			{
				DWORD dwVal;
				*this >> dwVal;
			}
			break;
		case OP_TAG_STRING:
			if (pTarget)
				*this >> ((CTag_String*) pTarget->m_pTag)->m_strValue;
			else
			{
				CString strVal;
				*this >> strVal; // This is a bit heave. Better just to skip this string
			}
			break;
		default:
			Exception();
		}

		if (pTarget)
			pTarget->m_pTag->m_bValid = true;
	}
}

//////////////////////////////////////////////////////////////////////
void CStream::Write(PCVOID, DWORD)
{
	Exception(); // not implemented
}

//////////////////////////////////////////////////////////////////////
void CStream::Read(PVOID, DWORD)
{
	Exception(); // not implemented
}

//////////////////////////////////////////////////////////////////////
DWORD CStream::SizeLeft()
{
	Exception(); // not implemented
	return 0;
}

//////////////////////////////////////////////////////////////////////
void CStream::Skip(DWORD dwSize)
{
	BYTE pBuf[0x1000];
	while (dwSize)
	{
		DWORD dwPortion = min(dwSize, sizeof(pBuf));
		Read(pBuf, dwPortion);
		dwSize -= dwPortion;
	}
}

//////////////////////////////////////////////////////////////////////
void CStream::Exception()
{
	CException stException;
	throw stException;
}

//////////////////////////////////////////////////////////////////////
void CStream_Measure::Write(PCVOID, DWORD dwSize)
{
	m_dwSize += dwSize;
}

//////////////////////////////////////////////////////////////////////
void CStream_Mem::Write(PCVOID pBuf, DWORD dwSize)
{
	if (dwSize > m_dwSize)
		Exception();
	CopyMemory(m_pPtr, pBuf, dwSize);
	m_pPtr += dwSize;
	m_dwSize -= dwSize;
}

//////////////////////////////////////////////////////////////////////
void CStream_Mem::Read(PVOID pBuf, DWORD dwSize)
{
	if (dwSize > m_dwSize)
		Exception();
	CopyMemory(pBuf, m_pPtr, dwSize);
	m_pPtr += dwSize;
	m_dwSize -= dwSize;
}

//////////////////////////////////////////////////////////////////////
void CStream_MemEx::Write(PCVOID pBuf, DWORD dwSize)
{
	if (dwSize > m_dwSize)
	{
		m_dwSizeExtra += dwSize;
		if (m_dwSize)
		{
			m_dwSizeExtra -= m_dwSize;
			m_dwSize = 0;
		}

	} else
	{
		CopyMemory(m_pPtr, pBuf, dwSize);
		m_pPtr += dwSize;
		m_dwSize -= dwSize;
	}
}

//////////////////////////////////////////////////////////////////////
COpCode::~COpCode()
{
}

//////////////////////////////////////////////////////////////////////
bool COpCode::Write(CStream& stStream, BYTE nTransport) const
{
	try
	{
		BYTE nProtocol = Z_Protocol();
		stStream.m_bUnicode = (OP_PROT_UNICODE == nProtocol);
		stStream << nProtocol;

		switch (nTransport)
		{
		case OP_TRANSPORT_TCP:
			{
				// write the body size
				CStream_Measure stMeasure;
				Z_Dump(stMeasure);

				stStream << stMeasure.m_dwSize + 1;
			}
			break;
		case OP_TRANSPORT_UDP:
			break;
		default:
			stStream.Exception(); // undefined transport format
		}

		stStream << GetID();

		// dump the rest of the opcode
		Z_Dump(stStream);

	}
	catch (CStream::CException)
	{
		return false;
	}
	return true; // ok
}

//////////////////////////////////////////////////////////////////////
COpCode* COpCode::CreateRaw(BYTE nID, BYTE nProtocol, T_CLIENT_TYPE eType)
{
	switch ((eType << 16) | (nProtocol << 8) | nID)
	{


#define BEGIN_OPCODE(id, name, prot, source)	case (T_CLIENT_##source << 16) | (OP_PROT_##prot << 8) | id: return new COpCode_##name;
		#define END_OPCODE
		#define PARAM_BYTE_JUNK(value)
		#define PARAM_BUF(name, type, count)
		#define PARAM_SIMPLE(name, type)
		#define PARAM_TAGS_BEGIN
		#define PARAM_TAGS_END
		#define PARAM_TAG_STR(name, id)
		#define PARAM_TAG_DWORD(name, id)
		#define PARAM_NTAG_DWORD(name, id)
		#define PARAM_TEXT_TRAILING(name)
		#define PARAM_ARRAY(name, type, counter)
		#define PARAM_COMPLEXARRAY_BEGIN(msg, name)
		#define PARAM_COMPLEXARRAY_END(name)

		#include "EmMsgs.h"

		#undef BEGIN_OPCODE
		#undef END_OPCODE
		#undef PARAM_BYTE_JUNK
		#undef PARAM_BUF
		#undef PARAM_SIMPLE
		#undef PARAM_TAGS_BEGIN
		#undef PARAM_TAGS_END
		#undef PARAM_TAG_STR
		#undef PARAM_TAG_DWORD
		#undef PARAM_NTAG_DWORD
		#undef PARAM_TEXT_TRAILING
		#undef PARAM_ARRAY
		#undef PARAM_COMPLEXARRAY_BEGIN
		#undef PARAM_COMPLEXARRAY_END
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////
COpCode* COpCode::Read(CStream& stStream, BYTE nID, BYTE nProtocol, T_CLIENT_TYPE eType)
{
	COpCode* pRet = CreateRaw(nID, nProtocol, eType);
	if (pRet)
		try
		{
			stStream.m_bUnicode = (OP_PROT_UNICODE == nProtocol);
			pRet->Z_Init(stStream);
		}
		catch (CStream::CException)
		{
			delete pRet;
			pRet = NULL;
		}

	return pRet;
}

//////////////////////////////////////////////////////////////////////
DWORD COpCode::GetSize(BYTE nTransport) const
{
	CStream_Measure stStream;
	return Write(stStream, nTransport) ? stStream.m_dwSize : 0;
}

//////////////////////////////////////////////////////////////////////
bool COpCode::Process(CEmEngine& stEngine)
{
	CEmClient* pClient = NULL;
	if (stEngine.m_stMain.IsInContext(*this))
	{
		pClient = stEngine.m_stMain.Lookup(m_hSocket);
		if (!pClient)
			return true; // no more relevant!
	}

	TRACE2(_T("\nProcessing opcode=%x, (socket=%u)"), GetID(), pClient ? pClient->m_hSocket : 0);
#ifdef _DEBUG
	ShowSelf(false);
#endif //_DEBUG

	return ProcessForClient(stEngine, pClient);
}

//////////////////////////////////////////////////////////////////////
// Debug stuff
#ifndef OPCODE_SKIP_DBGDUMP

void COpCode::CDbgDump::WriteParam(LPCTSTR szName, DWORD dwValue)
{
	CString strDelta;
	strDelta.Format(_T("\r\n%s = %u"), szName, dwValue);
	m_strTxt += strDelta;
}
void COpCode::CDbgDump::WriteParam(LPCTSTR szName, const CString& strValue)
{
	CString strDelta;
	strDelta.Format(_T("\r\n%s = %s"), szName, strValue);
	m_strTxt += strDelta;
}
void COpCode::CDbgDump::WriteParam(LPCTSTR szName, const ADDRPORT& stValue)
{
	CString strDelta;
	strDelta.Format(_T("\r\n%s = %u.%u.%u.%u:%u"), szName,
		((in_addr&)stValue._Addr).S_un.S_un_b.s_b1,
		((in_addr&)stValue._Addr).S_un.S_un_b.s_b2,
		((in_addr&)stValue._Addr).S_un.S_un_b.s_b3,
		((in_addr&)stValue._Addr).S_un.S_un_b.s_b4,
		stValue._Port);

	m_strTxt += strDelta;
}
void COpCode::CDbgDump::WriteParam(LPCTSTR szName, const void*, DWORD dwCount, DWORD)
{
	CString strDelta;
	strDelta.Format(_T("\r\n%s = [Count=%u]"), szName, dwCount);
	m_strTxt += strDelta;
}
void COpCode::CDbgDump::WriteParam(LPCTSTR szName, const CTag_DWORD& stValue)
{
	CString strDelta;
	strDelta.Format(_T("\r\n%s = [%c] %u"), stValue.m_bValid ? _T('+') : _T('-'), stValue.m_dwValue);
	m_strTxt += strDelta;
}
void COpCode::CDbgDump::WriteParam(LPCTSTR szName, const CTag_String& stValue)
{
	CString strDelta;
	strDelta.Format(_T("\r\n%s = [%c] %s"), stValue.m_bValid ? _T('+') : _T('-'), stValue.m_strValue);
	m_strTxt += strDelta;
}
template <class T>
void COpCode::CDbgDump::WriteParam(LPCTSTR szName, const CArray<T, T&>& arrValue)
{
	CString strDelta;
	strDelta.Format(_T("\r\n%s = [Count=%u]"), szName, arrValue.GetSize());
	m_strTxt += strDelta;
}

int COpCode::ShowSelf(bool bMessageBox /* = true */, int nStyle /* = MB_ICONINFORMATION | MB_OK | MB_TOPMOST */)
{
	CDbgDump stDump;
	stDump.m_strTxt.Format(_T("Opcode ID=%u, Ptr=%08X"), GetID(), this);
	Z_DbgDump(stDump);

	if(bMessageBox)
		return ::MessageBox(NULL, stDump.m_strTxt, _T("Opcode parameters"), nStyle);

	TRACE("%s", stDump.m_strTxt);

	return 0;
}
#endif // OPCODE_SKIP_DBGDUMP


//////////////////////////////////////////////////////////////////////
// All OpCodes
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Constructors

#undef PARAM_SIMPLE_ORD
#define BEGIN_OPCODE(id, name, prot, source)	COpCode_##name::COpCode_##name() {
#define END_OPCODE						}
#define PARAM_BYTE_JUNK(value)
#define PARAM_BUF(name, type, count)	ZeroMemory(_##name, count * sizeof(type));
#define PARAM_SIMPLE(name, type)
#define PARAM_SIMPLE_ORD(name, type)	_##name = 0;
#define PARAM_TAGS_BEGIN
#define PARAM_TAGS_END
#define PARAM_TAG_STR(name, id)
#define PARAM_TAG_DWORD(name, id)
#define PARAM_NTAG_DWORD(name, id)
#define PARAM_TEXT_TRAILING(name)
#define PARAM_ARRAY(name, type, counter)
#define PARAM_COMPLEXARRAY_BEGIN(msg, name) } COpCode_##msg::CStruct_##name::CStruct_##name() {
#define PARAM_COMPLEXARRAY_END(name)

#include "EmMsgs.h"

#undef BEGIN_OPCODE
#undef END_OPCODE
#undef PARAM_BYTE_JUNK
#undef PARAM_BUF
#undef PARAM_SIMPLE
#undef PARAM_TAGS_BEGIN
#undef PARAM_TAGS_END
#undef PARAM_TAG_STR
#undef PARAM_TAG_DWORD
#undef PARAM_NTAG_DWORD
#undef PARAM_TEXT_TRAILING
#undef PARAM_ARRAY
#undef PARAM_SIMPLE_ORD
#undef PARAM_COMPLEXARRAY_BEGIN
#undef PARAM_COMPLEXARRAY_END
#define PARAM_SIMPLE_ORD PARAM_SIMPLE

//////////////////////////////////////////////////////////////////////
// Destructors


#define BEGIN_OPCODE(id, name, prot, source)	COpCode_##name::~COpCode_##name() {
#define END_OPCODE						}
#define PARAM_BYTE_JUNK(value)
#define PARAM_BUF(name, type, count)
#define PARAM_SIMPLE(name, type)
#define PARAM_TAGS_BEGIN
#define PARAM_TAGS_END
#define PARAM_TAG_STR(name, id)
#define PARAM_TAG_DWORD(name, id)
#define PARAM_NTAG_DWORD(name, id)
#define PARAM_TEXT_TRAILING(name)
#define PARAM_ARRAY(name, type, counter)
#define PARAM_COMPLEXARRAY_BEGIN(msg, name)
#define PARAM_COMPLEXARRAY_END(name)

#include "EmMsgs.h"

#undef BEGIN_OPCODE
#undef END_OPCODE
#undef PARAM_BYTE_JUNK
#undef PARAM_BUF
#undef PARAM_SIMPLE
#undef PARAM_TAGS_BEGIN
#undef PARAM_TAGS_END
#undef PARAM_TAG_STR
#undef PARAM_TAG_DWORD
#undef PARAM_NTAG_DWORD
#undef PARAM_TEXT_TRAILING
#undef PARAM_ARRAY
#undef PARAM_COMPLEXARRAY_BEGIN
#undef PARAM_COMPLEXARRAY_END

//////////////////////////////////////////////////////////////////////
// GetID()

#define BEGIN_OPCODE(id, name, prot, source)	BYTE COpCode_##name::GetID() const { return id;
#define END_OPCODE						}
#define PARAM_BYTE_JUNK(value)
#define PARAM_BUF(name, type, count)
#define PARAM_SIMPLE(name, type)
#define PARAM_TAGS_BEGIN
#define PARAM_TAGS_END
#define PARAM_TAG_STR(name, id)
#define PARAM_TAG_DWORD(name, id)
#define PARAM_NTAG_DWORD(name, id)
#define PARAM_TEXT_TRAILING(name)
#define PARAM_ARRAY(name, type, counter)
#define PARAM_COMPLEXARRAY_BEGIN(msg, name)
#define PARAM_COMPLEXARRAY_END(name)

#include "EmMsgs.h"

#undef BEGIN_OPCODE
#undef END_OPCODE
#undef PARAM_BYTE_JUNK
#undef PARAM_BUF
#undef PARAM_SIMPLE
#undef PARAM_TAGS_BEGIN
#undef PARAM_TAGS_END
#undef PARAM_TAG_STR
#undef PARAM_TAG_DWORD
#undef PARAM_NTAG_DWORD
#undef PARAM_TEXT_TRAILING
#undef PARAM_ARRAY
#undef PARAM_COMPLEXARRAY_BEGIN
#undef PARAM_COMPLEXARRAY_END

//////////////////////////////////////////////////////////////////////
// Z_Protocol()

#define BEGIN_OPCODE(id, name, prot, source)	BYTE COpCode_##name::Z_Protocol() const { return OP_PROT_##prot;
#define END_OPCODE						}
#define PARAM_BYTE_JUNK(value)
#define PARAM_BUF(name, type, count)
#define PARAM_SIMPLE(name, type)
#define PARAM_TAGS_BEGIN
#define PARAM_TAGS_END
#define PARAM_TAG_STR(name, id)
#define PARAM_TAG_DWORD(name, id)
#define PARAM_NTAG_DWORD(name, id)
#define PARAM_TEXT_TRAILING(name)
#define PARAM_ARRAY(name, type, counter)
#define PARAM_COMPLEXARRAY_BEGIN(msg, name)
#define PARAM_COMPLEXARRAY_END(name)

#include "EmMsgs.h"

#undef BEGIN_OPCODE
#undef END_OPCODE
#undef PARAM_BYTE_JUNK
#undef PARAM_BUF
#undef PARAM_SIMPLE
#undef PARAM_TAGS_BEGIN
#undef PARAM_TAGS_END
#undef PARAM_TAG_STR
#undef PARAM_TAG_DWORD
#undef PARAM_NTAG_DWORD
#undef PARAM_TEXT_TRAILING
#undef PARAM_ARRAY
#undef PARAM_COMPLEXARRAY_BEGIN
#undef PARAM_COMPLEXARRAY_END

//////////////////////////////////////////////////////////////////////
// Z_Dump

#define BEGIN_OPCODE(id, name, prot, source)	void COpCode_##name::Z_Dump(CStream& stStream) const {
#define END_OPCODE						}
#define PARAM_BYTE_JUNK(value)			stStream << (BYTE) value;
#define PARAM_BUF(name, type, count)	stStream.Write(_##name, count * sizeof(type));
#define PARAM_SIMPLE(name, type)		stStream << _##name;
#define PARAM_TAGS_BEGIN				CTagEntry pTagsInfo[] = { 
#define PARAM_TAGS_END					}; stStream.DumpTags(pTagsInfo, _countof(pTagsInfo));
#define PARAM_TAG_STR(name, id)			(CTag_Pure*) &_##name, OP_TAG_STRING, 1, id,
#define PARAM_TAG_DWORD(name, id)		(CTag_Pure*) &_##name, OP_TAG_DWORD, 1, id,
#define PARAM_NTAG_DWORD(name, id)		(CTag_Pure*) &_##name, OP_TAG_DWORD, sizeof(id) - 1, (DWORD) id,
#define PARAM_TEXT_TRAILING(name)		stStream.DumpString(_##name);
#define PARAM_ARRAY(name, type, counter) stStream.DumpArr(_##name, sizeof(counter));
#define PARAM_COMPLEXARRAY_BEGIN(msg, name) stStream.DumpArr(_##name, sizeof(DWORD)); } template <> void StreamDumpElement(CStream& stStream, const COpCode_##msg::CStruct_##name& stElement) { stElement.Z_Dump(stStream); } void COpCode_##msg::CStruct_##name::Z_Dump(CStream& stStream) const {
#define PARAM_COMPLEXARRAY_END(name)

#include "EmMsgs.h"

#undef BEGIN_OPCODE
#undef END_OPCODE
#undef PARAM_BYTE_JUNK
#undef PARAM_BUF
#undef PARAM_SIMPLE
#undef PARAM_TAGS_BEGIN
#undef PARAM_TAGS_END
#undef PARAM_TAG_STR
#undef PARAM_TAG_DWORD
#undef PARAM_NTAG_DWORD
#undef PARAM_TEXT_TRAILING
#undef PARAM_ARRAY
#undef PARAM_COMPLEXARRAY_BEGIN
#undef PARAM_COMPLEXARRAY_END

//////////////////////////////////////////////////////////////////////
// Z_Init

#define BEGIN_OPCODE(id, name, prot, source)	void COpCode_##name::Z_Init(CStream& stStream) {
#define END_OPCODE						}
#define PARAM_BYTE_JUNK(value)			BYTE nJunk; stStream >> nJunk; if (value != nJunk) stStream.Exception();
#define PARAM_BUF(name, type, count)	stStream.Read(_##name, count * sizeof(type));
#define PARAM_SIMPLE(name, type)		stStream >> _##name;
#define PARAM_TAGS_BEGIN				CTagEntry pTagsInfo[] = { 
#define PARAM_TAGS_END					}; stStream.InitTags(pTagsInfo, _countof(pTagsInfo));
#define PARAM_TAG_STR(name, id)			(CTag_Pure*) &_##name, OP_TAG_STRING, 1, id,
#define PARAM_TAG_DWORD(name, id)		(CTag_Pure*) &_##name, OP_TAG_DWORD, 1, id,
#define PARAM_NTAG_DWORD(name, id)		(CTag_Pure*) &_##name, OP_TAG_DWORD, sizeof(id) - 1, (DWORD) id,
#define PARAM_TEXT_TRAILING(name)		stStream.InitTrailingString(_##name);
#define PARAM_ARRAY(name, type, counter) stStream.InitArr(_##name, sizeof(counter));
#define PARAM_COMPLEXARRAY_BEGIN(msg, name) stStream.InitArr(_##name, sizeof(DWORD)); } template <> void StreamInitElement(CStream& stStream, COpCode_##msg::CStruct_##name& stElement) { stElement.Z_Init(stStream); } void COpCode_##msg::CStruct_##name::Z_Init(CStream& stStream) {
#define PARAM_COMPLEXARRAY_END(name)

#include "EmMsgs.h"

#undef BEGIN_OPCODE
#undef END_OPCODE
#undef PARAM_BYTE_JUNK
#undef PARAM_BUF
#undef PARAM_SIMPLE
#undef PARAM_TAGS_BEGIN
#undef PARAM_TAGS_END
#undef PARAM_TAG_STR
#undef PARAM_TAG_DWORD
#undef PARAM_NTAG_DWORD
#undef PARAM_TEXT_TRAILING
#undef PARAM_ARRAY
#undef PARAM_COMPLEXARRAY_BEGIN
#undef PARAM_COMPLEXARRAY_END

//////////////////////////////////////////////////////////////////////
// Z_DbgDump

#ifndef OPCODE_SKIP_DBGDUMP

#define BEGIN_OPCODE(id, name, prot, source)	void COpCode_##name::Z_DbgDump(CDbgDump& stDump) const { stDump.m_strTxt.Format("%s (%u)  Ptr=%08X  Protocol=%u  Size=%u\n", #name, id, this, OP_PROT_##prot, GetSize(OP_TRANSPORT_UDP));
#define END_OPCODE						}
#define PARAM_BYTE_JUNK(value)
#define PARAM_BUF(name, type, count)	stDump.WriteParam(#name, _##name, count, sizeof(type));
#define PARAM_SIMPLE(name, type)		stDump.WriteParam(#name, _##name);
#define PARAM_TAGS_BEGIN
#define PARAM_TAGS_END
#define PARAM_TAG_STR(name, id)			stDump.WriteParam(#name, _##name);
#define PARAM_TAG_DWORD(name, id)		stDump.WriteParam(#name, _##name);
#define PARAM_NTAG_DWORD(name, id)		stDump.WriteParam(#name, _##name);
#define PARAM_TEXT_TRAILING(name)		stDump.WriteParam(#name, _##name);
#define PARAM_ARRAY(name, type, counter) stDump.WriteParam(#name, _##name);
#define PARAM_COMPLEXARRAY_BEGIN(msg, name) stDump.WriteParam(#name, _##name); } void COpCode_##msg::CStruct_##name::Z_DbgDump(CDbgDump& stDump) const {
#define PARAM_COMPLEXARRAY_END(name)
//#define PARAM_COMPLEXARRAY_BEGIN(msg, name) stStream.InitArr(_##name, sizeof(DWORD)); } template <> void StreamInitElement(CStream& stStream, COpCode_##msg::CStruct_##name& stElement) { stElement.Z_Init(stStream); } void COpCode_##msg::CStruct_##name::Z_Init(CStream& stStream) {
//#define PARAM_COMPLEXARRAY_END(name)

#include "EmMsgs.h"

#undef BEGIN_OPCODE
#undef END_OPCODE
#undef PARAM_BYTE_JUNK
#undef PARAM_BUF
#undef PARAM_SIMPLE
#undef PARAM_TAGS_BEGIN
#undef PARAM_TAGS_END
#undef PARAM_TAG_STR
#undef PARAM_TAG_DWORD
#undef PARAM_NTAG_DWORD
#undef PARAM_TEXT_TRAILING
#undef PARAM_ARRAY
#undef PARAM_COMPLEXARRAY_BEGIN
#undef PARAM_COMPLEXARRAY_END

#endif // OPCODE_SKIP_DBGDUMP

#endif
