// TaskProcessor.cpp: implementation of the CTaskProcessor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef NEW_SOCKETS

#include "TaskProcessor.h"
#include "EmEngine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

bool CTaskProcessor::Init()
{
	Uninit(); // if already initialized

	if (m_queueTasks.Init())
		if (m_hStop = CreateEvent(NULL, TRUE, FALSE, NULL))
		{
			THREAD_INIT_PARAM stParam;
			stParam.m_pThis = this;
			stParam.m_bInitResult = false;
			if (stParam.m_hInitEvent = CreateEvent(NULL, TRUE, FALSE, NULL))
			{
				UINT nThread = 0;
				if (m_hThread = (HANDLE) _beginthreadex(NULL, 0, ProcessingFunc, &stParam, 0, &nThread))
				{
					DWORD dwRes = WaitForSingleObject(stParam.m_hInitEvent, 1000);
					if (WAIT_OBJECT_0 == dwRes)
						if (stParam.m_bInitResult)
							return true;
						else
						{
							ASSERT(FALSE);
							AddDebugLogLine(false, "Task processor failed to initialize");
						}
					else
						if (WAIT_TIMEOUT == dwRes)
						{
							ASSERT(FALSE);
							AddDebugLogLine(false, "Task processor initialization timed out");
						}
						else
						{
							ASSERT(FALSE);
							AddDebugLogLine(false, "WaitForSingleObject failed");
						}
				} 
				else
				{
					ASSERT(FALSE);
					AddDebugLogLine(false, "_beginthreadex failed");
				}

			} 
			else
			{
				ASSERT(FALSE);
				AddDebugLogLine(false, "CreateEvent failed");
			}

		} 
		else
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "CreateEvent failed");
		}
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "Failed to create the queue");
	}

	Uninit();
	return false;
}

//////////////////////////////////////////////////////////////////////
// destruction
void CTaskProcessor::Uninit()
{
	if (m_hStop)
	{
		if (m_hThread)
		{
			if (!SetEvent(m_hStop))
			{
				ASSERT(FALSE);
				AddDebugLogLine(false, "SetEvent failed");
			}

			if (WAIT_TIMEOUT == WaitForSingleObject(m_hThread, 2000))
			{
				ASSERT(FALSE);
				AddDebugLogLine(false, "Failed to uninitialize. Terminating");
				if (!TerminateThread(m_hThread, -1))
					AddDebugLogLine(false, "TerminateThread failed");
			}

			if (!CloseHandle(m_hThread))
			{
				ASSERT(FALSE);
				AddDebugLogLine(false, "CloseHandle failed");
			}
			m_hThread = NULL;
		}

		if (!CloseHandle(m_hStop))
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, "CloseHandle failed");
		}
		m_hStop = NULL;
	}
	m_queueTasks.Uninit();
}

//////////////////////////////////////////////////////////////////////
// processing
UINT WINAPI CTaskProcessor::ProcessingFunc(PVOID pPtr)
{
	THREAD_INIT_PARAM* pParam = (THREAD_INIT_PARAM*) pPtr;
	if (pParam)
	{
		CTaskProcessor* pThis = pParam->m_pThis;
		bool bRes = pParam->m_bInitResult = pThis->Start();
		SetEvent(pParam->m_hInitEvent);

		if (bRes)
			for (DWORD dwWaitLast = GetTickCount(); ; )
			{
				DWORD dwWait = INFINITE;
				if (INFINITE != pThis->m_dwWaitTimeout)
				{
					DWORD dwElapsed = GetTickCount() - dwWaitLast;
					if (dwElapsed < pThis->m_dwWaitTimeout)
						dwWait = pThis->m_dwWaitTimeout - dwElapsed;
					else
					{
						pThis->ProcessTimeout();
						dwWaitLast = GetTickCount();
						dwWait = pThis->m_dwWaitTimeout;
					}
				}

				CTask* pTask = pThis->m_queueTasks.PopEx(pThis->m_hStop, dwWait); 
				if (pTask)
				{
					pThis->m_pCurrent = pTask;
					if (pThis->ProcessTask(*pTask))
						delete pTask;
					pThis->m_pCurrent = NULL;
				}
				else
					if (pThis->m_stEngine.IsShuttingDown())
						break; // finished!

			}

		pThis->Stop();
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////
// those functions are defined in child classes
bool CTaskProcessor::Start()
{
	return true;
}

void CTaskProcessor::Stop()
{
}

void CTaskProcessor::ProcessTimeout()
{
}

//////////////////////////////////////////////////////////////////////
// process single task
bool CTaskProcessor::ProcessTask(CTask& stTask)
{
	try
	{
		return stTask.Process(m_stEngine);
	}
	catch (_com_error& e)
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, _T("ATL Exception while processing task: Source=%s; Description=%s"), (PCTSTR) e.Source(), (PCTSTR) e.Description());
	}
	catch (...)
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "Unhandled exception while processing task");
	}

	try
	{
		// give it another chance
		return stTask.OnException(m_stEngine);
	}
	catch(_com_error& e)
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, _T("Second ATL exception while processing task: Source=%s; Description=%s"), (PCTSTR) e.Source(), (PCTSTR) e.Description());
	}
	catch (...)
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "Second unhandled exception while processing task");
	}

	return true; // delete this buggy task!
}

#endif