// TaskProcessor.h: interface for the CTaskProcessor class.
//
//////////////////////////////////////////////////////////////////////
#ifdef NEW_SOCKETS
#if !defined(AFX_TASKPROCESSOR_H__186018D8_26C9_4A91_9F7C_DDC406696088__INCLUDED_)
#define AFX_TASKPROCESSOR_H__186018D8_26C9_4A91_9F7C_DDC406696088__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Loggable.h"
#include "ThreadPool.h"

class CEmEngine;
struct CTask : public CLoggable {
public:
	virtual ~CTask();
	virtual bool Process(CEmEngine&) = 0;
	virtual bool OnException(CEmEngine&); // usually DB exception
	virtual bool ProcessForUI(CEmEngine&);
};


class CTaskProcessor : public CLoggable
{
	HANDLE m_hThread;
	HANDLE m_hStop;

	CTask* m_pCurrent;

	struct THREAD_INIT_PARAM {
		CTaskProcessor*	m_pThis;
		HANDLE		m_hInitEvent;
		bool		m_bInitResult;
	};

	static UINT WINAPI ProcessingFunc(PVOID pPtr);

	CPtrQueue<CTask> m_queueTasks;

	virtual bool Start();
	virtual void Stop();
	virtual void ProcessTimeout();
	bool ProcessTask(CTask&);

protected:
	DWORD m_dwWaitTimeout;

public:

	CEmEngine& m_stEngine;

	bool Init();
	void Uninit();

	CTaskProcessor(CEmEngine& stEngine) :
		m_hThread(NULL),
		m_hStop(NULL),
		m_pCurrent(NULL),
		m_stEngine(stEngine)
	{}
	~CTaskProcessor() { Uninit(); };

	inline void Push(CTask* pTask) 
	{
		if (pTask) 
			m_queueTasks.Push(pTask); 
		else
		{
			ASSERT(FALSE);
			AddDebugLogLine(false, _T("no memory"));
		}
	}
	inline bool IsInContext(const CTask& stTask) { return &stTask == m_pCurrent; }
};

#endif // !defined(AFX_TASKPROCESSOR_H__186018D8_26C9_4A91_9F7C_DDC406696088__INCLUDED_)
#endif