// TaskProcessor_Main.cpp: implementation of the CTaskProcessor_Main class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef NEW_SOCKETS

#include "TaskProcessor_Main.h"
#include "EmEngine.h"

//////////////////////////////////////////////////////////////////////
// starting
bool CTaskProcessor_Main::Start()
{
	m_mapClients.InitHashTable(997);
	m_dwWaitTimeout = 500;
	return true;
}

//////////////////////////////////////////////////////////////////////
// destroy all clients that are possibly left in our map
void CTaskProcessor_Main::Stop()
{
	for (POSITION pos = m_mapClients.GetStartPosition(); pos; )
	{
		CEmClient* pClient;
		SOCKET hSocket;
		m_mapClients.GetNextAssoc(pos, hSocket, pClient);
		ASSERT(pClient);
		delete pClient;
	}
	m_mapClients.RemoveAll();
}

//////////////////////////////////////////////////////////////////////
// processing timeout
// this should replace all old Process() functions
void CTaskProcessor_Main::ProcessTimeout()
{
	// keep connection alive once per minute
	// recommended by lugdunummaster
	if(m_stEngine.m_stData.IsConnected())
	{
		// sending empty OFFER FILES opcode
	}

	m_dwWaitTimeout = 1000; // perform check once per second
}

//////////////////////////////////////////////////////////////////////
// creating new client object
CEmClient* CTaskProcessor_Main::AllocClient(T_CLIENT_TYPE eType)
{
	switch (eType)
	{
	case T_CLIENT_PEER:
		return new CEmClient_Peer;
		break;
	case T_CLIENT_SERVER:
		return new CEmClient_Server;
		break;
	case T_CLIENT_WEB:
		return new CEmClient_Web;
		break;
	default:
		ASSERT(FALSE); // invalid type ???
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////
// connecting to any client (or server)
CEmClient* CTaskProcessor_Main::AllocTcpConnect(ULONG nAddr, USHORT nPort, T_CLIENT_TYPE eClientType)
{
	CEmClient* pClient = AllocClient(eClientType);
	if (pClient)
	{
		pClient->m_nAddr = nAddr;
		pClient->m_uPort = nPort;
		SOCKET hSocket = m_stEngine.m_stTcp.AllocConnect(nAddr, nPort, eClientType);
		if (INVALID_SOCKET == hSocket)
		{
			delete pClient;
			pClient = NULL;
		} else
		{
			pClient->m_hSocket = hSocket;
			pClient->m_bIsConnected = false;

			m_mapClients.SetAt(hSocket, pClient);
		}
	} 
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
	}

	return pClient;
}

//////////////////////////////////////////////////////////////////////
// disconnect existing client and destroy its object
void CTaskProcessor_Main::KillClient(CEmClient* pClient)
{
	ASSERT(pClient);
	SOCKET hSocket = pClient->m_hSocket;
	pClient->OnDisconnected(m_stEngine);
	delete pClient;
	m_mapClients.RemoveKey(hSocket);
}

#endif