// Tasks.cpp: implementation of the CTasks class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef NEW_SOCKETS

#include "Tasks.h"
#include "EmEngine.h"
#include "../emule.h"
#include "../EngineData.h"
#include "../server.h"
#include "TaskProcessor_Main.h"
#include "OpCode.h"

//////////////////////////////////////////////////////////////////////
// global and sockets task functions
CTask::~CTask()
{
}

bool CTask::OnException(CEmEngine&)
{
	// delete this buggy task
	return true;
}

bool CTask::ProcessForUI(CEmEngine&)
{
	return true;
}

bool CTask_Tcp_Err::Post(CEmEngine& stEngine, SOCKET hSocket, int nErrorCode)
{
	CTask_Tcp_Err* pTask = new CTask_Tcp_Err;
	if (pTask)
	{
		pTask->m_hSocket = hSocket;
		pTask->m_nError = nErrorCode;
		stEngine.m_stMain.Push(pTask);
	} 
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
	}

	return NULL != pTask;
}

bool CTask_Tcp_Err::Process(CEmEngine& stEngine)
{
	// check if we still have this client
	CEmClient* pClient = stEngine.m_stMain.Lookup(m_hSocket);
	if (pClient)
	{
		ASSERT(pClient->m_hSocket == m_hSocket);
		// here we can write to the log that this client has disconnected from us

		stEngine.m_stMain.KillClient(pClient);
	}

	return true;
}

bool CTask_Tcp_Accepted::Process(CEmEngine& stEngine)
{
	CEmClient* pClient = stEngine.m_stMain.AllocClient(m_eType);
	ASSERT(pClient);
	if (pClient)
	{
		pClient->m_hSocket = m_hSocket;
		pClient->m_bIsConnected = true;
		stEngine.m_stMain.m_mapClients.SetAt(m_hSocket, pClient);

		pClient->OnAccepted(stEngine);

	} else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
		if (closesocket(m_hSocket))
			AddDebugLogLine(false, "closesocket failed");
	}

	return true;
}

bool CTask_Tcp_Connected::Process(CEmEngine& stEngine)
{
	CEmClient* pClient = stEngine.m_stMain.Lookup(m_hSocket);
	if (pClient)
	{
		ASSERT(!pClient->m_bIsConnected);
		pClient->m_bIsConnected = true;

		pClient->OnConnected(stEngine);
	}
	return true;
}

bool CTask_Tcp_Web::Process(CEmEngine& stEngine)
{
	CEmClient_Web* pClient = (CEmClient_Web*) stEngine.m_stMain.Lookup(m_hSocket);
	if (pClient)
	{
		ASSERT(T_CLIENT_WEB == pClient->GetType());

		  CString strTxt;
		  strTxt.Format("Currently I have %d clients", stEngine.m_stMain.m_mapClients.GetCount());
		  stEngine.m_stTcp.AllocSend(m_hSocket, (LPCTSTR) strTxt, strTxt.GetLength());
		  stEngine.m_stTcp.AllocDisconnect(m_hSocket);
	}

	return true;
}
// end of global and socket task functions
//////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
// connect
CTask_Connect::CTask_Connect(ULONG nAddr, USHORT nPort, T_CLIENT_TYPE eType)
	: m_nAddr(nAddr), m_uPort(nPort), m_eType(eType)
{
}

//////////////////////////////////////////////////////////////////////
CTask_Connect::CTask_Connect(LPCTSTR sAddr, USHORT nPort, T_CLIENT_TYPE eType) 
	: m_nAddr(inet_addr(sAddr)), m_uPort(nPort), m_eType(eType)
{
}

//////////////////////////////////////////////////////////////////////
CTask_Connect::CTask_Connect(CServer* pServer /* = NULL*/)
{
	EMULE_TRY

	if(pServer)
	{
		AddLogLine(false, IDS_CONNECTINGTO, pServer->GetListName(), pServer->GetAddress(), pServer->GetPort());

		m_nAddr = inet_addr(pServer->GetAddress());
		m_uPort = pServer->GetPort();
	}
	else
		m_nAddr = m_uPort = NULL;

	m_eType = T_CLIENT_SERVER;

	EMULE_CATCH2
}

//////////////////////////////////////////////////////////////////////
bool CTask_Connect::Process(CEmEngine& stEngine)
{
	if(m_nAddr && m_uPort)
		stEngine.m_stMain.AllocTcpConnect(m_nAddr, m_uPort, m_eType);
	else
	{
		// Connect to any server
	}

	return true;
}

//////////////////////////////////////////////////////////////////////
// disconnect
CTask_KillClient::CTask_KillClient(CEmClient* pClient)
	: m_pClient(pClient)
{
}

//////////////////////////////////////////////////////////////////////
bool CTask_KillClient::Process(CEmEngine& stEngine)
{
	stEngine.m_stMain.KillClient(m_pClient);

	return true;
}

//////////////////////////////////////////////////////////////////////
// synchronize server state
CTask_ServerState::CTask_ServerState(bool bForConnected, ServerState stState, long nClientID)
{
	m_bForConnected = bForConnected;
	m_stState = stState;
	m_nClientID = nClientID;
}

//////////////////////////////////////////////////////////////////////
bool CTask_ServerState::Process(CEmEngine& stEngine)
{
	stEngine.PushToUI(this);

	return false;
}

//////////////////////////////////////////////////////////////////////
bool CTask_ServerState::ProcessForUI(CEmEngine&)
{
	if(g_eMuleApp.m_pdlgEmule->m_pEngineData)
		g_eMuleApp.m_pdlgEmule->m_pEngineData->ImportServerState(this);

	return true;
}

//////////////////////////////////////////////////////////////////////
// send list of shared files
CTask_SendSharedList::CTask_SendSharedList()
{
	m_bProcessedUI = false;
}

//////////////////////////////////////////////////////////////////////
bool CTask_SendSharedList::Process(CEmEngine& stEngine)
{
	if(!stEngine.m_stData.IsConnected())
		return true;

	// first collect shared files info
	if(!m_bProcessedUI)
	{
		stEngine.PushToUI(this);
		return false;
	}

	// afterwards, send it
	COpCode_OFFERFILES stMsg;
	for(int i = 0; i < m_Files.GetCount(); i++)
	{
		COpCode_OFFERFILES::CStruct_Files file;
		md4cpy(file._Hash, m_Files[i]._Hash);
		file._Client._Addr = 0;
		file._Client._Port = 0;
		file._FileName.m_strValue = m_Files[i]._FileName;
		file._FileName.m_bValid = true;
		file._FileSize.m_dwValue = m_Files[i]._FileSize;
		file._FileSize.m_bValid = true;
		stMsg._Files.Add(file);
	}
	stEngine.m_stTcp.AllocSend(stEngine.m_stData.GetConnectedServer(), stMsg);

	return true;
}

//////////////////////////////////////////////////////////////////////
bool CTask_SendSharedList::ProcessForUI(CEmEngine& stEngine)
{
	CKnownFile	*pKnownFile;
	CCKey		bufKey;
	for (POSITION pos = g_eMuleApp.m_pSharedFilesList->m_mapSharedFiles.GetStartPosition(); pos != 0; )
	{
		g_eMuleApp.m_pSharedFilesList->m_mapSharedFiles.GetNextAssoc(pos, bufKey, pKnownFile);
		if(pKnownFile)
		{
			OFFEREDFILE File;

			md4cpy(File._Hash, pKnownFile->GetFileHash());
			File._Addr = 0;
			File._Port = 0;
			File._FileName = pKnownFile->GetFileName();
			File._FileSize = pKnownFile->GetFileSize();

			m_Files.Add(File);

			pKnownFile->SetOfferedToServer(true);
		}
	}

	m_bProcessedUI = true;

	stEngine.m_stMain.Push(this);

	return false;
}


#endif